<?php

/**
 * @package Vikingboard
 *
 * Shows a forum or does forum-specific work
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.2
 *
 * $Id: forum.php 1859 2007-11-27 18:59:28Z jorgenboard $
 */

//yes, we are here
define('VBOARD', __FILE__);

// Require kernel
require_once('inc/kernel.php');

/*
 *	Vikingboard forum view class
 *
*/

class forum
{
	// Define variables
	var $data;
	var $user;
	var $screen;
	var $config;
	var $lang;

	/*
	 * Constructor
	 */
	function forum(&$data, &$user, &$screen, &$config)
	{
		//Set up some vars
		$this->data		=	&$data;
		$this->user		=	&$user;
		$this->screen	=	&$screen;
		$this->config	=	&$config;
		$this->lang		=	new lang_topics;
		$this->screen->set_lang('lang_topics');

		// Let`s find out what we want to do
		switch(ifsetor($_GET['mode'], false))
		{
			// Show a forum
			case 'f':
			$this->show_forum();
			break;

			// Add a forum subscription
			case 'subscribe':
			$this->add_forum_subscription();
			break;

			// Default is to show a forum
			default:
			$this->show_forum();
		}
	}

	/**
	 * Add forum subscription
	 *
	 * @param int topic
	 * @return void
	*/
	function add_forum_subscription()
	{
		$forum	=	safe_user_int('f', false);

		if ($forum)
		{
			$sub	=	$this->data->query('get_usersubscr_in_forum', array($this->user->id, $forum));
			$sub	=	$sub[0];

			if (!$sub->id)
			{
				$this->data->query('insert_new_usersub_forum', array($this->user->id, $forum));
				notify($this->lang->subscribe_header, $this->lang->new_forum_subscription);
			}
			else
			{
				/**
				 * @todo error_msg() is too harsh, and doesn't provide backlinks, and notify()
				 * 		really doesn't convey that something did not succeed, so the user will probably
				 * 		think the subscription was registered here. We need a middle ground, or
				 * 		a way to provide backlinks to error_msg()
				 */
				notify($this->lang->subscribe_header, $this->lang->existing_forum_subscription);
			}
		}
	}


	/**
	 * Show forum
	 *
	 * Show a specific forum
	 * @return void
	 */
	function show_forum()
	{
		// Do we even have a forum to view?
		$showforum	=	safe_user_int('f', false);

		if (!$showforum)
		{
			error_msg($this->lang->forum_missing, 1);
		}

		// Starting to show from topic number x..
		$showfrom	=	safe_user_int('s', 0);

		// Updating forum read cookie (yummy!)
		setcookie($this->config->cookie . "_forumread[{$showfrom}]", forumtime(), forumtime() + (86400 * 365), "/");

		// Getting the category id for this forum and the name for usage in the navigation menu
		$db_forum	=	$this->data->cachedQuery('get_forum_info', array($showforum));
		$db_forum	=	$db_forum[0];

		// Are we in the middle of nowhere?
		if (!$db_forum)
		{
			error_msg($this->lang->forum_missing, 1);
		}

		// The name of the category is?
		$db_cat	=	$this->data->cachedQuery('get_category', array($db_forum->cat));
		$db_cat	=	$db_cat[0];

		// Getting topics in this forum :)
		$db_topics	=	$this->data->query('get_topics_from_forum', array($showforum, $showfrom, $this->config->show_maxtopics, $this->config->prune_topics));

		// Build navigation link for the forum view
		require_once('inc/lib/pages.php');

		// Total page numbers
		$pages	=	ceil($db_forum->stat_threads / $this->config->show_maxtopics);

		// The current page
		$showfrom		=	($showfrom / $this->config->show_maxtopics) + 1;
		$pageNavigation	=	($pages > 1) ? $lang_topic->pages . pages($showfrom, $pages, "forum.php?f={$showforum}&s=", $this->config->show_maxtopics) : false;

		$i			=	0;
		$topiclist	=	'';

		// Loop through each topic and save information about it to user
		if ($db_topics)
		{
			foreach ($db_topics as $db_topics2)
			{
				$replies	=	$db_topics2->replies;
				$views		=	$db_topics2->views;

				// Alternate row coloring
				$i++;
				$id	=	$i%2 + 1;

				// Let's do the page navigation for each topic
				$pages		=	ceil(($replies + 1) / $this->config->show_maxposts);

				if($pages > 1)
				{
					$pages		=	pages(1, $pages, "topic.php?t={$db_topics2->id}&amp;start=", $this->config->show_maxposts);
					$pages		=	"{$lang_topic->pages}{$pages}";
				}
				else
				{
					$lastPageID	=	0;
					$pages		=	'';
				}

				// Figure out stickieness, normality and closedness
				$closed = false;
				$sticky = false;
				$ordinary = true;

				if ($db_topics2->status == 0)
				{
					$closed = true;
					$ordinary = false;
				}
				if ($db_topics2->sticky == 1)
				{
					$sticky = true;
					$ordinary = false;
				}

				if ($sticky && $closed)
				{
					$closed = false;
					$sticky = false;
					$ordinary = false;
					$closed_sticky = true;
				}
				else
				{
					$closed_sticky = false;
				}

				// Save variables to an array
				$tohtml	=	array
				(
					'pages'			=>	$pages,
					'rowID'			=>	$id,
					'topicID'		=>	$db_topics2->id,
					'topicName'		=>	$db_topics2->title,
					'author'		=>	$db_topics2->starter_name,
					'authorID'		=>	$db_topics2->starter_id,
					'replies'		=>	$replies,
					'lastAuthor'	=>	$db_topics2->lpost_name,
					'views'			=>	$views,
					'lastPostDate'	=>	date($this->config->timestamp1, forumtime($db_topics2->lpost_time)),
					'closed'		=>  $closed,
					'sticky'		=> 	$sticky,
					'closed_sticky'	=>  $closed_sticky,
					'ordinary'		=>  $ordinary
				);

				// Add to output
				$topiclist	.=	$this->screen->return_html('list_topic', $tohtml);
			}
		}
		else
		{
			$content = array('colspan' 		=> 5,
							 'empty_row' 	=> $this->lang->notopics);

			$topiclist = $this->screen->return_html('empty_row', $content);
		}

		// Build global template array
		$tohtml	=	array
		(
			//'lastPageID"	=>	$lastpage,
			// @todo 'user_can_poll' =>	$this->user->g_create_new_polls,
			'user_can_poll'	=>	false,
			'user_can_topic'=>	$this->user->g_create_new_posts,
			'navigation'	=>	$pageNavigation,
			'forumname'		=>	$db_forum->name,
			'forumID'		=>	$showforum,
			'topiclist'		=>	$topiclist,
			'user_id'		=>	$this->user->id,
			'user_subscribe'	=>	((isset($forum_subscriptions->fid)) ? true : false),
		);

		// Defines variables to use in "skin"
		$navigation	=	array
		(
			"index.php?c={$db_forum->cat}" => $db_cat->name,
			"forum.php?f={$db_forum->id}" => $db_forum->name,
		);

		// add rest of the page
		$this->screen->add_html('forum', $tohtml);

		// Wrap it! (umm, wraps, yummmy)
		$this->screen->add_wrapper($db_forum->name, $navigation);

		// Print page
		$this->screen->print_page();
	}
}

$f	=	new forum($data, $user, $screen, $config);

?>
