<?php

/**
CREATE TABLE `vboard_announcements` (
  `id` int(11) NOT NULL auto_increment,
  `title` varchar(255) collate utf8_unicode_ci NOT NULL,
  `message` text collate utf8_unicode_ci NOT NULL,
  `active` tinyint(1) NOT NULL,
  `start` int(11) NOT NULL,
  `end` int(11) NOT NULL,
  `importance` tinyint(1) NOT NULL,
  `groups` varchar(255) collate utf8_unicode_ci NOT NULL,
  PRIMARY KEY  (`id`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci AUTO_INCREMENT=1 ;
*/

/**
 * Announcements
 *
 * Administration of forum announcements
 * @package Vikingboard
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2.0
 * $Id: announcement.php 1399 2007-05-25 12:25:01Z paitrakt $
 */

// Oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}


/**
 * The general, standard admin class for handling admin-pages
 * and other stuff.
 */
class admin
{
	/**
	 * Class loaded, this is called by the main admin page script.
	 * This script handles the management of categories and forums
	 * 
	 * @param obj screen
	 * @param obj data
	 * @param obj config
	 */
    function admin($screen, $data, $config)
    {
		$this->screen	=	$screen;
		$this->data		=	$data;
		$this->config	=	$config;
		$this->lang		=	new lang_announcements();
		$this->screen->set_lang('lang_announcements');

		switch(true)
		{
			// Add announcement
			case isset($_GET['add']):
			$this->new_announcement();
			break;

			// Edit announcement
			case safe_user_int('edit', false):
			$this->edit_announcement();
			break;

			// Save announcement
			case isset($_GET['save']):
			$this->save_announcement();
			break;

			// Delete announcement
			case safe_user_int('delete', false):
			$this->delete_announcement();
			break;

			// View announcements
			default:
			$this->announcements();
			break;
		}

		return $this->html;
	}

	function new_announcement()
	{
		$groups	=	$this->data->query('get_groups');

		$data	=	array
		(
			'current_day'		=>	date('j'),
			'current_month'		=>	date('n'),
			'current_year'		=>	date('Y'),
			'current_hour'		=>	date('H'),
			'current_minute'	=>	date('i'),
			'groups'			=>	$groups,
		);

		$this->html	=	$this->screen->return_html('new_announcement', $data);
	}

	/**
	 * Add announcement
	 */
	function save_announcement()
	{
		$title		=	ifsetor($_POST['title'], false);
		$message	=	ifsetor($_POST['message'], false);
		$importance	=	safe_user_int('importance', false);
		$all_groups	=	ifsetor($_POST['all_groups'], false);
		$groups		=	ifsetor($_POST['groups'], false);

		if (!$title)
		{
			notify('Announcement error', 'You must enter a title for the announcement');
		}

		if (!$message)
		{
			notify('Announcement error', 'You must enter a message for the announcement');
		}

		$period_length	=	ifsetor($_POST['period_length'], false);

		if ($period_length == 'custom')
		{
			if (isset($_POST['start_now']))
			{
				$start	=	time();
			}
			else
			{
				$day_start		=	safe_user_int('day_start', date('j'));
				$month_start	=	safe_user_int('month_start', date('n'));
				$year_start		=	safe_user_int('year_start', date('Y'));
				$hour_start		=	safe_user_int('hour_start', date('H'));
				$minute_start	=	safe_user_int('minute_start', date('i'));

				$start	=	mktime($hour_start, $minute_start, 0, $month_start, $day_start, $year_start);

				if ($start < time())
				{
					$start	=	time();
					//notify('Announcement error', 'Start date cannot be in the past');
				}
			}

			$day_end	=	safe_user_int('day_end', false);
			$month_end	=	safe_user_int('month_end', false);
			$year_end	=	safe_user_int('year_end', false);
			$hour_end	=	safe_user_int('hour_end', false);
			$minute_end	=	safe_user_int('minute_end', false);

			if (!$day_end || !$month_end || !$year_end || !$hour_end || !$minute_end)
			{
				// Or notify('Must fill out end time')?
				$end	=	time() + 3600;
			}
			else
			{
				$end	=	mktime($hour_end, $minute_end, 0, $month_end, $day_end, $year_end);
			}

			if ($end < time())
			{
				notify('Announcement error', 'End date cannot be in the past');
			}
		}
		else
		{
			$num_periods	=	safe_user_int('num_periods', false);
			if ($num_periods)
			{
				$period_length	=	$period_length * $num_periods;

				$start	=	time();
				$end	=	time() + $period_length;
			}
			else
			{
				notify('Announcement error', 'The period length must be a valid integer.');
			}
		}

		if (!$importance)
		{
			notify('Announcement error', 'Importance is not valid');
		}

		if (!$all_groups && (!is_array($groups) || count($groups) == 0))
		{
			notify('Announcement error', 'At least one group must be selected.');
		}
		elseif ($all_groups)
		{
			$groups	=	$this->data->query('get_all_group_ids');
			foreach($groups as $group)
			{
				$g[]	=	$group->id;
			}
			$groups	=	$g;
		}

		$data	=	array
		(
			$title,
			$message,
			$start,
			$end,
			$importance,
			implode(';', $groups),
		);

		if ($_POST['add_or_edit'] == 'add')
		{
			$this->data->query('add_announcement', $data);
			notify('Announcement', 'Announcement added, congrats!', 'admin.php?act=announcements', 'Back to announcements');	
		}
		elseif ($_POST['add_or_edit'] == 'edit')
		{
			$data[] =	((isset($_POST['active'])) ? '1' : '0');
			$data[]	=	$_GET['edit'];
			$this->data->query('update_announcement', $data);
			notify('Announcement', 'Announcement updated, congrats!', 'admin.php?act=announcements', 'Back to announcements');	
		}
		else
		{
			notify('Stupid error', 'Stop messing with my HTML, you bitch', 'http://disney.com', 'Go somewhere else');
		}
	}

	/**
	 * Edit announcement
	 */
	function edit_announcement()
	{
		$id	=	$_GET['edit'];

		$announcement	=	$this->data->query('get_announcement', array($id));
		$announcement	=	$announcement[0];

		$start			=	$announcement->start;
		$day_start		=	date('j', forumtime($start));
		$month_start	=	date('n', forumtime($start));
		$year_start		=	date('Y', forumtime($start));
		$minute_start	=	date('i', forumtime($start));
		$hour_start		=	date('H', forumtime($start));

		$end			=	$announcement->end;
		$day_end		=	date('d', forumtime($end));
		$month_end		=	date('m', forumtime($end));
		$year_end		=	date('Y', forumtime($end));
		$minute_end		=	date('i', forumtime($end));
		$hour_end		=	date('H', forumtime($end));

		$data	=	array
		(
			'id'			=>	$id,
			'title'			=>	$announcement->title,
			'message'		=>	$announcement->message,
			'active'		=>	$announcement->active,
			'day_start'		=>	$day_start,
			'month_start'	=>	$month_start,
			'year_start'	=>	$year_start,
			'minute_start'	=>	$minute_start,
			'hour_start'	=>	$hour_start,
			'day_end'		=>	$day_start,
			'month_end'		=>	$month_end,
			'year_end'		=>	$year_end,
			'minute_end'	=>	$minute_end,
			'hour_end'		=>	$hour_end,
			'importance'	=>	$announcement->importance,
			'groups'		=>	explode(';', $announce->groups),
		);

		$this->html	=	$this->screen->return_html('edit_announcement', $data);
	}

	/**
	 * Delete announcement
	 */
	function delete_announcement()
	{
		$this->data->query('delete_announcement', array($_GET['delete']));
		notify('Announcement', 'Announcement deleted');
	}

	/**
	 * View announcements
	 */
	function announcements()
	{
		$start	=	safe_user_int('start', 0);
		$end	=	safe_user_int('end', 30);

		$current_announcements	=	$this->data->query('get_announcements', array(time(), $start, $end));
		$upcoming_announcements	=	array(array('id' => 1, 'title' => 'Test announcement', 'start' => 'Someday', 'end' => 'Whenever', 'importance' => 'Moderate (3)'));
		$finished_announcements	=	'';

		$data	=	array
		(
			'announcements'				=>	(($current_announcements || $upcoming_announcements || $finished_announcements) ? true : false),
			'current_announcements'		=>	(($current_announcements) ? $current_announcements : false),
			'upcoming_announcements'	=>	(($finished_announcements) ? $upcoming_announcements : false),
			'finished_announcements'	=>	(($finished_announcements) ? $finished_announcements : false),
		);

		$this->html	=	$this->screen->return_html('announcements', $data);
	}
}

$admin	=	new admin($screen, $data, $config);

?>