<?php
/**
 * BBCode admin page.
 *
 * Handles BBCodes; adding them, editing them and removing them from the system.
 *
 * @package Vikingboard
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.2
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 * @link http://vikingboard.com/
 * @version $Id: bbcode.php 1627 2007-08-21 19:56:25Z jorgenboard $
 *
 */

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}

/**
 * Default admin class
 *
 */
class admin
{
	/**
	 * Class loaded, this is called by the main admin page script.
	 *
	 * @param obj Screen
	 * @param obj Data
	 * @param obj Config
	 *
	 * @return string html
	 */
	function admin($screen, $data, $config)
	{
		//Set up variables that we use
		$this->screen	= $screen;
		$this->data		= $data;
		$this->config	= $config;

		$this->screen->set_lang('lang_admin_bbcode');

		//Find out what to do
		switch (true)
		{
			case isset($_GET['edit']):
				$this->edit_bbcode();
			break;

			case isset($_GET['add']):
				$this->add_bbcode();
			break;

			case isset($_GET['delete']):
				$this->delete_bbcode(safe_user_int('id', false));
			break;

			default:
				$this->show_bbcodes();
			break;
  		}

		//Return html page to the main admin page
		return $this->html;
	}

	/**
	 * Show a list of all BBCodes/parse rules
	 *
	 */
	function show_bbcodes()
	{
		// Get the C0D3S
		$bbcodes = $this->data->cachedQuery('get_bbcodes');

		$list = '';
		foreach ($bbcodes as $bbcode)
		{
			// Making both edit and replace "safe"
			$find 		= htmlentities($bbcode->find);
			$replace 	= htmlentities($bbcode->replace);

			// Wooh, syntax highlighting! 8-)
			$find 		= preg_replace('#:([a-z]+?):#is', '<span class="keyword">:' . "$1" . ':</span>', $find);
			$replace 	= preg_replace('#:([a-z]+?):#is', '<span class="keyword">:' . "$1" . ':</span>', $replace);

			$html = array(	'descr' 	=> $bbcode->descr,
							'find' 		=> $find,
							'replace' 	=> $replace,
							'id'		=> $bbcode->id);

			$list .= $this->screen->return_html('admin_bbcode_row', $html);
		}

		$html = array(	'list' 		=> $list,
						'saved' 	=> ifsetor($_SESSION['saved'], false),
						'deleted' 	=> ifsetor($_SESSION['deleted'], false),
						'inserted'	=> ifsetor($_SESSION['inserted'], false)
					);

		// Killing of unused session variables
		unset($_SESSION['saved'], $_SESSION['deleted'], $_SESSION['inserted']);

		$this->html = $this->screen->return_html('admin_bbcode', $html);
	}

	/**
	 * Edit a BBCode or parser rule
	 *
	 */
	function edit_bbcode()
	{
		// Get an ID or you can't get in here
		$id = safe_user_int('id', false);

		if (!$id)
		{
			redirect('admin.php?act=bbcode');
		}

		if (empty($_POST))
		{
			// Get info
			$bbcode = $this->data->cachedQuery('get_bbcode', array($id));
			$bbcode = $bbcode[0];

			// Then output to user
			$html = array(	'descr' 	=> $bbcode->descr,
							'id'		=> $bbcode->id,
							'find'		=> htmlentities($bbcode->find),
							'replace' 	=> htmlentities($bbcode->replace));

			$this->html = $this->screen->return_html('admin_bbcode_edit', $html);
		}
		else
		{
			// Stripping away nasties
			$find 		= html_entity_decode($_POST['find']);
			$replace 	= html_entity_decode($_POST['replace']);
			$descr		= strip_tags($_POST['descr']);

			// Put into db
			if ($this->data->query('save_bbcode', array($id, $descr, $find, $replace)))
			{
				// We are SAVED! Wuhu! Let's go back to whence we came.
				$_SESSION['saved'] = true;
				redirect('admin.php?act=bbcode');
			}
		}

	}

	/**
	 * Remove a BBCode/parse rule
	 *
	 * @param int $id
	 * @return true if success, false if error
	 */
	function delete_bbcode($id)
	{
		// No ID?
		if (!$id)
		{
			redirect('admin.php?act=bbcode');
			return false;
		}

		// We have an ID, let's remove the code
		if ($this->data->query('delete_bbcode', array($id)))
		{
			$_SESSION['deleted'] = true;
			redirect('admin.php?act=bbcode');
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Add a new BBCode
	 *
	 */
	function add_bbcode()
	{
		if (empty($_POST))
		{
			$this->html = $this->screen->return_html('admin_bbcode_add');
		}
		else
		{
			// Put it in there!
			$find 		= html_entity_decode($_POST['find']);
			$replace 	= html_entity_decode($_POST['replace']);
			$descr		= strip_tags($_POST['descr']);

			// Put into db
			if ($this->data->query('add_bbcode', array($descr, $find, $replace)))
			{
				// Wuhu! Let's go back to whence we came.
				$_SESSION['inserted'] = true;
				redirect('admin.php?act=bbcode');
			}
		}
	}


}

?>