<?php
/**
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2.0
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: config_site.php 1737 2007-10-19 18:32:05Z ueland $
 */

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}

class admin
{

	/**
	 * Class loaded, this is called by the main admin page script.
	 * This script handles the site(s) configuration
	 *
	 * @param obj screen
	 * @param obj data
	 * @param obj config
	 *
	 * @return string html
	 */
	function admin($screen, $data, $config)
	{
		//Set "everything"
		$this->lang 	= new lang_admin_site_config();
		$this->screen	= $screen;
		$this->data 	= $data;
		$this->config 	= $config;
		$this->screen->set_lang('lang_admin_site_config');

		//Lets do something
		switch(true)
		{
			//Save config
			case isset($_GET['save']):
			$this->save_config();
			break;

			case isset($_GET['remove']):
			$this->remove_site();
			break;

			case isset($_GET['create']):
			$this->new_site();
			break;

			//Show config
			default:
			$this->show_config();
		}

		//Return html page to the main admin page
		return $this->html;
	}

	/**
	 * Creates a new site, which is not activated as default
	 *
	 * @return none
	 */
	function new_site()
	{
		//Add the new site
		$name = $this->lang->new_host;
		$url  = $this->lang->url;
		$host = $this->lang->host;
		$site = $this->data->query('create_new_site', array($name, $url, $host, $this->config->seed));
		$site = $site[0]->id;

		//Reset the filecache
		$this->data->flush_cache();

		//Then redirect to the new site's configuration
		redirect("admin.php?act=config_site&site={$site}");
	}

	/**
	 *
	 * Deletes a site based on ID input
	 *
	 */
	function remove_site()
	{
		//Make sure we have a site ID
		$id = safe_user_int('id', false);
		if ($id)
		{
			//We cant delete the last site, or there would be no forum left
			$sites = $this->data->query('count_sites');
			if ($sites[0]->number == 1)
			{
				error_msg($this->lang->cant_delete_last_site, 1);
			}
			else
			{
				$deleted = $this->data->query('delete_site', array($id));

				if ($deleted)
				{
					//Site deleted sucessfully
					notify($this->lang->success, $this->lang->site_deleted, 'admin.php?act=config_site', $this->lang->back_to_config);
				}

				//Reset the filecache
				$this->data->flush_cache();
			}
		}
		else
		{
			error_msg($this->lang->cant_delete_nothing, 1);
		}
	}


	/**
	 * Saves changes of the configuration
	 *
	 * @return bool
	 */
	function save_config()
	{
		//Get vars from POST/GET
		$id				=	safe_user_int('site', false);
		$forum_name		=	ifsetor($_POST['forum_name'], false);
		$forum_url		=	ifsetor($_POST['forum_url'], false);
		$host_name		=	ifsetor($_POST['hostname'], false);
		$homepage		=	ifsetor($_POST['forum_home'], false);
		$forum_email	=	ifsetor($_POST['forum_email'], false);
		$timestamp_1	=	ifsetor($_POST['timestamp1'], false);
		$timestamp_2	=	ifsetor($_POST['timestamp2'], false);
		$default_tpl	=	safe_user_int('template', false);
		$active			=	safe_user_int('active', false);
		$main_cat		=	safe_user_int('category', false);
		$main_language	=	safe_user_int('language', false);
		$topics			=	safe_user_int('num_topics', false);
		$posts			=	safe_user_int('num_posts', false);
		$prune_time		=	safe_user_int('prune_time', false);

        //Unstrip some vars
        $forum_name =   htmlentities( strip_magic_quotes_gpc($forum_name) );

		if (!$id)
		{
			redirect('admin.php?act=config_site');
		}

		if (!$forum_name)
		{
			error_msg($this->lang->missing_forum_name, 'admin.php?act=config_site&site=' . $id);
		}

		if (!is_valid_url($forum_url))
		{
			error_msg($this->lang->forum_url_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		if (!$host_name)
		{
			error_msg($this->lang->missing_host_name, 'admin.php?act=config_site&site=' . $id);
		}

		if (!is_valid_url($homepage))
		{
			error_msg($this->lang->homepage_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		if (!is_valid_email_address($forum_email))
		{
			error_msg($this->lang->forum_email_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		if (!$timestamp_1 || !$timestamp_2)
		{
			error_msg($this->lang->timestamps_missing,'admin.php?act=config_site&site=' . $id);
		}

		if (!$default_tpl)
		{
			error_msg($this->lang->default_template_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		if (!$active)
		{
			error_msg($this->lang->active_missing, 'admin.php?act=config_site&site=' . $id);
		}

		if (!$main_cat)
		{
			error_msg($this->lang->main_cat_missing, 'admin.php?act=config_site&site=' . $id);
		}

		if (!$main_language)
		{
			error_msg($this->lang->main_language_missing, 'admin.php?act=config_site&site=' . $id);
		}

		if ($topics < 1)
		{
			error_msg($this->lang->topics_per_page_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		if ($posts < 1)
		{
			error_msg($this->lang->posts_per_page_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		if ($prune_time < 1)
		{
			error_msg($this->lang->prune_time_not_valid, 'admin.php?act=config_site&site=' . $id);
		}

		// Update database
		$this->data->query('update_site', array(
			$id,
			$forum_name,
			$forum_url,
			$forum_email,
			$homepage,
			$host_name,
			$default_tpl,
			$main_cat,
			$main_language,
			$active,
			$topics,
			$posts,
			$prune_time,
			$timestamp_1,
			$timestamp_2
		));

		// Reset cache
		$this->data->flush_cache();

		notify($this->lang->site_saved_header, $this->lang->site_saved, 'admin.php?act=config_site&site=' . $id, $forum_name);
	}


	/**
	 * Shows the configuration for site foo
	 *
	 * @return bool
	 */
	function show_config()
	{
		//Construct a list of the different sites/domain on this board
		$sites = $this->data->cachedQuery('get_sites');

		$element = '';
		$i = 0;
		//Create a sites drop down list
		foreach ($sites as $site)
		{
			$selected	=	false;
			//Get "this" site`s config
			if (ifsetor($_GET['site'], false) == $site->id)
			{
				$selected		=	true;
				$current_site	=	$site;
			}
			elseif (!isset($_GET['site']))
			{
				if ($i == 0)
				{
					$selected		=	true;
					$current_site	=	$site;
					++$i;
				}
			}

			//Set some variables used in drop-down
			$e['selected']	=	$selected;
			$e['value']		=	$site->id;
			$e['name']		=	$site->forum_name;
			$element		.=	$this->screen->return_html('adm_conf_global_drop_content', $e);
			unset($selected);
		}

		//General/Simple tags
		$html['id']			=	$current_site->id;
		$html['site']		=	$current_site->forum_name;
		$html['siteList']	=	$element;
		$html['forum_name']	=	$current_site->forum_name;
		$html['forum_url']	=	$current_site->forum_url;
		$html['hostname']	=	$current_site->site_host;
		$html['email']		=	$current_site->forum_email;
		$html['homepage']	=	$current_site->site_homepage;
		$html['time1']		=	$current_site->timestamp1;
		$html['time2']		=	$current_site->timestamp2;
		$html['num_topics']	=	$current_site->show_maxtopics;
		$html['num_posts']	=	$current_site->show_maxposts;
		$html['prune_time']	=	$current_site->prune_topics;

		//Drop down list containing default site template
		unset($element);
		$templates = $this->data->cachedQuery('get_template_sets');

		$element = '';
		foreach ($templates as $template)
		{
			if ($template->id == $current_site->def_tpl)
			{
				$selected = true;
			}
			$e['value']	=	$template->id;
			$e['name']	=	$template->name;
			$e['selected']= $selected;
			unset($selected);
			$element	.=	$this->screen->return_html('adm_conf_global_drop_content', $e);
		}

		$html['siteTemplate']	=	$element;
		unset($element);


		//Drop down to show if the site is activated or not
		if ($current_site->active == 1)
		{
			$e['selected']= true;
		}
		else
		{
			$e['selected']= false;
		}
		$e['value']	=	1;
		$e['name']	=	'Yes';
		$element	=	$this->screen->return_html('adm_conf_global_drop_content', $e);

		if ($current_site->active == 0)
		{
			$e['selected']= true;
		}
		else
		{
			$e['selected']= false;
		}
		$e['value']	=	1;
		$e['name']	=	'No';
		$element	.=	$this->screen->return_html('adm_conf_global_drop_content', $e);
		$html['siteActive']	=	$element;
		unset($element);


		//Drop down list containing default site forum category
		unset($element);
		$categories = $this->data->cachedQuery('get_forum_categories');

		$element = '';
		foreach ($categories as $c)
		{
			if ($template->id == $current_site->def_cat)
			{
				$selected = true;
			}
			$e['value']	=	$c->id;
			$e['name']	=	$c->name;
			$e['selected']= $selected;
			unset($selected);
			$element	.=	$this->screen->return_html('adm_conf_global_drop_content', $e);
		}

		$html['siteCategory']	=	$element;
		unset($element);


		//Drop down list containing default site language
		unset($element);
		$lang = $this->data->cachedQuery('get_languages');

		$element = '';
		foreach ($lang as $l)
		{
			$selected = '';
			if ($l->id == $current_site->def_lang)
			{
				$selected = true;
			}
			$e['value']		= $l->id;
			$e['name']		= $l->language;
			$e['selected']	= $selected;
			unset($selected);
			$element	.=	$this->screen->return_html('adm_conf_global_drop_content', $e);
		}

		$html['siteLanguage']	=	$element;
		unset($element);

		//Return finished HTML page
		$this->html		 =   $this->screen->return_html('adm_conf_site', $html);

	}
}
?>
