<?php
/**
 * Admin CP - Errors
 *
 * Displays the different system errors reported by Vikingboard, along with the
 * option to flush the log completely.
 *
 * @package Vikingboard
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.2.0
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 * @link http://vikingboard.com/
 * @version $Id: errors.php 1848 2007-11-24 00:30:54Z jorgenboard $
 * @todo There are some major enhancements that can be done when there is time;
 * 	- Simple tracking-based error-handling
 *  - Assigning errors priority based on number of occurrences and the initial
 * 	  severity of the error. One mysql query error (sign of successful SQL insertion
 * 	  exploit) will get priority over 25 minor notices (like a mismatched template
 * 	  variable). Could go well with colour-coding.
 *  - Removing all but the X highest-"ranked" errors, to reduce clutter and
 * 	  improve manageability
 *  - Removing a specific type of error or a specific error itself (perhaps
 * 	  as part of the tracking-based solution?)
 *
 */

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}

/**
 * Default loadable admin class
 *
 */
class admin
{

	/**
	 * Class loaded, this is called by the main admin page script.
	 *
	 * @param obj screen
	 * @param obj data
	 * @param obj config
	 *
	 * @return string Template data
	 */
	function admin($screen, $data, $config)
	{
		//Set up variables that we use
		$this->lang 	= new lang_admin_errors();
		$this->screen	= $screen;
		$this->data		= $data;
		$this->config	= $config;

		$this->screen->set_lang('lang_admin_errors');

		//Find out what to do
		switch (ifsetor($_POST['empty'], false))
		{
			case (1):
				$this->empty_log();
			break;

			default:
				$this->show_log();
			break;
		}

		//Return html page to the main admin page
		return $this->html;
	}

	/**
	 * Read error log
	 *
	 * @return array Error log file (one key/value per line)
	 */
	function read_error_log()
	{
		$fp = fopen('cache/temp/error.php', 'r');

		//strip away first line. Dirty hack, but hey, it works fine, and does not burden the while loop
		$void = fgets($fp, 4096);

		// Read the error file into an array
		$f = array();
		while ($file = fgetcsv($fp))
		{
			$f[] = $file;
		}

		// Report error in recovering error log (:s)
		if (empty($f))
		{
			return false;
		}

		// Reverse the array, to get the most recent entries at the top
		$f = array_reverse($f);

		return $f;
	}

	/**
	 * Empty error log
	 *
	 * Empties the log file entirely, cleaning it of all previous log file entries
	 *
	 * @return bool true if successful
	 */
	function empty_log()
	{
		// Open file, (over)write first line and close it
		$fp 	= fopen('cache/temp/error.php', 'w');
		$write 	= fwrite($fp, "<?php die(); ?>\n");
		$close	= fclose($fp);

		if ($write && $close)
		{
			redirect('admin.php?act=errors');
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Show error log
	 *
	 */
	function show_log()
	{
		/**
		 * STOP!
		 *
		 * Let's not do this like a simple (and uninformative) list of errors, but instead group them
		 * in relevant groups. Icons separate different croups (security, PHP, MySQL etc).
		 * Obvious duplicates/triplicates/++ are grouped as one "item", and the colour of the
		 * "item"s box is a sign of the relevance. The number of times this event has occurred
		 * since a given timestamp is also counted and contributes to the total relevance of the event.
		 *
		 * Eg, a network connection error (mysql server down) gets a medium priority, and
		 * gets placed below the Poison NULL attack attempt and the query error, but when the
		 * network connection error has occured over 20 times (or some other set value),
		 * it will get a higher relevance than the other. At a set threshold, the relevance is
		 * high enough to justify waking the admin up (like sending email notices,
		 * or contacting the admin through SMS or similar systems).
		 *
		 * @todo This is for 0.3/0.4, there is not enough time to make this a nice, sorted
		 * 		 event-oriented interface for 0.2. Let's instead do a rewrite in 0.3/0.4, and
		 * 		 just have a somewhat working version for 0.2. Sad, but time keeps going...
		 *
		 *
		 */
		// Get the log
		$log = $this->read_error_log();

		if ($log)
		{
			$error_row = '';

			foreach ($log as $row)
			{
				$time = date($this->config->timestamp1, forumtime($row[6]));

				$errcodes = array(	E_ERROR				=> 'Error', 						// 1
									E_WARNING 			=> 'Warning', 						// 2
									E_PARSE				=> 'Parse Error',					// 4
									E_NOTICE			=> 'Notice',						// 8
									E_CORE_ERROR		=> 'Core Error',					// 16
									E_CORE_WARNING		=> 'Core Warning',					// 32
									E_COMPILE_ERROR		=> 'Compile Error',					// 64
									E_COMPILE_WARNING	=> 'Compile Warning',				// 128
									E_USER_ERROR		=> 'User Error (VB-generated)',		// 256
									E_USER_WARNING		=> 'User Warning (VB-generated)',	// 512
									E_USER_NOTICE		=> 'User Notice  (VB-generated)',	// 1024
									E_ALL				=> 'All errors',					// 6143 (PHP 5.2.x, previously 2047)
									E_STRICT			=> 'Runtime Notice',				// 2048
									E_RECOVERABLE_ERROR	=> 'Catchable Fatal Error');		// 4096

				$data = array
				(
					'type' 			=> $row[0],
					'error' 		=> $row[1],
					'filename'		=> $row[2],
					'linenum'		=> $row[3],
					'called_from'	=> $row[4],
					'query'			=> $row[5],
					'time'			=> $time,
					'error_code'	=> $errcodes[$row[7]],
					'backtrace'		=> $row[8]

				);

				$error_row .= $this->screen->return_html('admin_errors_row', $data);
			}
		}
		else
		{
			$html = array(	'colspan' 	=> 5,
							'empty_row' => $this->lang->no_errors);

			$error_row = $this->screen->return_html('empty_row', $html);
		}

		$this->html = $this->screen->return_html('admin_errors', array('error_rows' => $error_row));

	}

}

?>
