<?php

/**
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Robin Aaberg
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.2
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: user.php 1611 2007-08-18 16:55:20Z paitrakt $
 */

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}

class admin
{

	/**
	 * Class loaded, this is called by the main admin page script.
	 * User management in Admin CP
	 *
	 * @param obj screen
	 * @param obj cache
	 * @param obj db
	 * @param obj config
	 */
	function admin($screen, $data,  $config)
	{
		global $user;

		// Setting a few variables
		$this->screen	=	$screen;
		$this->data		=	$data;
		$this->config	=	$config;
		$this->user		=	$user;
		$this->lang		=	new lang_admin_user();
		$this->screen->set_lang('lang_admin_user');

		//Determinating what to do
		switch ($_GET['cmd'])
		{
			case 'add':
			$this->add_user();
			break;

			case 'add_data':
			$this->add_user_data();
			break;

			case 'edit':
			$this->edit_user();
			break;

			case 'edit_data':
			$this->edit_user_data();
			break;

			case 'delete':
			$this->delete_user();
			break;

			case 'manage_ban':
			$this->manage_ban();
			break;

			case 'ban':
			$this->ban_user();
			break;

			default:
			/**
			 * @todo The default should be a memberlist-clone with buttons per user
			 * for edit, delete and ban, as well as buttons for going to add user mode
			 * or other modes. TODO for 0.3
			 */
			$this->add_user();
		}
	}


	/**
	 * Add user
	 * Displays the form that let's an admin add a user
	 *
	 * @return void
	 *
	 */
	function add_user()
	{

		// Get all user groups and display in the dropdown
		$groups			=	$this->data->cachedQuery('get_groups');
		$options		=	'';

		foreach ($groups as $group)
		{
			// Is this group the default one?
			if ($group->id == $this->config->reg_group)
			{
				$selected	=	'selected="selected"';
			}
			else
			{
				$selected	=	'';
			}

			$option	=	array(
				'value'		=>	$group->id,
				'selected'	=>	$selected,
				'name'		=>	$group->name,
			);

			$options	.=	$this->screen->return_html('dropdown_option', $option);
		}


		//construct the rest of the form
		$html	=	array(
			'options'		=>	$options
		);

		//have we already added a user?
		if (isset($_GET['u']))
		{
			$html['notice'] = str_replace('%user%', htmlspecialchars($_GET['u']), $this->lang->add_added);
		}

		//output the form
		$this->html	=	$this->screen->return_html('admin-tpl_usermanage_add', $html);
	}


	/**
	 * Add user
	 * Adds the new user to the database
	 *
	 * @return void
	 *
	 */
	function add_user_data()
	{
		//input
		$username	=	$_POST['name'];
		$password	=	$_POST['password'];
		$cpassword	=	$_POST['cpassword'];
		$email		=	$_POST['email'];
		$group		=	safe_user_int('group', false);

		//check for various invalid inputs
		if (!$username)
		{
			error_msg($this->lang->add_error_username, 1);
		}

		if ($password != $cpassword)
		{
			error_msg($this->lang->add_error_pwnomatch, 1);
		}

		if (!is_valid_email_address($email))
		{
			error_msg($this->lang->add_error_nonvalidemail, 1);
		}

		//Do we allow multible registrations from the same mail adress?
		if (!$this->config->multi_reg)
		{
			$multimail	=	$this->data->cachedQuery('email_in_use', array($email));

			if ($multimail[0])
			{
				error_msg($this->lang->add_error_email_in_use, 1);
			}
		}

		//check the minimum length for the password
		if (strlen($password) < $this->config->password_length)
		{
			$error = str_replace('%chars%', $this->config->password_length, $this->lang->add_error_pw_too_short);

			error_msg($error, 1);
		}

		//check for the minimum password complexity
		if (password_strength($password) < $this->config->password_strength)
		{
			error_msg($this->lang->add_error_pw_too_simple, 1);
		}

		//check for minimum username length
		if (strlen($username) < $this->config->username_length)
		{
			$error = str_replace('%chars%', $this->config->username_length, $this->lang->add_error_usr_too_short);

			error_msg($error, 1);
		}

		// Checking the username to make sure we don't get any clones
		$usr_numcheck	=	$this->data->cachedQuery('username_in_use', array($username));
		$usr_numcheck   =   $usr_numcheck[0];

		if ($usr_numcheck->id)
		{
			error_msg($this->lang->add_error_usr_in_use, 1);
		}

		//check for "wrong" input
		$username	=	htmlentities($username);
		$username	=	strip_tags($username);

		// Update stats (we love stats!)
		$this->data->query('update_global_stats', array('users'));

		// Adding the user..
		$this->data->query('insert_new_user_data', array($username, encrypt($password), $email, $group));

		// Adding user to member_extra
		$new_id	=	$this->data->query('get_latest_user');
		$new_id	=	$new_id[0]->id;

		$this->data->query('insert_extra_user_data', array($new_id));

		//And returns to the form again
		redirect('admin.php?act=user&cmd=add&u=' . $username);
	}


	/**
	 * Edit user
	 * Displays the edit user form
	 *
	 * @todo It should be possible to search for a user, based on parts of the username
	 * @return void
	 *
	 */
	function edit_user()
	{
		//display the form/search
		$user_id	=	ifsetor($_POST['user'], false);
		if (!$user_id)
		{
			$this->html	=	$this->screen->return_html('admin_edit_user_frontpage');
		}
		else
		{
			//get user info
			$user	=	$this->data->query('select_user_from_name', array($user_id));
			$user	=	ifsetor($user[0], false);

			//is there no user to match?
			if (!$user)
			{
				error_msg($this->lang->edit_error_user_no_exists);
			}

			$info	=	str_replace('%id%', $user->id, $this->lang->edit_user_info);
			$info	=	str_replace('%name%', $user->name, $info);

			$groups		=	'';
			$all_groups	=	$this->data->query('get_groups');
			foreach($all_groups as $group)
			{
				$data	=	array
				(
					'value'	=>	$group->id,
					'name'	=>	$group->name,
				);
				if ($group->id == $user->group)
				{
					$data['selected']	=	'selected="selected"';
				}
				$groups	.=	$this->screen->return_html('dropdown_option', $data);
			}

			$languages		=	'';
			$all_languages	=	$this->data->query('get_languages');
			foreach($all_languages as $language)
			{
				$data	=	array
				(
					'value'	=>	$language->id,
					'name'	=>	$language->language,
				);
				if ($language->id == $user->lang)
				{
					$data['selected']	=	'selected="selected"';
				}
				$languages	.=	$this->screen->return_html('dropdown_option', $data);
			}

			$templates		=	'';
			$all_templates	=	$this->data->query('get_template_sets');
			foreach($all_templates as $template)
			{
				$data	=	array
				(
					'value'	=>	$template->id,
					'name'	=>	$template->name,
				);
				if ($template->id == $user->skin_set)
				{
					$data['selected']	=	'selected="selected"';
				}
				$templates	.=	$this->screen->return_html('dropdown_option', $data);
			}

			if ($user->signature)
			{
				require_once('inc/lib/parser.php');
				$Parser		=	new Parser($this->data, $this->screen, $this->config, 1);
				$signature	=	$Parser->deparse($user->signature);
			}

			//display edit form
			$data	=	array
			(
				'id'			=>	$user->id,
				'info'			=>	$info,
				'name'			=>	$user->name,
				'email'			=>	$user->email,
				'title'			=>	$user->title,
				'groups'		=>	$groups,
				'languages'		=>	$languages,
				'templates'		=>	$templates,
				'avatar'		=>	($user->avatar) ? $user->avatar : false,
				'location'		=>	$user->location,
				'homepage'		=>	$user->homepage,
				'real_name'		=>	$user->real_name,
				'birthday'		=>	$user->birthday,
				'im_network'	=>	$user->im_networks,
				'interests'		=>	$user->interests,
				'signature'		=>	$signature,
				'use_smileys'	=>	$user->use_smiley,
				'use_signature'	=>	$user->use_sig,
			);

			$this->html = $this->screen->return_html('admin_user_edit', $data);

		}
	}


	/**
	 * The actual user editing is done here, by order of the edit_user() function
	 *
	 * @return unknown
	 *
	 */
	function edit_user_data()
	{
		$id = safe_user_int('id', false);

		if (!$id)
		{
			error_msg($this->lang->edit_error_user_no_exists);
		}

		$name					=	ifsetor($_POST['name'], false);
		$email					=	ifsetor($_POST['email'], false);
		$title					=	ifsetor($_POST['title'], false);
		$group					=	ifsetor($_POST['group'], false);
		$new_password			=	ifsetor($_POST['new_password'], false);
		$new_password_confirm	=	ifsetor($_POST['new_password_confirm'], false);
		$language				=	ifsetor($_POST['language'], false);
		$template				=	ifsetor($_POST['template'], false);
		$avatar					=	ifsetor($_POST['avatar'], false);
		$location				=	ifsetor($_POST['location'], false);
		$homepage				=	ifsetor($_POST['homepage'], false);
		$real_name				=	ifsetor($_POST['real_name'], false);
		$birthday				=	ifsetor($_POST['birthday'], false);
		$im_network				=	ifsetor($_POST['im_network'], false);
		$interests				=	ifsetor($_POST['interests'], false);
		$signature				=	ifsetor($_POST['signature'], false);
		$use_smileys			=	ifsetor($_POST['use_smileys'], false);
		$use_signature			=	ifsetor($_POST['use_signature'], false);

		$user	=	$this->data->cachedQuery('select_user_from_id', array($id));
		$user	=	ifsetor($user[0], false);

		// Check if we have the needed vars
		if (!$name)
		{
			error_msg($this->lang->no_name);
		}

		if (!$email)
		{
			error_msg($this->lang->no_email);
		}
		// Oh yes, but is it valid?
		elseif (!is_valid_email_address($email))
		{
			error_msg($this->lang->email_not_valid);
		}

		if (!$title)
		{
			error_msg($this->lang->no_title);
		}

		if (!$group)
		{
			error_msg($this->lang->no_group);
		}

		if (($new_password && !$new_password_confirm) || (!$new_password && $new_password_confirm))
		{
			error_msg($this->lang->missing_one_password_field);
		}
		elseif ($new_password && $new_password_confirm)
		{
			if ($new_password != $new_password_confirm)
			{
				error_msg($this->lang->passwords_not_matching);
			}
			elseif (strlen($new_password) < $this->config->password_length)
			{
				error_msg($this->lang->password_not_long_enough);
			}
			elseif (password_strength($new_password) < $this->config->password_strength)
			{
				error_msg($this->lang->password_not_strong_enough);
			}
		}

		if (!$language)
		{
			error_msg($this->lang->no_language);
		}

		if (!$template)
		{
			error_msg($this->lang->no_template);
		}

		if ($avatar && !is_valid_url($avatar))
		{
			error_msg($this->lang->avatar_not_valid);
		}

		if ($homepage && !is_valid_url($homepage))
		{
			error_msg($this->lang->homepage_not_valid);
		}

		// Do we allow multiply email ?
		if (!$this->config->multi_reg && ($user->email != $email))
		{
			$multimail	=	$this->data->cachedQuery('get_num_emails', array($email));

			if (!empty($multimail[0]->id))
			{
				error_msg(str_replace('%s', $email, $this->lang->add_error_multimail));
			}
		}

		// No multiply name entries in the db thankyou.
		if ($user->name != $name)
		{
			$usr_numcheck	=	$this->data->cachedQuery('get_username_from_name', array($name));
			$usr_numcheck	=	$usr_numcheck[0];

			if ($usr_numcheck->id)
			{
				error_msg(str_replace('%s', $_POST['name'], $this->lang->add_error_nameistaken), 1);
			}
		}

		// Is there a signature we need to parse?
		if ($signature)
		{
			require_once('inc/lib/parser.php');
			$Parser		=	new Parser($this->data, $this->screen, $this->config, 1);
			$signature	=	$Parser->parse($signature);
		}

		// And add the stuff to our nice little DB
		$new_user_info	=	array
		(
			$id,
			$name,
			$email,
			$title,
			$group,
			$language,
			$template,
			$avatar,
			$location,
			$homepage,
			$real_name,
			$birthday,
			$im_network,
			$interests,
			$signature,
			$use_smileys,
			$use_signature,
		);

		$this->data->query('update_user', $new_user_info);

		// Update password?
		if ($new_password)
		{
			$new_password	=	encrypt($new_password);
			$this->data->query('update_user_password', array($id, $new_password));
		}

		//returning the the edit thingy
		redirect('admin.php?act=user&cmd=edit');
	}


	/**
	 * Delete user
	 * Let an admin delete a user
	 *
	 * @todo Search instead of precise input
	 *
	 * Both for this and edit user, it could be possible to make ONE search for user-form (with search backend),
	 * and just replace some labels to make it "custom". 0.3-material, I think.
	 *
	 */
	function delete_user()
	{
		// Do we have a user to delete ? If not, print html
		if ($_POST['name'] || $_POST['id'] || $_GET['id'])
		{

			$id 	= safe_user_int('id', false);
			$name	= htmlspecialchars($_POST['name']);

			//if we don't know the ID, let's find it
			if (empty($id) && !empty($name))
			{
				$id = $this->data->cachedQuery('get_id_from_name', array($name));
				$id = $id[0]->id;
			}

			//and let's get the name, for good measure
			if (empty($name) && !empty($id))
			{
				$name = $this->data->cachedQuery('select_user_from_id', array($id));
				$name = $name[0]->name;
			}

			// Just so some dumbass won't delete himself:
			if ($id == $this->user->id)
			{
				error_msg($this->lang->drop_error_selfdrop, 1);
			}

			//do the dropping
			$this->data->query('drop_user_from_id', array($id));

			// Did we come from the ban thingie?
			if ($id == $_GET['id'])
			{
				redirect('admin.php?act=user&cmd=ban');
			}

			//return, and notify that we did it
			redirect('admin.php?act=user&cmd=delete&notice=' . str_replace('%s', $name, $this->lang->drop_deleted_name));
		}

		//output the form and stop execution
		$html	=	array(
				'notice'		=>	$_GET['notice']
		);

		$this->html	=	$this->screen->return_html('admin-tpl_usermanage_drop', $html);
	}

	/**
	 * Ban user
	 *
	 * Let an admin ban a user
	 *
	 */
	function ban_user()
	{
		if (isset($_GET['unban']))
		{
			$id		=	safe_user_int('unban', 0);

			// Does the user exist?
			$user	=	$this->data->cachedQuery('select_user_from_id', array($id));
			$user	=	$user[0];

			// Do you exist?
			if (empty($user->id) || $id == 0 || $id != $user->id)
			{
				error_msg($this->lang->ban_error_unban_nouser);
				die();
			}

			// If so, welcome back!.
			$this->data->query('unban', array($id));
		}


		// Decided which user to ban yet ? if not, html.
		if (!isset($_POST['id']) && !isset($_POST['name']))
		{
			// Getting the ban list
			$banned_users	=	$this->data->cachedQuery('get_banlist');

			$ban_list = '';

			if ($banned_users)
			{
				foreach ($banned_users as $banned_user)
				{
					$list_fills	=	array(
						'id'	=>	$banned_user->id,
						'name'	=>	$banned_user->name,
					);

					$ban_list	.=	$this->screen->return_html('admin-tpl_user_ban_list', $list_fills);
				}
			}
			else
			{
				$ban_list	.=	$this->screen->return_html('empty_row', array('colspan' => '4', 'empty_row' => $this->lang->ban_no_banned));
			}

			$html['ban_list'] 	= $ban_list;
			$html['notice']		= false;

			if (isset($_GET['notice']))
			{
				$html['notice'] = htmlspecialchars($_GET['notice']);
			}

			// Printing the screen hallelujah.
			$this->html	=	$this->screen->return_html('admin-tpl_user_ban', $html);
		}
		else
		{
			$user = '';

			// Finding the user to ban
			$id = safe_user_int('id', 0);

			//by id
			if ($id)
			{
				$user	=	$this->data->cachedQuery('select_user_from_id', array($id));
			}
			//or name
			elseif (!empty($_POST['name']))
			{
				$user	=	$this->data->query('select_user_from_name', array($_POST['name']));
			}

			$user = $user[0];

			// Banning a nonexisting user is not legal!
			if (empty($user->id))
			{
				error_msg($this->lang->ban_error_nouser, 1);
			}

			// Haha, dumbass! Trying to ban yourself are yah?
			if ($user->id == $this->user->id)
			{
				error_msg($this->lang->ban_error_selfban, 1);
			}

			// Banning the user
			$this->data->query('ban_user', array($user->id));

			// He's been banned ;)
			redirect('admin.php?act=user&cmd=ban');
		}

	}

	/**
	 * The following is more modularized and more feature-ish 0.2-stuff, I presume?
	 *
	 */

	function manage_ban()
	{
		$userid	=	$_POST['id'];
		$action	=	$_POST['action'];

		switch($action)
		{
			case 'add':
			$this->add_user_ban($userid, $bantime, $reason);
			break;

			case 'remove':
			$this->remove_user_ban($userid);
			break;
		}
	}


	/**
	 * Add user ban
	 *
	 * @param int userid
	 * @param int bantime
	 * @param string reason
	 */
	function add_user_ban($userid, $bantime, $reason)
	{
		if ($userid != $this->user->id)
		{
			$this->db->add_user_ban($userid);
			return true;
		}
		else
		{
			return false;
		}
	}


	/**
	 * Remove user ban
	 *
	 * Removes ban from a user
	 * @param int userid
	 * @return bool
	 */
	function remove_user_ban($userid)
	{
		if ($userid != $this->user->id)
		{
			$this->db->remove_user_ban($userid);
			return true;
		}
		else
		{
			return false;
		}
	}
}

?>
