<?php

/**
 * MySQL file for topic.php
 *
 * @package Vikingboard
 * @version 0.2.0
 * $Id: sql_MySQL_mod.php 1813 2007-11-19 21:12:56Z ueland $
 */

class sql extends SQLGlobal
{
	/**
	 * SQL class constructor
	 *
	 */
	function sql($prefix)
	{

		$this->prefix	=	$prefix;
	}

	// Getting information around a post, topic name etc
	function get_post_information($in)
	{
		return "SELECT p.author_id, t.id, t.title, t.forum, t.cat, p.tid
				FROM {$this->prefix}post p, {$this->prefix}topic t
				WHERE t.id=p.tid
					AND p.id={$in[0]}
				LIMIT 1";
	}

	// Gets the very first post in topic
	function first_post_in_topic($in)
	{
		return "SELECT `id`
				FROM `{$this->prefix}post`
				WHERE `tid` = {$in[0]}
				ORDER BY `id` ASC
				LIMIT 1";
	}

	/**
	 * Delete a topic
	 *
	 * @param int $id
	 */
	function delete_topic($in)
	{
		return "DELETE FROM `{$this->prefix}topic`
				WHERE `id` = {$in[0]}
				LIMIT 1";
	}

	/**
	 * Reduce the topic count in a forum by one
	 * when deleting a topic
	 *
	 * @param int $forum_id
	 */
	function reduce_forum_topic_count($in)
	{
		return "UPDATE {$this->prefix}forum
				SET stat_threads = stat_threads - 1
				WHERE id={$in[0]}
				LIMIT 1";
	}

	// Counts all posts in a topic
	function count_posts_in_topic($in)
	{
		return "SELECT COUNT(*) AS total
				FROM `{$this->prefix}post`
				WHERE `tid` = '{$in[0]}'";
	}

	/**
	 * Delete all posts in a topic
	 *
	 * @param int $topic_id
	 */
	function delete_posts_in_topic($in)
	{
		return "DELETE FROM `{$this->prefix}post`
				WHERE `tid` = '{$in[0]}'
				LIMIT 0";
	}

	/**
	 * Reduce the forum post-count when
	 * deleting a topic
	 *
	 * @param int $num_posts
	 * @param int $forum_id
	 */
	function reduce_forum_post_count_dyn($in)
	{
		return "UPDATE {$this->prefix}forum
				SET `stat_posts` = ( `stat_posts` - {$in[0]})
				WHERE id = '{$in[1]}'
				LIMIT 1";
	}

	/**
	 * Gets the last post in a topic
	 *
	 * @param int $topic
	 */
	function get_last_post($in)
	{
		return "SELECT *
				FROM {$this->prefix}post p, {$this->prefix}topic t
				WHERE p.tid = '{$in[0]}'
					AND t.id = p.tid
				ORDER BY p.id DESC
				LIMIT 1";
	}

	/**
	 * Get last topic from a forum
	 *
	 * @param int $forum_id
	 */
	function get_last_topic($in)
	{
		return "SELECT * FROM `{$this->prefix}topic` WHERE `forum` = '{$in[0]}' ORDER BY `lpost_time` DESC LIMIT 1";
	}

	/**
	 * Updates the lastpost for a forum, on eg. topic deletion
	 *
	 * @param int $forum
	 * @param string $title
	 * @param int $tid
	 * @param int $pid
	 * @param int $uid
	 * @param int $time
	 * @param string $name
	 */
	function update_last_topic($in)
	{
		return "UPDATE `{$this->prefix}forum`
				SET lpost_title = '{$in[1]}', lpost_tid = {$in[2]}, lpost_pid = {$in[3]}, 
					lpost_uid = {$in[4]}, lpost_time = {$in[5]}, lpost_name = '{$in[6]}'
				WHERE id = {$in[0]}";
	}

	/**
	 * Delete a post
	 *
	 * @param int $id
	 */
	function delete_post($in)
	{
		return "DELETE FROM `{$this->prefix}post`
				WHERE `id` = '{$in[0]}' LIMIT 1";
	}

	/**
	 * REduce the reply count in a topic
	 * when deleting a post
	 *
	 * @param int $topic_id
	 */
	function reduce_topic_reply_count($in)
	{
		return "UPDATE `{$this->prefix}topic`
				SET `replies` = `replies` - '1'
				WHERE `id` = '{$in[0]}'	LIMIT 1";
	}

	/**
	 * Reduce the post count in a forum
	 * when deleting a post
	 *
	 * @param int $forum_id
	 */
	function reduce_forum_post_count($in)
	{
		return "UPDATE `{$this->prefix}forum`
				SET `stat_posts` = `stat_posts` - 1
				WHERE `id` = '{$in[0]}'	LIMIT 1";
	}

	/**
	 * Update lastpost-stats for a topic
	 *
	 * @param string $author
	 * @param int $id
	 * @param int $uid
	 * @param int $time
	 * @param int $topic
	 */
	function update_topic_last_post($in)
	{
		return "UPDATE `{$this->prefix}topic`
				SET `lpost_name` = '{$in[0]}', `lpost_id` = '{$in[1]}',
					`lpost_uid` = '{$in[2]}',`lpost_time` = '{$in[3]}'
				WHERE `id` = '{$in[4]}'";
	}

	/**
	 * Update lastpost-stats for a forum
	 *
	 * @param int $id
	 * @param int $uid
	 * @param int $time
	 * @param string $author
	 * @param int $forum
	 */
	function update_forum_last_post($in)
	{
		return "UPDATE `{$this->prefix}forum`
				SET `lpost_pid` = '{$in[0]}', `lpost_uid` = '{$in[1]}',
					`lpost_time` = '{$in[2]}', `lpost_name` = '{$in[3]}'
				WHERE `id` = '{$in[4]}'";
	}

	/**
	 * Change topic status
	 *
	 * 0 = closed
	 * 1 = open
	 *
	 * Other statuscodes may be added later
	 *
	 * @param int $tid
	 * @param int $status
	 */
	function change_topic_status($in)
	{
		return "UPDATE `{$this->prefix}topic` SET `status` = '{$in[1]}'	WHERE `id` = '{$in[0]}'	LIMIT 1";
	}

	/**
	 * Change a topic's sticky status (on or off)
	 *
	 * @param int $tid
	 * @param int $sticky
	 */
	function change_topic_sticky($in)
	{
		return "UPDATE `{$this->prefix}topic` SET `sticky` = '{$in[1]}' WHERE `id` = '{$in[0]}' LIMIT 1";
	}
}

?>
