<?php

/**
 * Data class
 *
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @version 0.2.0
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: data.php 1885 2007-12-03 00:16:43Z jorgenboard $
 *
 */

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}


class Data
{
	// Declare the class variables
	var $config;
	var $debug			=	0;
	var $hostName		=	'undefined';
	var $connected		=	false;
	var $queries;
	var $queryCount 	=	0;
	var $cacheCount 	=	0;
	var $sqlTime		=	0;
	var $cacheTime		=	0;
	var $debugData		=	'';
	var $cacheRoot		=	'./cache/';
	var $queryCacheDir;

	/**
	 * Data class constructor
	 *
	 * @param $conf Vikingboard configuration
	 */
	function Data($conf)
	{
		//Set up config
		$this->config = $conf;

		//Are we debugging?
		if (isset($_GET['debug']) && $_GET['debug'] == 1)
		{
			$this->debug = 1;
		}

		//Turn off caching when in development
		if (IN_DEVELOPMENT == true)
		{
			$this->config->cache_active = 0;
		}

		//Set up hostname
		$this->hostName = str_replace(':', '', $_SERVER['HTTP_HOST']);

		//Set up cached query path
		$this->queryCacheDir	=	$this->cacheRoot.$this->hostName."/";

		if (is_dir($this->queryCacheDir) === false)
		{
			mkdir($this->queryCacheDir, 0777);
			chmod($this->queryCacheDir, 0777);
		}

		// Done
		return true;
	}


	/**
	 * Performs a cached query
	 * This function first tries to locate a cached version of a query
	 * if that fails, it will perform a actual dataQuery.
	 *
	 * @param $query The query name to run
	 * @param $lifetime The max age of the cached query
	 */
	function cachedQuery($query, $parameters = array(), $lifetime = false)
	{
		// Start timer
		$timer	=	new timer;

		// Lifetime of cache?
		if (is_numeric($lifetime) == false )
		{
			$lifetime	=	$this->config->cache_lifetime;
		}

		// Cachefile we are working with (path, queryname_signature.cache.php)
		$parameters[0]	=	ifsetor($parameters[0], false);
		$parameters[1]	=	ifsetor($parameters[1], false);

		$cacheFile	=	$this->queryCacheDir."query_{$query}_{$parameters[0]}{$parameters[1]}.cache.php";


		// Should the request go directly to the query system?
		if ($this->config->cache_active != 1)
		{
			$toReturn	=	$this->query($query, $parameters);
		}
		else
		{
			// Increase hit counter
			$this->cacheCount++;

			// Cachefile does not exists, create it
			if (is_file($cacheFile) == false )
			{
				$handle = fopen($cacheFile, 'w+');
				$toCache = serialize($this->query($query, $parameters));
				if (fwrite($handle, $toCache) === false)
				{
					// Woah! We need to report this!
					trigger_error('Cache: Could not write cache data to disk. File: ' . $cacheFile, E_USER_WARNING);
				}
				$toReturn = unserialize($toCache);
			}
			else
			{
				// Get data from cachefile
				$toReturn	=	unserialize(file_get_contents($cacheFile));

				// Update cache if the data is to old
				if (filemtime($cacheFile) < time() - $lifetime)
				{
					$handle		=	fopen($cacheFile, 'w+');
					$toCache	=	serialize($this->query($query, $parameters));
					if (fwrite($handle, $toCache) === false)
					{
						// Notify the authorities!
						trigger_error('Cache: Could not write cache data to disk. File: ' . $cacheFile, E_USER_WARNING);
					}
					$toReturn	=	unserialize( $toCache );
				}
			}

			// Get debug info
			$debug_cachetime	=	$timer->stop_and_display(4);
			$this->cacheTime	+=	$debug_cachetime;

			if ($this->debug == 1)
			{
				$this->debugData	.=	'<table class="debug-cache">
	<tr class="query">
		<th>Cache query</th>
		<td><pre>' . $query . '</pre></td>
	</tr>
	<tr class="time">
		<th>Query time</th>
		<td>' . $debug_cachetime . ' seconds</td>
	</tr>
</table>
';
			}
		}

		// And we are done, let`s finish :)
		return $toReturn;
	}

	/**
	 * Performs a query against the database system (through driver)
	 *
	 *
	 * @param string $query
	 * @param array $parameters Parameters to pass to SQL
	 */
	function query($query, $parameters = array())
	{
		// Start timer
		$timer	=	new timer;

		// Increase counter
		$this->queryCount++;

		// Connect to database if needed
		if ($this->connected == false)
		{
			$this->connect();
		}

		// Prepare, and safen up
		$safe_parameters	=	'';
		foreach ($parameters as $unsafe)
		{
			$safe_parameters[]	=	$this->dbDriver->escape_string($unsafe);
		}

		// Check first if query function contains multiple queries
		$queries = $this->queries->$query($safe_parameters);
		if (is_array($queries))
		{
			foreach ($queries as $q)
			{
				$data	=	$this->dbDriver->query($q, $safe_parameters);
			}
		}
		else
		{
			//Do single query
			$data	=	$this->dbDriver->query($queries, $safe_parameters);
		}

		// Get debug info
		$debug_sqltime	=	$timer->stop_and_display(4);
		$this->sqlTime	+=	$debug_sqltime;

		if ($this->debug == 1)
		{
			$this->debugData	.=	'<table class="debug-sql">
	<tr class="query">
		<th>SQL Query</th>
		<td><pre>' . str_replace("\t", '', $this->dbDriver->currentQuery) . '</pre></td>
	</tr>
	<tr class="time">
		<th>Query time</th>
		<td>' . $debug_sqltime . ' seconds</td>
	</tr>
</table>
';
		}

		// And done
		return $data;
	}

	/**
	 * Connect to the database
	 *
	 */
	function connect()
	{
		// Get file name
		$file	=	explode('/', $_SERVER['PHP_SELF']);
		$file	=	str_replace('.php', '', $file[ ( count($file) - 1 ) ]);

		// Require some libs
		require_once('inc/drivers/sql/driver_' . DRIVER_SQL . '.php');
		require_once('inc/drivers/sql/sql_' . DRIVER_SQL . '_global.php');
		require_once('inc/drivers/sql/sql_' . DRIVER_SQL . '_' . $file . '.php');

		$this->dbDriver	=	new Database($this->config);
		$this->queries	=	new sql($this->dbDriver->db_prefix);

		return true;
	}

	/**
	 * Flush the cache
	 *
	 * This goes through the entire cache folder, and unlinks all cache files,
	 * regardless of age.
	 *
	 * @todo Is nesting it this deep really necessary? Recursiveness could
	 * 		possibly reduce the nesting to a bare minimum (only one while loop).
	 * @param boolean $only_templates Only unlink the template files
	 */
	function flush_cache($only_templates = false)
	{
		if ($dir = opendir($this->cacheRoot))
		{
			while (($file = readdir($dir)) !== false)
			{
				if (is_dir($this->cacheRoot . $file) && $file != '..' && $file != '.')
				{
					if ($subDir = opendir($this->cacheRoot . $file))
					{
						while (($cacheFile = readdir($subDir)) !== false)
						{
							$path = $this->cacheRoot . $file . '/' . $cacheFile;

							if (is_file($path))
							{
								$tmp = explode('/', $path);
								if ($only_templates == true)
								{
									//If templates are deleted, we need to delete all cache
									if (substr($tmp[3],0, 4) == 'tpl_')
									{
										unlink($path);
									}
									if (substr($tmp[3],0, 4) == 'quer')
									{
										unlink($path);
									}
								}
								else
								{
									if (substr($tmp[3],0, 4) == 'quer')
									{
										unlink($path);
									}
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Get the last ID from an insert
	 */
	function last_insert_id()
	{
		return $this->dbDriver->last_insert_id();
	}
}

?>