<?php
/**
 * Error_handler.php - The custom error handler for PHP specific errors.
 *
 * Defaults to only log the errors, but may later be extended to involve
 * taking active precautions based on what is happening (ex. DoS).
 *
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @package Vikingboard
 *
 * $Id: error_handler.php 1848 2007-11-24 00:30:54Z jorgenboard $
 */


/**
 * Custom error handler
 *
 * Handles all troublesome PHP errors, and disposes of them where you need. The
 * default action is to only log to error.log, but future revisions may perform
 * additional logic, based on the severity of the error.
 *
 * @param int $errno
 * @param string $err_msg
 * @param string $filename
 * @param int $linenum
 * @param array $errcontext
 */
function error_handler($errno, $err_msg, $filename, $linenum, $errcontext)
{
	switch ($errno)
	{
		// Block out all unmanagable or unwanted errors
		case E_COMPILE_ERROR:
		case E_COMPILE_WARNING:
		case E_CORE_ERROR:
		case E_CORE_WARNING:
		case E_ERROR:
		case E_PARSE:
		case E_STRICT:
		break;


		// These error classes are we able to handle. Notices is ignored in production
		case E_NOTICE:
			if (IN_DEVELOPMENT == false)
			{
				break;
			}

		case E_USER_ERROR:
		case E_USER_NOTICE:
		case E_USER_WARNING:
		case E_WARNING:


		$Error = new Error;
		/**
		 * @todo Replace large if/elseif/else-structure with a switch and some functions
		 *
		 *		Perhaps a simple Error class, with methods for each error type? (SQL, PHP, Cache, template ++).
		 * 		It certainly allows for much more optimization and customization of the handling of the different
		 * 		error types.
		 */

		// Is DB error?
		if (strchr($err_msg, 'SQL: '))
		{

			// DB error
			$type = 'SQL';

			//Remove "SQL" from error string
			$err_msg = str_replace('SQL: ', '', $err_msg);

			// Get the current query
			if (key_exists('q', $errcontext))
			{
				$current_query	=	$errcontext['q'];
			}
			else
			{
				$current_query	=	'';
			}

			// Critical errors require us to kill the board (Downed MySQL-server or worse)
			// Later revisions will silently notify the admin and continue working in a
			// restricted mode (read-only) from cache.
			if ($errno == E_USER_WARNING)
			{
				die('The database server appears to be offline. The staff has been alerted, please try again in a little while.');
				// $this->Error->alert();
			}

		}
		// Is cache error?
		elseif (strchr($err_msg, 'Cache:'))
		{
			$type = 'Cache';

			$err_msg = str_replace('Cache: ', '', $err_msg);

			$filename 		= '';
			$linenum 		= '';
			$current_query 	= '';

			// Get the cache file that generated the error
			if (strchr($err_msg, 'File: '))
			{
				$filename = explode('File: ', $err_msg);
				$err_msg = trim($filename[0]);
				$filename = $filename[1];
			}

		}
		// Is PHP error
		else
		{
			$type = 'PHP';

			// Remove HTML, html_errors may be set in php.ini
			$err_msg = strip_tags($err_msg);

			// advanced debug info if IN_DEVELOPMENT
			if ((IN_DEVELOPMENT == true) && @$_GET['backtrace'] == 1)
			{
				$backtrace = serialize(debug_backtrace());
			}
			else
			{
				$context = '';
			}

			if (IN_DEVELOPMENT == true)
			{
				// Get the last successful query
				if (key_exists('data', $errcontext))
				{
					$current_query	=	$errcontext['data']->dbDriver->currentQuery;
				}
				else
				{
					$current_query	=	'';
				}
			}
		}

		// We need the Log engine
		require_once 'inc/lib/log.php';

		// Log the error and exit
		$Log = new Log;

		if (!$Log->error($type, $err_msg, $filename, $linenum, VBOARD, $current_query, $errno, $backtrace))
		{

			// Failback if the logging don't work (error.php not writable) ?

			/**
			 * @todo Remove placeholder, and build actual functionality :p
			 */
			echo 'Could not write to error log. Check permissions.<br />';

			// Warn admin
			//$this->alert_admin();
		}


		break;
	}
}


class Error
{
	function SQL()
	{

	}

	function PHP()
	{

	}

	function Cache()
	{

	}

	function Security()
	{

	}

	function Template()
	{

	}
}
?>