<?php
/**
 * log.php - System logger
 *
 * This file is the home of the Log class.
 *
 * @package Vikingboard
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version $Rev: 1885 $
 *
 *
 */


/**
 * class Log
 *
 */
class Log
{
	/**
	 * Log error(s)
	 *
	 * This method logs errors like ordinary code errors (warnings and fatal
	 * errors) or an external service being down.
	 *
	 * It also logs the severity of these errors, and takes action when
	 * certain thresholds are reached, like when an error of a certain severity
	 * has occurred, or a number of accumulated errors has occurred.
	 *
	 * @param string $descr Full description of the specific error
	 *
	 * @todo Use error codes instead of strings, to be able to translate errors
	 * 			and don't break the database normalization too much.
	 */
	function error($type, $error, $in_file, $in_line, $calling_file, $current_query = '', $errno, $context = '')
	{
		//construct the log data array
		$log_data = array(	'type' 			=> $type,
							'error' 		=> $error,
							'in_file'		=> $in_file,
							'in_line'		=> $in_line,
							'calling_file'	=> $calling_file,
							'current_query'	=> $current_query,
							'datetime' 		=> time(),
							'errno' 		=> $errno,
							'context'		=> $context);

		// write the array to the CSV (Comma Separated Values) flatfile
		return $this->write_log($log_data, 'error.php');
	}

	function admin_action($where, $what)
	{
		//or should this be written to SQL, for simplicity?
		return $this->write_log($what, $where);
	}

	/**
	 * Write the log file (CSV format)
	 *
	 * @param array $data
	 * @return bool true if success, false if failure
	 */
	function write_log($data, $logfile)
	{
		//do we have fputcsv?
		if (!function_exists('fputcsv'))
		{
			require_once('fputcsv.php');
		}

		//open log file handle
		if (!$fp = fopen('cache/temp/' . $logfile, 'a'))
		{
			trigger_error('Corrupt log file or insufficient file permissions. Check permissions and consistency of /cache/temp/' . $logfile, E_USER_WARNING);
			return false;
		}

		//write the error line to the error logfile
		if (fputcsv($fp, $data, ',', '"'))
		{
			return true;
		}
		else
		{
			trigger_error('Cannot write to error log. Check file permissions.', E_USER_ERROR);
			return false;
		}
	}
}

?>