<?php
/**
*
* This is the Vikingboard template handler file
*
* @package Vikingboard
* @author Tor Henning Ueland <tor.henning@vikingboard.com>
* @version 0.1
* @copyright Copyright (c) 2004-2006 The Vikingboard Project
* @license GPL {@link http://www.gnu.org/licenses/gpl.html}
* @link http://www.vikingboard.com
*
* $Id: screen.php 1885 2007-12-03 00:16:43Z jorgenboard $
*/

//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}

/**
 * Screen
 *
 * The screen handler, controls all output made to the screen. There should ideally
 * be no echo()'s in the code. Direct output is only to be made in emergencies.
 *
 */
class Screen
{

	//Lets initialize some variables
	var $html_body		=	'';	//Data within the wrapper
	var $html_wrapper	=	'';	//Wrapper to the forum
	var $cached			=	array();	//html cache
	var $cachedHTML		=	array();	//html actual cache
	var $config;
	var $timer;
	var $langFunction;
	var $compileInLang	=	0;


	/**
	 *	Constructor!
	 *
	 */
	function Screen(&$data, &$config, &$user, &$timer)
	{
		$this->config = &$config;
		$this->data   = &$data;
		$this->user   = &$user;
		$this->timer  = &$timer;

		//Set up hostname
		$this->hostName = str_replace(':', '', $_SERVER['HTTP_HOST']);

		// Are we in development?
		if (IN_DEVELOPMENT == true)
		{
			$this->config->cache_active = 0;
		}

	}

	/**
	 * Set`s a language function to use in templates
	 */
	function set_lang($name)
	{
		$this->langFunction = $name;
		$this->compileInLang = 1;
		return true;
	}

	/**
	 * Builds up a template file
	 *
	 */
	function build_template($name, $path)
	{
		if (is_file($path) && $this->config->cache_active == true)
		{
			return true;
		}
		else
		{
			// Add inn language class
			$startCode	=	'';

			if ($this->compileInLang == 1)
			{
				$startCode = '<?php $lang = new '.$this->langFunction.'; ?>';
				$this->compileInLang = 0;
			}
			elseif (!empty($this->langFunction))
			{
				$startCode = '<?php if(isset($lang) == false) { $lang = new '.$this->langFunction.';} ?>';
			}

			// Attempting a read
			$html	=  file_get_contents('static/'.$this->config->def_tpl.'/templates/'.$name.'.html');

			// Couldn't read template
			if (!$html)
			{
				trigger_error('Cache: Could not read template file: ' . $path, E_USER_WARNING);
				return false;
			}

			$html	=  $startCode.$html;

			// Make code sane
			$html	=	preg_replace('/{\$user->(.*?)}/i', '<?php echo \$this->user->$1; ?>', $html);  //echo user info (cannot be used in if-check(s))
			$html	=	str_replace('{$skin_id}', '<?php echo $this->user->skin_set; ?>', $html); //echo skin ID
			$html	=	preg_replace('/{\$lang\-\>(.*?)}/i', '<?php echo $lang->$1; ?>', $html); //echo from language
			$html	=	preg_replace('/{\$([^\.]*?)}/i', '<?php echo \$data[\'$1\']; ?>', $html); //Default echo

			// Create if/else tags
			$html	=	preg_replace('#{if \$([_a-zA-Z0-9]+?) (.*?)}#is', '<?php if (\$data[\'$1\'] $2 ) : ?>', $html); // start if as foo
			$html	=	preg_replace('#{if \$([_a-zA-Z0-9]+?)}#is', '<?php if (\$data[\'$1\']) : ?>', $html); // start if
			$html 	=	preg_replace('#{elseif \$([_a-zA-Z0-9]+?) (.*?)}#is', '<?php elseif (\$data[\'$1\'] $2 ) : ?>', $html); // elseif as foo
			$html	=	preg_replace('#{elseif \$([_a-zA-Z0-9]+?)}#is', '<?php elseif (\$data[\'$1\']) : ?>', $html); // elseif
			$html	=	preg_replace('#{if \$([_a-zA-Z0-9]+?)\.([_a-zA-Z0-9]+?)}#is', '<?php if (\$$1->$2) : ?>', $html); // start if within foreach loop
			$html	=	preg_replace('#{elseif \$([_a-zA-Z0-9]+?)\.([_a-zA-Z0-9]+?)}#is', '<?php elseif (\$$1->$2) : ?>', $html); // elseif within foreach loop
			$html	=	str_replace('{else}', '<?php else : ?>', $html);  // else start
			$html	=	str_replace('{endif}', '<?php endif; ?>', $html); // end if check

			// Create loop tags
			$html	=	preg_replace('#{foreach \$([_a-zA-Z0-9]+?) as \$(.*?)}#is', '<?php foreach(\$data[\'$1\'] as \$$2): ?>', $html); //Start
			$html	=	preg_replace('/{\$(.*?)\.(.*?)}/i', "<?php echo \$$1->$2; ?>", $html); //Get value tag
			$html	=	str_replace('{endforeach}', '<?php endforeach; ?>', $html); //end tag

			// Save code to file
			$h	=	fopen($path, 'w+');
			if (fwrite($h, $html) == false)
			{
				trigger_error('Cache: Could not write template file to disk. File: ' . $path, E_USER_WARNING);
			}

			fclose($h);
			return true;
		}
	}

	/**
	 * Returns a parsed template
	 *
	 * @return bool
	 */
	function return_html($templatename, $data = false)
	{

		//Figure out template path
		$tplFile = './cache/'.$this->hostName.'/tpl_'.$this->user->skin_set.'_'.$templatename.'.php';

		// build_template() will figure out if it is cached or not
		if ((is_file($tplFile) == false) || ($this->config->cache_active == false))
		{
			$this->build_template($templatename, $tplFile);
		}

		//By this point the template is good to go, so go
		ob_start();
		include($tplFile);
		$html = ob_get_contents();
		ob_end_clean();
		return $html;
	}

	/**
	 * Parse a template and add the content to output buffer
	 *
	 * @return bool
	 */
	function add_html($templatename, $data = array())
	{

		//Figure out template path
		$tplFile = './cache/'.$this->hostName.'/tpl_'.$this->user->skin_set.'_'.$templatename.'.php';

		// build_template() will figure out if it is cached or not
		if (is_file($tplFile) == false || $this->config->cache_active == false)
		{
			$this->build_template($templatename, $tplFile);
		}

		//By this point the template is good to go, so go
		ob_start();
		include($tplFile);
		$this->html_body .= ob_get_contents();
		ob_end_clean();
	}

	/**
	 * Add the main forum menu to the output buffer
	 *
	 * @return none
	 */
	function add_menu()
	{
		// Set language
		$this->set_lang('lang_header');

		//Get user`s group
		$uID = $this->user->group;

		//  Menu for a guest that is not logged in
		/**
		 * @todo Replace with $user->is_guest
		 */
		if ($uID == '6' || $uID == '5' || $uID == '4' ||  !$uID)
		{
			$html['forum_url']	=	$this->config->forum_url;

			// add template to output
			return $this->return_html('mainmenu_guest', $html);
		}
		// Menu for a logged in user
		else
		{
			// Create an array of words to use here
			$words	=	array
			(
				'forum_url'			=>	$this->config->forum_url,
				'newPMs'			=>	$this->user->new_pms,
				'logged_in_as_data'	=>	$this->user->name,
				'uid'				=>	$this->user->id
			);

			// add template to output
			return $this->return_html('mainmenu_member', $words);
		}
	}


	/**
	 * Add the admin menu to the output buffer
	 *
	 * @return none
	 */
	function add_admin_menu()
	{
		$this->set_lang('lang_admin_global');

		// add template to output
		return $this->return_html('adminmenu');
	}

	/**
	 * Adds a admin wrapper to output buffer, used in admin CP
	 *
	 * @return none
	 */
	function add_admin_wrapper($htmlCode)
	{

		//Debug info is nice :)
		if (!empty($_SERVER['QUERY_STRING']))
		{
			$qstring	=	str_replace('&amp;debug=1', '', $_SERVER['QUERY_STRING']);
			$qstring	=	str_replace('debug=1', '', $qstring);

			$qstring	=	'?' . $qstring . '&amp;debug=1';
		}
		else
		{
			$qstring	=	'?debug=1';
		}

		// Run stats
		$stats	=	'Run time: ' . $this->timer->stop_and_display(3) . ' seconds, used ' .
					'<a href="' . $_SERVER['PHP_SELF'] . $qstring . '">' .
					$this->data->queryCount . '</a> queries.';

		//Show copyright information
		$copyright	=	'Powered by <a href="http://vikingboard.com/">Vikingboard</a> (' . $this->config->version . ')';

		//show debug?
		$debugdata = '';
		if ($this->config->show_debuginfo && ifsetor($_GET['debug'], false))
		{
			$debugdata	=	ifsetor($this->data->debugData, false);
		}

		if (IN_DEVELOPMENT)
		{
			$copyright .= '<br /><em>Development mode, things are quite slow.</em>';
		}

		//define array witch contains global wrapper variables
		$data	=	array
		(
			'title'		=>	'Vikingboard Admin CP',
			'menu'		=>	$this->add_admin_menu(),
			'body'		=>	$htmlCode,
			'copyright'	=>	$copyright,
			'stats'		=>	$stats,
			'debugdata'	=>	$debugdata,
		);

		//Figure out template path
		$tplFile = './cache/'.$this->hostName.'/tpl_'.$this->user->skin_set.'_adminwrapper.php';

		// build_template() will figure out if it is cached or not
		if(is_file($tplFile) == false || $this->config->cache_active == false) {
			$this->build_template('adminwrapper', $tplFile);
		}
		//By this point the template is good to go, so go
		ob_start();
		include($tplFile);
		$this->html_body .= ob_get_contents();
		ob_end_clean();

		return true;
	}

	/**
	 * Adds a forum wrapper to the main forum pages, which again is added to output buffer.
	 *
	 * @return none
	 */
	function add_wrapper($title, $nav=array())
	{
		//language
		$lang	=	new lang_global;
		$this->set_lang('lang_global');

		//Start of navigation
		$navigation	=	$this->return_html('navigation_start', array('forumname' => $this->config->forum_name));

		//The navigation links
		foreach ($nav as $row => $value)
		{
			if ($row)
			{
				$thisRow	=	array
				(
					'url'	=>	$row,
					'name'	=>	$value,
				);
				$navigation .= $this->return_html('navigation_link', $thisRow);
			}
			else
			{
				$thisRow	=	array
				(
					'text'	=>	$value,
				);
				$navigation .= $this->return_html('navigation_no_link', $thisRow);
			}
		}

		$navigation	.=	$this->return_html('navigation_end');

		//show run time and queries?
		$runtime	=	'';
		$queries	=	'';
		if ($this->config->exec && $this->user->g_is_admin)
		{
			if (!empty($_SERVER['QUERY_STRING']))
			{
				$qstring = str_replace('&amp;debug=1', '', $_SERVER['QUERY_STRING']);
				$qstring = str_replace('debug=1', '', $qstring);

				$qstring = '?' . $qstring . '&amp;debug=1';
			}
			else
			{
				$qstring = '?debug=1';
			}

			$runtime = 'Runtime was ' . $this->timer->stop_and_display(3) . ' seconds';
			if (isset($_GET['debug']) && $_GET['debug'] == 1)
			{
				$runtime .= ' (SQL: '.$this->data->sqlTime.' sec, cache: '.$this->data->cacheTime.' sec)';
			}

			$queries = ', used <a href="' . $_SERVER['PHP_SELF'] . $qstring . '">' . $this->data->queryCount . ' queries</a>';
			if (isset($_GET['debug']) && $_GET['debug'] == 1)
			{
				$queries .= ' (' . $this->data->cacheCount . ' cachehits)';
			}
		}

		//show debug?
		if ($this->config->show_debuginfo && (isset($_GET['debug'])) && $this->user->g_is_mod)
		{
			$debugdata = $this->data->debugData;
		}
		else
		{
			$debugdata = '';
		}


		$category_id	=	ifsetor($_GET['c'], false);
		$forum_id		=	ifsetor($_GET['f'], false);
		$topic_id		=	ifsetor($_GET['t'], false);
		$everywhere		=	($category_id || $forum_id || $topic_id) ? true : false;

		//define array witch contains global wrapper variables
		$data	=	array
		(
			'title' 		=>	$title . ' - ' . $this->config->forum_name,
			'forum_name'	=>	$this->config->forum_name,
			'forum_url'		=>	$this->config->forum_url,
			'navigation'	=>	$navigation,
			'quick_login'	=>	(($this->user->id == '1') ? true : false),
			'body' 			=>	$this->html_body,
			'menu' 			=>	$this->add_menu(),
			'category_id'	=>	$category_id,
			'forum_id'		=>	$forum_id,
			'topic_id'		=>	$topic_id,
			'everywhere'	=>	$everywhere,
			'run_stats' 	=>	$runtime . $queries,
			'debugdata'		=>	$debugdata,
			'is_admin'		=>	$this->user->g_is_admin,
			'show_version'	=>	(($this->config->hide_version) ? false : true),
			'version'		=>	$this->config->version,
			'show_stats'	=>	(($this->config->exec && $this->user->g_is_admin) ? true : false),
			'dev_mode'		=>	IN_DEVELOPMENT
		);

		//Figure out template path
		$tplFile = './cache/'.$this->hostName.'/tpl_'.$this->user->skin_set.'_wrapper.php';

		// build_template() will figure out if it is cached or not
		if (is_file($tplFile) == false || $this->config->cache_active == false)
		{
			$this->build_template('wrapper', $tplFile);
		}

		//build page
		ob_start();
		include($tplFile);
		$this->html_body = ob_get_contents();
		ob_end_clean();

		return true;

	}

	/**
	 * Flushes output buffer to the webclient
	 *
	 * @return none
	 */
	function print_page()
	{
		if (IN_DEVELOPMENT == true)
		{
			$this->html_body = preg_replace('#a href="(.+?).php"#is', "a href=\"$1.php?error=1\"", $this->html_body);
			$this->html_body = preg_replace('#a href="(.+?).php\?([^(error)]+?)"#is', "a href=\"$1.php?$2&error=1\"", $this->html_body);
		}
		echo $this->html_body;
		flush();
		unset($this->html_body);
	}
}

?>