<?php
/**
 * Upload.php
 *
 * Handles file uploads and fetching uploaded files
 *
 * @package Vikingboard
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 *
 * $Id: upload.php 1390 2007-05-20 17:44:01Z jorgenboard $
 */


//oh no!
if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}

/**
 * Upload
 *
 * The class for uploading files and fetching uploaded files. Stores the files
 * on disk with random names.
 *
 */
class Upload
{

	/**
	 * Default constructor, accepts the download data
	 *
	 * @param object $data	The Data class
	 * @param object $screen The Screen class
	 * @param object $user The User class
	 * @return Upload
	 */
	function Upload($data, $screen, $user)
	{

		//what shall we do
		//to fill the empty spaces
		//where we used to talk?
		// - The Wall

		$this->screen 	= $screen;
		$this->data 	= $data;
		$this->user		= $user;
		$this->lang_upload	= $lang_upload;
	}


	/**
	 * Display the upload form
	 *
	 * @param array $html Screen-parseable array
	 * @return string Upload form data
	 */
	function display_form($html)
	{
		return $this->screen->return_html('upload_form', $html);
	}


	/**
	 * Upload the file
	 *
	 * The upload itself happens here. This function
	 * will get the file and decide what to do with it.
	 *
	 * @return void
	 */
	function upload_file()
	{
		//Get file info
		$filename 	= (string)$_FILES['upload']['name'];
		$tmpname	= $_FILES['upload']['tmp_name'];
		$type		= (string)$_FILES['upload']['type'];
		$size 		= (int)$_FILES['upload']['size'];

		//randomizing name, so that you cannot execute the file directly from the server
		$rnd_name 	= md5(uniqid(rand(), true));

		//check for valid mime type
		if (!$this->is_valid_mime($type))
		{
			error_msg($this->lang_upload->invalid_mimetype, 1);
		}

		//is this a valid/approved file extension?
		$file_ext = explode('.', $filename);
		$file_ext = end($file_ext);

		if (!$this->is_valid_file_ext($file_ext))
		{
			error_msg($this->lang_upload->invalid_extension, 1);
		}

		//check for max sizes, and quota (per user/per post limit)
		//if (($size > $config->max_file_size) || (user_quota_used($user->id) + $size > user_max_quota($user->id)))
		//{
		//	error_msg($lang_upload->quota_full, 1);
		//}

		//put the file info into DB
		$this->data->query('add_file_info', array($filename, $type, $size, $rnd_name, $this->user->id, 1));

		//and moving the file to the uploads directory and go do something fun
		if (move_uploaded_file($tmpname, 'uploads/' . $rnd_name))
		{
			notify($this->lang_upload->success);
		}
		else
		{
			error_msg($this->lang_upload->upload_attack, 1);
		}

	}

	/**
	 * Is the MIME-type valid?
	 *
	 * @param string $mimetype MIME-type to test
	 * @return boole true if valid, false if invalid
	 */
	function is_valid_mime($mimetype)
	{
		//get all allowed mimetypes (whitelist)
		//this will be put into the DB later on
		$allowed_mimes = array(	'text/plain', 'image/jpeg',
								'application/pdf', 'image/png',
								'application/zip', 'text/x-c++',
								'video/x-ms-wmv');

		if (in_array($mimetype, $allowed_mimes))
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Is the file extension valid?
	 *
	 * @param string $file_ext File extension
	 * @return bool true if valid, false if invalid
	 */
	function is_valid_file_ext($file_ext)
	{
		//get all allowed file extensions (whitelist)
		//fetch this from DB later on
		$allowed_exts	=	array(	'pdf', 'txt', 'png', 'jpg', 'jpeg',
									'gif', 'doc', 'odt', 'html', 'css',
									'js', 'psd');

		if (in_array($file_ext, $allowed_exts))
		{
			return true;
		}
		else
		{
			return false;
		}

	}


	/**
	 * Get file
	 *
	 * Fetches and displays, or downloads the specified
	 * uploaded file
	 *
	 * @param int $id File ID
	 * @param int $download Download or just display in browser?
	 * @return mixed $file_data The file data
	 */
	function get_file($id, $download)
	{
		//get file info
		$file = $this->data->cachedQuery('get_file', array($id));
		$file = $file[0];

		if (!$file)
		{
			die('This file does not exist');
		}

		/**
		 * @todo implement group-based or forum-based permissions (chown/chmod)
		 */
		if ($file->public != '1' && $this->user->id != $file->uid)
		{
			die('You are not allowed to see this file');
		}

		//assume that files from DB are already valid and A-okay

		/*check if it is possible to show this file without download.
		 .html and .php should of course ONLY be available through dl
		 although the admin should be able to edit this themselves through admin cp */
		if ($download != '1')
		{
			header('Content-type: ' . $file->mime);

			$file_data = file_get_contents('uploads/' . $file->filename);
			return $file_data;
		}
	}
}

?>