<?php
/**
*
* @package Vikingboard
* @author Tor Henning Ueland <tor.henning@vikingboard.com>
* @copyright Copyright (c) 2004-2007 The Vikingboard Project
* @license GPL {@link http://www.gnu.org/licenses/gpl.html}
* @link http://vikingboard.com/
*
* $Id: user.php 1843 2007-11-23 20:36:42Z ueland $
*
 */

if (!defined('VBOARD'))
{
	die('Hacker attempt in ' . basename(__FILE__));
}



/**
 * Vikingboard main user class file, handles user management, such as
 * figuring out who "you" are and so on.
 *
 */
class User
{

	/**
	 * Find and process information about a user
	 *
	 * @param object &$data
	 * @param object &$config
	 * @return bool
	 *
	 */
	function User(&$data, &$config, &$session)
	{
		$this->data 	= &$data;
		$this->config 	= &$config;
        $this->session  = &$session;

		//A while since last session update?
        if (isset($this->session->user->updateTime))
        {
            if (forumtime() >= ($this->session->user->updateTime + 30))
		    {
			    $this->session->update_session();
		    }
        }

		// If the session is owned by a guest, and we have a username/password stored, lets login
		if (isset($this->session->user->uid)&& $this->session->user->uid == 1)
		{
			if (isset($_COOKIE["{$this->config->cookie}_username"]) && isset($_COOKIE["{$this->config->cookie}_password"]))
			{
				$login = $this->data->query('get_info_about_user_logging_in', array(
                                                        $_COOKIE[ "{$this->config->cookie}_password" ],
                                                        $_COOKIE[ "{$this->config->cookie}_username" ]));
                $login = $login[0];
				if ($login->name && $login->group && $login->id)
				{
					// We found a stored user, lets log him/her in :)

					// Should the user be hidden?
					if ($_COOKIE["{$this->config->cookie}_hideme"] == 1)
					{
						$hideme = 1;
					}
					else
					{
						$hideme = 0;
					}

					// Update sessions table
                    $this->session->login($login->id, $login->group, $login->name, $hideme, 1);
				}
				else
				{
					// If wrong login info is stored we will delete the cookies
					setcookie("{$this->config->cookie}_username", '', forumtime()+31536000);
					setcookie("{$this->config->cookie}_password", '', forumtime()+31536000);
					setcookie("{$this->config->cookie}_hideme", '', forumtime()+31536000);
				}
			}
		}

		//Figure out where the user is
		$loc = explode('/', $_SERVER['REQUEST_URI']);
		$loc = html_entity_decode($loc[(count($loc) - 1)], ENT_COMPAT, 'UTF-8');

		if ($loc != 'login.php')
		{
			$this->session->set('reffering_page', ifsetor($_SESSION['current_page'], false));
			$this->session->set('current_page',$loc);
		}

		//Now actually define data to the user class

		/**
		 * @todo This seems like a backwards way of doing things. All user vars
		 * should be available in User without it being just a memory-consuming
		 * placeholder for the real data in Session.
		 */
		$this->name		 				=	$this->session->user->name;
		$this->id						=	$this->session->user->uid;
		$this->group					=	$this->session->user->group;
		$this->lang						=	$this->session->user->lang;
		$this->title					=	$this->session->user->title;
		$this->new_pms					=	$this->session->user->new_pms;
		$this->use_emailer				=	$this->session->user->use_emailer;
		$this->g_view_offline			=	$this->session->user->g_view_offline;
		$this->g_viewforum				=	$this->session->user->g_viewforum;
		$this->g_can_search				=	$this->session->user->g_can_search;
		$this->g_can_delete_own_posts   =	$this->session->user->g_can_delete_own_posts;
		$this->g_can_delete_own_threads =	$this->session->user->g_can_delete_own_topics;
		$this->g_can_edit_own_posts		=	$this->session->user->g_can_edit_own_posts;
		$this->g_create_new_posts		=	$this->session->user->g_create_new_posts;
		$this->g_create_new_polls		=	$this->session->user->g_create_new_polls;
		$this->g_is_admin				=	$this->session->user->g_is_admin;
		$this->g_is_mod					=	$this->session->user->g_is_mod;
		$this->show_sig					=	$this->session->user->show_sig;
		$this->skin_set					=	$this->session->user->skin_set;
		$this->memberlist_per_page		=	$this->session->user->memberlist_per_page;
		$this->posts					=	$this->session->user->posts;
		$this->avatar					=	$this->session->user->avatar;
		$this->g_name					=	$this->session->user->g_name;
		$this->hide						=	$this->session->user->hide;
	}

	/**
	 * Reload User
	 *
	 * Kill the User sesssion object, to force a reload of all user properties
	 *
	 */
	function reload()
	{
	    $this->session->clear('user');
    }

	/**
	 * Get users online/offline status
	 *
	 * @param int $uid User id
	 * @return bool true if online, int last seen time if offline, bool false if never logged in
	 */
	function is_online($uid)
	{
		// Get user info
		$userinfo = $this->data->cachedQuery('get_last_update', array($uid));

		// Never logged in at all
		if (!$userinfo)
		{
			return false;
		}

		$userinfo = $userinfo[0];

		// Find out if the user is online (or has been within the cut time)
		$update_age = forumtime() - $userinfo->time;
		$max_age	= ($this->config->cut_time * 60);

		if ($update_age <= $max_age)
		{
			return true;
		}
		else
		{
			// Not online, returning the last known time (if any)
			if (!empty($userinfo->time))
			{
				return $userinfo->time;
			}
		}

	}
}
?>
