<?php
/**
 *
 * This is the Vikingboard main index file
 *
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2.0
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: index.php 1885 2007-12-03 00:16:43Z jorgenboard $
 *
 * *************************************************************
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * http://www.gnu.org/licences/gpl.html
 * *************************************************************
 *
 * The legal statements made in this file is valid for most of the
 * other files. :p
 */

// Yes, we are here
define('VBOARD', __FILE__);

// Are we installed?
if(file_exists('install/install.lock') == false)
{
	header('Location: install/install.php');
}

// Require kernel
require_once('inc/kernel.php');


 /**
  *	Vikingboard frontpage class
  *
 */
class Index
{
	// Define variables
	var $data;
	var $user;
	var $screen;
	var $config;
	var $lang;


	/**
	 * Frontpage constructor
	 */
	function Index(&$data, &$user, &$screen, &$config)
	{
		// Set up some vars
		$this->data		=	&$data;
		$this->user		=	&$user;
		$this->screen	=	&$screen;
		$this->config	=	&$config;
		$this->lang		=	new lang_frtpage;

		$this->screen->set_lang('lang_frtpage');

		// Lets find out what we want to do
		switch(ifsetor($_GET['act'], false))
		{
			case 'mod':
			require('./inc/lib/module_loader.php');
			load_module();
			break;

			case 'task':
			require('./inc/lib/task_loader.php');
			load_task();
			break;

			case 'phpinfo':
			$this->show_phpinfo();
			break;

			case 'vikings':
			$this->show_vikings();
			break;

			default:
			$this->show_index();
		}
	}


	/**
	 * Generate a list of forums in a category
	 *
	 * @param int cat
	 * @return string forums
	 */
	function forums($cat)
	{
		// Load data from cache
		$db_forum	=	$this->data->cachedQuery('get_forums_in_category', array($cat));

		// Create error message if we don't have any forums here
		if (!isset($db_forum[0]))
		{
			 $forums	=	$this->screen->return_html('no_forum', array('no_forum' => $this->lang->no_forum));
		}
		// If we got some forums, lets loopie :)
		else
		{
			// Add the forums to the template
			$forums = '';
			foreach ($db_forum as $forum)
			{
				// Create a last post link
				if ($forum->stat_posts != 0)
				{
					$find	=	array(
						'%topic_id%',
						'%topic_title%',
						'%author_id%',
						'%author%',
						'%time%',
					);
					$replace	=	array
					(
						$forum->lpost_tid,
						$forum->lpost_title,
						$forum->lpost_uid,
						$forum->lpost_name,
						date($this->config->timestamp1, forumtime($forum->lpost_time)),
					);

					$lang_global	=	new lang_global;
					$last_post		=	str_replace($find, $replace, $lang_global->last_post_medium);
				}
				else
				{
	       	    	$last_post	=	false;
	  			}

				// Check for new posts in this forum
				$cookie		=	ifsetor($_COOKIE[$this->config->cookie.'_forumread'][$forum->id], false);
				$new_post	=	($forum->lpost_time >= $cookie) ? true : false;

				// Create array of data to put in output
				$html	=	array
				(
					'forum_name'		=>	$forum->name,
					'forum_id'			=>	$forum->id,
					'forum_description'	=>	$forum->descr,
					'topics'			=>	$forum->stat_threads,
					'posts' 			=>	$forum->stat_posts,
					'new_post' 			=>	$new_post,
					'last_post' 		=>	$last_post,
				);

				// Add to output
				$forums	.=	$this->screen->return_html('list_forum', $html);
			}
		}

		return $forums;
	}


	/**
	 * Frontpage constructer
	 *
	 * @return bool
	 */
	function show_index()
	{
		// Should we show a special category?
		$cat	=	safe_user_int('c', 0);

		// Load data from the database
		$_SESSION['stats']	=	ifsetor($_SESSION['stats'], false);
		if (is_object($_SESSION['stats']))
		{
			$onl_users	=	$_SESSION['stats'];
		}
		else
		{
			$onl_users	=	$this->data->query('get_online_users', array( (forumtime() - ($this->config->cut_time * 60) ) ), 10);
			$onl_users	=	$onl_users[0];
			$_SESSION['stats'] =	$onl_users;
		}

		// Load data from cache
		$categories	=	$this->data->cachedQuery('get_categories');
		$boardstats	=	$this->data->cachedQuery('get_forum_stats');
		$boardstats	=	$boardstats[0];
		$lastUser	=	$this->data->cachedQuery('get_latest_user');
		$lastUser	=	$lastUser[0];

		// Did we get a visitor record? (congrats! :))
		if ($onl_users->users > $boardstats->max_users_online)
		{
		  	// Update stats in the database
			$this->data->query('update_max_users_online', array($onl_users->users, forumtime()));
		}

		// Setting up the categories
		$categorylist = '';
		foreach ($categories as $category)
		{
			// Should we extend this cat?
			if (($category->extend == '1' && !$cat) || ($cat == $category->id) || ($category->id == $this->config->def_cat))
			{
				$html	=	array
				(
					'category_id' 	  	=>	$category->id,
					'category_name'	  	=>	$category->name,
					'list_forums' 	  	=>	$this->forums($category->id),
				);

				$categorylist	.=	$this->screen->return_html('extended_cat', $html);
			}
			elseif ($category->extend == '0' || ($category->extend == '1' && ($cat != $category->id)))
			{
				// Make to-template array
				$html	=	array
				(
					'categoryid'	=> $category->id,
					'categoryname'	=> $category->name,
					'categorydesc'	=> $category->descr,
				);

				// Add to output
				$categorylist	.=	$this->screen->return_html('list_category', $html);
			}
		}

		// Inform user of any new messages
		if ($this->user->new_pms !=0)
		{
			$newPms	=	str_replace("%num%", $this->user->new_pms, $this->lang->newPM);
		}

		// Board stats
		$find	=	array
		(
			'%users%',
			'%posts%',
			'%topics%',
			'%online%',
			'%onlinetime%',
			'%most_online%',
			'%most_online_date%',
			'%last_user%',
			'%last_user_id%',
		);
		$replace	=	array
		(
			$boardstats->total_users,
			$boardstats->total_posts,
			$boardstats->total_topics,
			$onl_users->users,
			$this->config->cut_time,
			$boardstats->max_users_online,
			date($this->config->timestamp1, forumtime($boardstats->max_users_date)),
			$lastUser->name,
			$lastUser->id,
		);
		$board_stats	=	str_replace($find, $replace, $this->lang->stats_data);

		// Global frontpage
		$lang_general	=	new lang_global;

		$html	=	array
		(
			'list_categories'	=>	$categorylist,
			'board_stats'		=>	$board_stats,
		);

		// Add the wrapper
		$lang_header	=	new lang_header();
		$navigation		=	array
		(
			"index.php?c={$cat}"	=>	$lang_header->menu_frtpage,
		);

		$this->screen->add_html('frontpage', $html);
		$this->screen->add_wrapper($lang_header->menu_frtpage, $navigation);
		$this->screen->print_page();
	}


	/**
	 * Prints out the PHPINFO if an administrator wants to view it
	 *
	 * @return bool
	 */
	function show_phpinfo()
	{
		if ($this->user->g_is_admin ==1)
		{
			phpinfo();
		}
		else
		{
			redirect('index.php');
		}
	}


	/**
	 * Vikingboard core function for images and media
	 *
	 * @return bool
	 */
	function show_vikings()
	{
		header('Content-type: image/jpeg');
		include $this->config->vikingpath;
		$viking	= 'viking';
		echo base64_decode((${$this->config->vikings}()));
	}
}

$I = new Index($data, $user, $screen, $config);

?>