<?php

/**
 *
 * The Vikingboard installer. Sets up your forum, and will automagically
 * use FTP if it detects that some file stuff is wrong or missing.
 *
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @name install - Vikingboard installer
 * @package Vikingboard
 *
 *
 * $Id: install.php 1852 2007-11-24 13:52:29Z jorgenboard $
 *
 */

//yes, we are here
define('VBOARD', __FILE__);

//Off with errors
error_reporting(E_ALL);

//Turn off magic_quotes if possible (devils work)
INI_SET('magic_quotes_gpc', 'off');
INI_SET('magic_quotes_runtime', 'off');


/**
 * Get the name of the PHP executing user, *NIX only.
 *
 * @return string PHP username or (bool)false
 */
function get_php_user()
{
	static $exec_user = null;

	if ($exec_user == null)
	{
		if (PHP_OS !== 'WINNT')
		{
			ob_start();
			passthru( 'whoami' );
			$exec_user = trim( ob_get_clean() );

			/*
			Alternative:

			$tmp = tempnam( '.', 'tmp_' );
			ob_start();
			passthru( 'ls -l '.$tmp );
			$con = og_get_clean();
			unlink( $tmp );

			do
			{
				$c = 0;
				$con = preg_replace( '#  #', ' ', $con, -1, $c );
			} while( $c > 0 );

			list( , , $user, $group ) = explode( ' ', $con );

			$exec_user = $user;
			*/
		}
		else
		{
			$exec_user = false;
		}
	}

	return $exec_user;
}


/**
 * ifsetor()
 *
 * If variable $param is set, return it. If not, return $or. Useful for globals,
 * where you most often won't know whether they are set or not, to avoid getting
 * the nasty notices.
 *
 * @param string $param Variable to check
 * @param string $or Value to assign to it if not set
 * @return mixed Either $param or $or
 */
function ifsetor(&$param, $or)
{
	return isset($param) ? $param : $or;
}

// Start with some HTML output
echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">

<head>
	<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
	<link href="install.css" rel="stylesheet" media="screen" type="text/css" />
	<title>Vikingboard - Installation</title>
</head>

<body>

<div id="wrapper">
	<div id="content">
		<h1><span>Vikingboard</span></h1>
		';

// Is Vikingboard already installed?
if (file_exists('install.lock'))
{
	die('<h1>Installation error</h1> <p>Vikingboard is already installed on this location, please remove install.lock if you want to reinstall.</p>');
}

// And starting the FTP class, just in case
include '../inc/drivers/ftp/ftp.php';

// Protecting PHP4-compability. To be gone in 0.3.
include '../inc/lib/file_put_contents.php';

$ftp	=	new FTP;
$page	=	ifsetor($_GET['page'], false);

// Where are we, and what do we want?
switch ($page)
{
	// Welcome message and a system requirements check
	default:
		welcome();
	break;

	// Does not exists?
	case '1':
		convert();
	break;

	// User input
	case '2':
		new_install();
	break;

	// Inserting stuffies in the DB
	case '3':
		database();
	break;

	// And happy days, we are done!
	case '4':
		complete();
	break;
}

/**
 * Greets the user and runs a small selftest
 *
 * @todo Help the user some more to more info, and link to manual install from the start
 * @return void
 *
 */
function welcome()
{

	// Lets meet and greet the user
	echo '
		<h2>Vikingboard installer</h2>
		<p>Welcome to the Vikingboard installer. Please read all fields and make sure that
		you type in the correct values on each field. If something goes wrong within the installation
		you will be notified of this.</p>

		<h2>Got bugs?</h2>
		<p>If you discover a bug in some way in this installer, please report the bug through our bug system, we will
		then investigate the problem as soon as possible.<br />
		Bug reports: <a href="http://sourceforge.net/tracker/?group_id=126121&amp;atid=704723">http://sourceforge.net/</a>.</p>

		<h2>Step 1: System requirements / selftest</h2>
';

	// Code reuse
	$ok = '<span class="ok">Yes</span>';
	$no = '<span class="error">No</span>';
	$nw	= '<span class="error">Not writable!</span>';

	// PHP version check
	if (version_compare(phpversion(), '4.3.0', '<'))
	{
		$version = '<span class="error">PHP version too old (PHP 5 recommended, PHP 4.3.0 required. You have ' . phpversion() . '), installation halted</span>';
		$error = '1';
	}
	elseif (version_compare(phpversion(), '5.0.0', '<'))
	{
		$version = '<span class="warning">You have ' . phpversion() . ', but PHP 5 is highly recommended</span>';
	}
	else
	{
		$version = $ok;
	}

	// MySQL function check
	if (function_exists('mysql_connect'))
	{
		$gotMySQL	=	$ok;
	}
	else
	{
	  	$gotMySQL	=	$no;
	  	$error = '1';
	}

	// A small integrity check
	if (!file_exists('../inc/drivers/sql/driver_MySQL.php'))
	{
		$files = '<span class="error">driver_MySQL.php missing, faulty upload?</span>';
		$error = '1';
	}
	else
	{
		$files = $ok;
	}

	// Write access checks

	if (is_writable('../cache/'))
	{
		$f_cache	=	$ok;
	}
	else
	{
	  	if (is_dir('../cache/') == false)
	  	{
			mkdir('../cache/');
		}

	  	if (!is_writable('../cache/'))
	  	{
			$f_cache	=	$nw;
			$error		=	1;
		}
		else
		{
			$f_cache	=	'<span class="ok">Created!</span>';
		}
	}

	if (is_writable('../cache/temp/error.php'))
	{
		$f_error	=	$ok;
	}
	else
	{
		$f_error	=	$nw;
		$error		=	1;
	}

	if (is_writable('../inc/drivers/sql/driver_MySQL.php'))
	{
		$f_sql	=	$ok;
	}
	else
	{
		$f_sql	=	$nw;
		$error		=	1;
	}

	if (is_writable('./'))
	{
		$f_inst	=	$ok;
	}
	else
	{
		$f_inst	=	$nw;
		$error		=	1;
	}

	if (is_writable('../inc/conf.php'))
	{
		$f_conf	=	$ok;
	}
	else
	{
		$f_conf	=	$nw;
		$error		=	1;
	}

	if (is_writable('../inc/conf_extended.php'))
	{
		$f_conf_ext	=	$ok;
	}
	else
	{
		$f_conf_ext	=	$nw;
		$error		=	1;
	}

	if (is_writable('../inc/lang/1/index.php'))
	{
		$f_lang1	=	$ok;
	}
	else
	{
		$f_lang1	=	$nw;
		$error		=	1;
	}

	if (is_writable('../inc/lang/1/admin.php'))
	{
		$f_lang2	=	$ok;
	}
	else
	{
		$f_lang2	=	$nw;
		$error		=	1;
	}


	// Add more checks here if they are needed

	// And output the results
	echo '		<dl id="selftest">
			<dt>Required PHP version ( > 4.3.0)</dt>
			<dd>' . $version . '</dd>

			<dt>Required MySQL libraries</dt>
			<dd>' . $gotMySQL . '</dd>

			<dt>Uploaded: /inc/drivers/sql/driver_MySQL.php</dt>
			<dd>' . $files . '</dd>

			<dt>Writable: /cache/</dt>
			<dd>' . $f_cache . '</dd>

			<dt>Writable: /cache/temp/error.php</dt>
			<dd>' . $f_error . '</dd>

			<dt>Writable: /inc/drivers/sql/driver_MySQL.php</dt>
			<dd>' . $f_sql . '</dd>

			<dt>Writable: /install</dt>
			<dd>' . $f_inst . '</dd>

			<dt>Writable: /inc/conf.php</dt>
			<dd>' . $f_conf . '</dd>

			<dt>Writable: /inc/conf_extended.php</dt>
			<dd>' . $f_conf_ext . '</dd>

			<dt>Writable: /inc/lang/1/index.php</dt>
			<dd>' . $f_lang1 . '</dd>

			<dt>Writable: /inc/lang/1/admin.php</dt>
			<dd>' . $f_lang2 . '</dd>
		</dl>';

	// Is the FTP wizard invoked?
	$use_ftp = ifsetor($_GET['ftp_usage'], false);

	if ($use_ftp)
	{
		echo '		<h2>FTP login</h2>
		<p>The setup has analyzed your server, and found that it need to use FTP or SFTP to make some special folders and do some file changes.
		The installer will do this for you via FTP/SFTP if available, or you could do it manually. Click the skip link below if you want to use
		manual install. Automatic install is strongly recommended.</p>';

		echo <<<FORM

			<form action="install.php?page=2" method="post">
				<fieldset>
					<legend>FTP login info</legend>

					<!-- <label for="ftp_connection" class="form1">FTP connection type: </label>
						<input type="radio" id="ftp_connection" class="nofloat" name="ftp_connection" value="ftp" /> FTP
						<input type="radio" id="ftp_connection" class="nofloat" name="ftp_connection" value="sftp" /> SFTP (FTP over SSH)<br /> -->

					<label for="ftp_hostname" class="form1">FTP hostname: </label>
						<input type="text" id="ftp_hostname" name="ftp_hostname" value="ftp.{$_SERVER['HTTP_HOST']}" /> <span class="extrainfo"> You may need to change this</span> <br />
					<label for="ftp_username" class="form1">FTP username: </label>
						<input type="text" id="ftp_username" name="ftp_username" value="" /><br />
					<label for="ftp_password" class="form1">FTP password: </label>
						<input type="text" id="ftp_password" name="ftp_password" value="" /><br />
					<label for="ftp_path" class="form1">FTP path: </label>
						<input type="text" id="ftp_path" name="ftp_path" value="" /><br /><span class="extrainfo">The path to where VB is being installed, relative to FTP root</span><br />

					<input type="hidden" id="using_ftp" name="using_ftp" value="1" />

				</fieldset>

					<input type="submit" class="nolabel" value="Continue" /> <input type="reset" class="nolabel2" value="Reset" />

			</form>

			<p><a href="http://dev.vikingboard.com/wiki/index.php/Manual_install">Read about manual install</a></p>
FORM;

	}
	else
	{
		if (isset($error))
		{
			echo '<h2 id="selftest-error">Error(s) has been detected</h2>
				<p>Please correct the errors above and refresh this page. PHP is running as
					user "' . get_php_user() . '" and must have read/write access.</p>
				<p>Note that you can try repairing any file permission problems by using the built
					in FTP feature, if you want to use the FTP solution, please locate your login
					to your servers FTP area, and follow this
					<a href="install.php?ftp_usage=indeed">link</a>.</p>';
		}
		else
		{
			echo '<p class="next_step"><a href="install.php?page=2">Proceed with install &rarr;</a></p>';
		}
	}

}


/**
 *
 * Checks the FTP POST-variables, and uses FTP if set. Do the actual file altering, and
 * echo the user some info that things went fine and all is swell. Else, make error thingy and
 * tell the user to do it the hard way.
 *
 * Then move on to do the database thingies. Just echo out the form and let the database stuff be done on the next step.
 * Send the FTP variables to the next page via POST-data, since it will be writing the database file.
 *
 * @todo Extend the rm_dir()-part, so that it will completely clean old cache folders, to eliminate old reoccurring cache and exploits.
 */
function new_install()
{
	global $ftp;

	echo '<h2>Step 2: Database setup</h2>
		<p>Now we need some info about the database setup you have.<br />
		This installer is currently only for the MySQL version of the database driver,
		but check <a href="http://vikingboard.com">Vikingboard.com</a> for drivers
		for your database type.</p>

		<p>Fill out <strong>all</strong> form (database password optional)
		fields under to start populating the database. Remember to double-check the
		connection information and database name before you start.</p>

		<p>The prefix is in case you need to have two Vikingboard installs
		in the same database, or suspect Vikingboard may use some table names you
		already have in your database. If you need more than one Vikingboard install
		in the same DB, remember that the prefix needs to be unique for each install.</p>

		<p>Here you will also set up the forum administrator, which has full
		access to everything on the board. Remember to use a secure password, you really
		don\'t want people to crack your forum. :-)</p>

		<p>Please note that <strong>the passwords are not hidden</strong>
		when you type them in, so watch your back!</p>';

	// Starting by checking the FTP variables
	if (isset($_POST['using_ftp']))
	{
		$ftp_connection	=	ifsetor($_POST['ftp_connection'], false);
		$ftp_hostname	=	ifsetor($_POST['ftp_hostname'], false);
		$ftp_username	=	ifsetor($_POST['ftp_username'], false);
		$ftp_password	=	ifsetor($_POST['ftp_password'], false);
		$ftp_path		=	ifsetor($_POST['ftp_path'], false);

		if ($ftp_hostname && $ftp_username && $ftp_password && $ftp_path)
		{
			// Echoing the hidden form fields
			$hiddenfields = '<input type="hidden" id="ftp_connection" name="ftp_connection" value="' . $ftp_connection . '" />
							<input type="hidden" id="ftp_hostname" name="ftp_hostname" value="' . $ftp_hostname . '" />
							<input type="hidden" id="ftp_username" name="ftp_username" value="' . $ftp_username . '" />
							<input type="hidden" id="ftp_password" name="ftp_password" value="' . $ftp_password . '" />
							<input type="hidden" id="ftp_path" name="ftp_path value="' . $ftp_path . '" />
							<input type="hidden" id="using_ftp" name="using_ftp" value="1" />';

			// Let's do all the FTP thingies

			// Connecting
			if (!$ftp->connect($ftp_hostname))
			{
				die('<span class="error">FTP connection error, go back and try again</span>');
			}

			// Logging on
			if (!$ftp->login($ftp_username, $ftp_password))
			{
				die('<span class="error">FTP login error, do you have the right login-info?</span>');
			}

			// Do we need to chmod?
			if(substr(sprintf('%o', fileperms('../cache/')), -4) != '0777')
			{
				if (!$ftp->chmod($ftp_path . '/cache/', 0777))
				{
					die('<span class="error">FTP chmod error, do you have the right access?</span>');
				}
			}

			// Cache folders are always overwritten
			if (!$ftp->mkdir($ftp_path . '/cache/temp/') && !$ftp->mkdir($ftp_path . '/cache/' . str_replace(":", "_", $_SERVER['HTTP_HOST']) . '/'))
			{
				die('<span class="error">FTP mkdir error, do you have the right access?</span>');
			}

			// Do we have to chmod the sql-file?
			if (substr(sprintf('%o', fileperms('../inc/drivers/sql/driver_MySQL.php')), -4) != '0777')
			{
				if (!$ftp->chmod($ftp_path . '/inc/drivers/sql/driver_MySQL.php', 0777))
				{
					die('<span class="error">FTP chmod error, do you have the right access?</span>');
				}
			}
		}
		else
		{
			die('<span class="error">Please fill out all form fields. Please go back and try again.');
		}
	}
	else
	{
		// Let's do this without FTP (who needs FTP anyway?)

		// Do we have a cache folder?
		if(!is_dir('../cache/' . str_replace(":", "_", $_SERVER['HTTP_HOST'])))
		{
			// Let's make one, then
			if (!mkdir('../cache/' . str_replace(":", "_", $_SERVER['HTTP_HOST'])))
			{
				die('<span class="error">Error in file permissions. CHMOD the /cache folder to 0777 (to make absolutely sure its writable) and try again.');
			}
		}
		// Oh no, there already is a cache folder.  Let's remove it.
		else
		{
			// Well, if this doesn't work, we'll just have to leave it at that
			@rmdir('../cache/' . str_replace(':', '_', $_SERVER['HTTP_HOST']));
		}

		if(!is_dir('../cache/temp'))
		{
			if (!mkdir('../cache/temp'))
			{
				die('<span class="error">Error in file permissions. CHMOD the /cache folder to 0777 (to make absolutely sure its writable) and try again.');
			}
		}

		if (!is_writable('../inc/drivers/sql/driver_MySQL.php') && !is_writable('../cache/'))
		{
		 	if (!chmod('../inc/drivers/sql/driver_MySQL.php', 0777) && !chmod('../cache/', 0777))
			{
				die('<span class="error">Error in file permissions. CHMOD the /inc/drivers/sql/driver_MySQL.php file to 0777 (to make absolutely sure its writable) and try again.');
			}
		}

	}

	// Getting some nice variables
	$host	=	$_SERVER['HTTP_HOST'];
	$self	=	$_SERVER['PHP_SELF'];

	// Getting the forum url and removing additional stuff
	$url	=	substr('http://' . $host . $self, 0, -19);

	// Is $hiddenfields set?
	$hiddenfields	=	ifsetor($hiddenfields, false);

	// Making the form (urh, heredoc again)
	$form = <<<FORM

		<form action="install.php?page=3" method="post">
		<br />
		<fieldset>
			<legend>Connection info</legend>

			<label for="hostname" class="form1">Database hostname</label>
				<input type="text" id="hostname" name="hostname" value="localhost" /> <span class="extrainfo">Default: localhost</span><br />
			<label for="username" class="form1">Database username</label>
				<input type="text" id="username" name="username" /><br />
			<label for="password" class="form1">Database password</label>
				<input type="text" id="password" name="password" /> <span class="extrainfo">Password is in cleartext.</span><br />
			<label for="dbname" class="form1">Database name</label>
				<input type="text" id="dbname" name="dbname" /><br />
			<label for="prefix" class="form1">Database table prefix</label>
				<input type="text" id="prefix" name="prefix" value="vboard_" /><br />
		</fieldset>

		<fieldset>
			<legend>Site info</legend>

			<label for="forum_url" class="form1">Forum url</label>
				<input type="text" id="forum_url" name="forum_url" value="$url" /><br />
			<label for="homepage" class="form1">Homepage url</label>
				<input type="text" id="homepage" name="homepage" value="http://$host/" /><br />
		</fieldset>
		<fieldset>
			<legend>Administrator user</legend>

			<label for="admin_user" class="form1">Administrator username</label>
				<input type="text" id="admin_user" name="admin_user" value="administrator" /><br />
			<label for="admin_pass" class="form1">Administrator password</label>
				<input type="text" id="admin_pass" name="admin_pass" /> <span class="extrainfo">Password is in cleartext.</span><br />
			<label for="admin_email" class="form1">Admin/board email</label>
				<input type="text" id="admin_email" name="admin_email" /><br />
		</fieldset>

				$hiddenfields
		<input type="submit" class="continue" value="Continue" /> <input type="reset" class="reset" value="Reset" />
		</form>
FORM;

	echo $form;

}

/**
 * Database
 *
 * Doing all the database related stuff. Most of the installing happens here.
 * Takes FTP-info and stuff from the previous page via POST, and puts the mysql-driver via FTP if necessary.
 */
function database()
{
	global $ftp;

	//Fetching variables

	// Connection vars
	$username 	= 	ifsetor($_POST['username'], false);
	$hostname	=	ifsetor($_POST['hostname'], false);
	$password	=	ifsetor($_POST['password'], false);
	$dbname		=	ifsetor($_POST['dbname'], false);
	$prefix 	=	ifsetor($_POST['prefix'], false);

	// URLs and stuff
	$forumurl 	=	ifsetor($_POST['forum_url'], false);
	$homepage	=	ifsetor($_POST['homepage'], false);

	// Administrator account
	$admin_user	=	ifsetor($_POST['admin_user'], false);
	$admin_pass	=	ifsetor($_POST['admin_pass'], false);
	$admin_email=	ifsetor($_POST['admin_email'], false);

	// Error handling
	if (!$username || !$hostname || !$dbname || !$prefix ||
		!$forumurl || !$homepage || !$admin_user || !$admin_pass ||
		!$admin_email)
	{
		die('<h2>Error</h2> <p>You didn\'t fill out all fields. ALL input fields except the database password needs to be filled to install Vikingboard.</p>');
	}

	// @todo Expand to include all restricted mysql words (like "group")
	if (strchr($prefix, '.') || strchr($prefix, '-'))
	{
		die('<h2>Error</h2> <p>Table prefix may not contain "." or "-". This makes MySQL cry, and you don\'t want MySQL to cry, do you?</p>');
	}

	// Reading out the database login info from the driver_mySQL file
	$file	=	file('../inc/drivers/sql/driver_MySQL.php');

	$dbfile	=	'';

	// Selecting specific lines and doing stuff with them

	/**
	 * @todo Rewrite this to put DB credentials somewhere else. Messing with the
	 * 		 comments or some whitespace in driver_MySQL.php will mess up the entire
	 * 		 install now, and that is unacceptable in the long run...
	 */
	foreach ($file as $line_num => $line)
	{
		switch ($line_num)
   		{
   			case 30:
   			$dbfile	.=	"\t" . 'var $db_srv = "' . $hostname . "\";\n";
   			break;

   			case 31:
   			$dbfile	.=	"\t" . 'var $db_usr = "' . $username . "\";\n";
   			break;

   			case 32:
   			$dbfile	.=	"\t" . 'var $db_pwd = "' . $password . "\";\n";
   			break;

   			case 33:
   			$dbfile	.=	"\t" . 'var $db_db = "' . $dbname . "\";\n";
   			break;

   			case 34:
   			$dbfile	.=	"\t" . 'var $db_prefix = "' . $prefix . "\";\n";
   			break;

   			default:
   			$dbfile	.=	$line;
   			break;
   		}
	}
	echo '<h2>Step 3: Finishing up</h2>
	<dl id="db-install">';
	echo '<dt>Writing the DB connection file... </dt>';

	// Add it back into the driver file again

	// Still using FTP?
	if (isset($_POST['using_ftp']))
	{
		$ftp_connection	=	ifsetor($_POST['ftp_connection'], false);
		$ftp_hostname	=	ifsetor($_POST['ftp_hostname'], false);
		$ftp_password	=	ifsetor($_POST['ftp_password'], false);
		$ftp_path		=	ifsetor($_POST['ftp_path'], false);

		if ($ftp_connection && $ftp_hostname && $ftp_username && $ftp_password && $ftp_path)
		{
			// Let's do all the FTP thingies

			// Connecting
			if (!$ftp->connect($ftp_hostname))
			{
				die('<dd class="error">FTP connection error, go back and try again</dd>');
			}

			// Logging on
			if (!$ftp->login($ftp_username, $ftp_password))
			{
				die('<dd class="error">FTP login error, do you have the right login-info?</dd>');
			}

			// chmod
			if (!$ftp->chmod($ftp_path . '/inc/drivers/sql/driver_MySQL.php', 0777))
			{
				die('<dd class="error">FTP chmod error, do you have the right access?</dd>');
			}

			// PuTtY!
			if(!$ftp->put($dbfile, $ftp_path . '/inc/drivers/sql/driver_MySQL.php', FTP_ASCII))
			{
				die('<dd class="error">FTP put error, do you have the right access?</dd>');
			}

			// chmod back again
			if (!$ftp->chmod($ftp_path . '/inc/drivers/sql/driver_MySQL.php', 0555))
			{
				die('<dd class="error">FTP chmod error, do you have the right access?</dd>');
			}
			else
			{
				echo '<dd class="ok">Ok.</dd>';
			}

		}
	}
	else
	{
		// Let's do this without FTP (who needs FTP anyway?)
		if (file_put_contents('../inc/drivers/sql/driver_MySQL.php', $dbfile))
		{
			echo '<dd class="ok">Ok</dd>';
		}

	}

	echo '<dt>Connecting to the MySQL database...</dt>';

	// Trying to connect
	include '../inc/drivers/sql/driver_MySQL.php';

	$db = new Database(false);

	if ($db->created) // Test if connection is created
	{
		echo '<dd class="ok">Ok</dd>';
	}
	else
	{
		die('<dd class="error">Could not connect. Are you sure you have the right login info?</dd>');
	}

	// Loading the installation.sql file
	$sql	=	file_get_contents('installation.sql');

	// Splitting into single queries, not so elegantly done due to platform independence shit
	$sql	=	str_replace("\r\n","\n", $sql); //windows
	$sql	=	str_replace("\r", "\n", $sql); //macOS

	$sql	=	explode(";\n", $sql);

	// A simple function to parse and execute the installation.sql file
	function querify($sql, $prefix)
	{
		$db	=	new Database(false);

		foreach ($sql as $q)
		{
			// Fixing the prefix stuff.
			$q	=	str_replace('PREFIX|', $prefix, $q);

			// Shoot!
			$db->query($q);
		}

		if (!mysql_error())
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	// Filling the database with the installation.sql file
	echo '<dt>Trying to populate database with tables and data...</dt>';
	if (querify($sql, $prefix))
	{
		echo '<dd class="ok">Ok</dd>';
	}
	else
	{
		die('<dd class="error">Error</dd>');
	}

	// Filling it with additional custom rows (site config etc)
	echo '<dt>Populating database with additional data...</dt>';

	// Running the queries

	// Forum_config first
	$q1	=	$db->query("INSERT INTO `" . $prefix . "config`
				(	 `id`,
				 	 `forum_name`,
				 	 `forum_url`,
				 	 `forum_email`,
					 `site_homepage`,
					 `site_host`,
					 `def_tpl`,
					 `def_cat`,
					 `def_lang`,
					 `active`,
					 `show_maxtopics`,
					 `show_maxposts`,
					 `prune_topics`,
					 `timestamp1`,
					 `timestamp2`
				 ) VALUES (
				 	 '1',
					 'Vikingboard',
					 '" . $forumurl . "',
					 '" . $admin_email . "',
					 '" . $homepage . "',
					 '" . $_SERVER['HTTP_HOST'] . "',
					 '1',
					 '1',
					 '1',
					 '1',
				 	'20',
					 '50',
					 '30',
				 	'd.m.y, H:i',
					 'd.m.Y')"
			);

	// Inserting default forum
	$q2	=	$db->query("INSERT INTO `" . $prefix . "forum`
		(`id`, `name`, `descr`, `cat`, `order`,
		`lpost_title`, `lpost_tid`, `lpost_pid`, `lpost_uid`, `lpost_time`,
		`lpost_name`, `stat_threads`, `stat_posts`)
		VALUES (1, 'Vikingboard default forum', 'You may remove this later.',
		1, 1, 'Welcome to Vikingboard!', 1, 1, 2, UNIX_TIMESTAMP(),
		'" . $admin_user . "', 1, 1)");

	// Inserting default category
	$q3	=	$db->query("INSERT INTO `" . $prefix . "category`
		(`id`, `name`, `extend`, `order`) VALUES
		(1, 'Vikingboard default category', '1', '1')");

	// Inserting default start post
	$q4	=	$db->query("INSERT INTO `" . $prefix . "post`
		(`id`, `tid`, `author_name`, `author_id`, `posted`, `post`,
		`ip`, `queued`, `times_edited`, `last_edit`, `last_edit_by`)
		VALUES (1, '1', '" . $admin_user . "', '2',
		UNIX_TIMESTAMP(), 'Hi, and welcome to your new Vikingboard installation!<br /><br />
The board is now fully set up, and you may start using it. A good way to start will be to delete this post and thread and rearrange the forums and categories in a way you like.<br /><br />
The Vikingboard team.', '127.0.0.1', '0',
		'0', '', '')");

	// Inserting default start topic
	$q5	=	$db->query("INSERT INTO `" . $prefix . "topic`
		(`id`, `forum`, `cat`, `title`, `status`, `replies`, `views`,
		`sticky`, `starter_id`, `starter_name`, `started`,
		`lpost_name`, `lpost_id`, `lpost_uid`, `lpost_time`)
		VALUES (1, 1, 1, 'Welcome to Vikingboard!', 1, 0, 0, '0', 2, '" . $admin_user . "',
		UNIX_TIMESTAMP(), '" . $admin_user . "', 1, 2, UNIX_TIMESTAMP())");

	// Checking up
	if ($q1 && $q2 && $q3 && $q4 && $q5)
	{
		echo '<dd class="ok">Ok</dd>';
	}
	else
	{
		die ('<dd class="error">Error in query</dd>');
	}

	echo '<dt>Adding the administrator account...</dt>';

	// Generating password seed, security through rediculosity.
	$time	=	time();

	if ($time % 2)
	{
		$seed	=	'lol, 1337 er saa 1999, assa' . md5($_SERVER['HTTP_HOST'] . $_SERVER['HTTP_USER_AGENT'] . $time);
		$seed	.=	abs((rand(0,755) * rand(0,9) / (1/$time)));
	}
	else
	{
		$seed	=	'lol, 1337 ruler gaten saa sinsykt, assa' . md5($_SERVER['HTTP_HOST'] . $_SERVER['HTTP_USER_AGENT'] . $time);
		$seed	.=	abs((rand(0,755) * rand(0,9) / (1/$time)));
	}

	// Pad to random length
	$rnum	=	rand(100, 255);
	$seed	=	str_pad($seed, $rnum, rand(0, 1000));
	$seed	=	mysql_escape_string($seed);

	// Inserting password seed
	$q6	=	$db->query("UPDATE `" . $prefix . "config` SET  `pw_seed` = '{$seed}'");

	// Encrypting admin password
	$admin_pass	=	md5($seed . $admin_pass . md5('lol, 1337'));

	// Adding the admin user
	$q7	=	$db->query("INSERT INTO `". $prefix . "member`
				(`id`, `name`, `password`, `email`,
				`lang`, `skin_set`, `signature`, `avatar`, `posts`,
				`registered`, `location`, `interests`, `real_name`, `birthday`, `user_photo`,
				`im_networks`, `websites`, `allow_pm`, `allow_email`, `group`, `title`, `use_smiley`,
				`use_sig`, `use_emailer`, `show_sig`, `show_avatars`,
				`new_pms`, `warnlevel`, `show_pmpopup`,
				`done_pmpopup`, `memberlist_per_page`)
				VALUES ('2', '" . $admin_user . "', '" . $admin_pass . "',
				'" . $admin_email . "', 1, 1, '', '', '1', UNIX_TIMESTAMP(), '', '', '', '', '', '', '', 1, 0,
				'1', 'Board Admin', '1', '1', '1', '1', '1', 0, 0, 1, 0, 25)");

	$q8	=	$db->query("INSERT INTO `" . $prefix . "member_extra` (`mid`, `act_key`, `admin_act_waiting`)
						VALUES ('2', '', '0')");

	if ($q6 && $q7 && $q8)
	{
		echo '<dd class="ok">Ok</dd>';
	}
	else
	{
		die('<dd class="error">Error in query</dd>');
	}

	echo '<dt>Locking forum installer... </dt>';

	if ($file = fopen('install.lock', 'w'))
	{
		fwrite($file, 'lock', 4);
		fclose($file);

		chmod('install.lock', 0666);

		echo '<dd class="ok">Ok</dd>
		</dl>
		<p id="db-install-complete">Database setup is complete.</p>

		<h2>Installation complete</h2>
		<p>Although the installer is now locked (to re-install, remove the
		file "install.lock" and empty the folder "cache"), for added security,
		please remove /install/install.php before continuing.</p>';
	}
	else
	{
		echo '<dd class="error">No</dd>
		</dl>
		<h2>Installation partly complete</h2>
			<p>Could not lock the forum installer. Please either delete this file
			(/install/install.php) or make a new file named "install.lock" in
			the /install folder to lock the installer. This	is very important,
			as anyone can completely erase your entire forum if you haven\'t done this.</p>';
	}

	// DONE! Print the link back to the forum
	echo '<p><a href="' . $_POST['forum_url'] . '">Your new Vikingboard forum</a> is ready for use.</p>';
}


// Tthen let us supply a back link
$page	=	ifsetor($_GET['page'], false);
$page	=	is_numeric($page) ? $page : false;

if (($page) && $page != 3)
{
	if ($page == 2)
	{
		$page	=	'';
	}
	else
	{
		$page	=	'?page=' . ($page - 1);
	}

	echo '<p class="back"><a href="install.php' . $page . '">&larr; Previous step</a></p>';
}
?>

	</div>
	<p class="powered_by">
		<a href="http://vikingboard.com/">Vikingboard.com</a>
	</p>
</div>
</body>
</html>
