<?php

/**
 * Login page
 *
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @version 0.2.0
 *
 * $Id: login.php 1849 2007-11-24 13:30:45Z jorgenboard $
 */

//yes, we are here
define('VBOARD', __FILE__);

// Require "kernel" file
require_once('inc/kernel.php');


/**
 * Vikingboard Log* class
 */
class Login
{
	// Variables
	var $data;
	var $user;
	var $screen;
	var $config;
	var $lang;

	/**
	 * Constructor
	 */
	function Login(&$data, &$user, &$screen, &$config)
	{
		//Set vars
		$this->data		=	&$data;
		$this->user		=	&$user;
		$this->screen	=	&$screen;
		$this->config	=	&$config;
		$this->lang		=	new lang_login;
		$this->screen->set_lang('lang_login');

		//Do something
		switch(true)
		{
			case isset($_GET['login']):
			$this->do_login();
			break;

			case isset($_GET['logout']):
			$this->do_logout();
			break;

			case isset($_GET['multi_login']):
			$this->multi_login($_GET['site'], $_GET['i']);
			break;

			default:
			$this->login_page();
		}
	}


	/*
	 * Do a log-out
	 */
	function do_logout()
	{
		// Changing the sessions record to a guest only state
		$this->data->query('update_sessions_logout_user', array($this->user->session->id()));

		// Make sure that we dont have username and password stored
		setcookie("{$this->config->cookie}_username", '', time() - 10);
		setcookie("{$this->config->cookie}_password", '', time() - 10);
		setcookie("{$this->config->cookie}_hideme", '', time() - 10);

		// We need to empty the sessions table sometimes, and we are doing that here.
		$this->data->query('delete_remove_old_sessions');

		// If the board contains several sites, we need to log out from all of them.
		if ($this->config->use_multi_login)
		{
			// Get sites and loop through them
			$db_sites	=	$this->data->cachedQuery('get_sites', array());
			$db_sites	=	$db_sites[0];

			// Multi-logging out!
			$this->multi_logout($db_sites->id);
		}
		else
		{
			// Unsetting old session info
			unset($_SESSION['user']);

			// Logout complete, let`s redirect user
			if ($_SESSION['reffering_page'] != '')
			{
				redirect($_SESSION['reffering_page']);
			}
			else
			{
				redirect('index.php');
			}
		}
	}


	/**
	 * Login method, checks login data and logs the user in if everything
	 * is good to go.
	 */
	function do_login()
	{

		//First of all, are we already logged in?
		if ($this->user->id > 1)
		{
			redirect('index.php');
		}

		// Getting form data
		$username		=	ifsetor($_POST['username'], false);
		$password		=	ifsetor($_POST['password'], false);
		$remember_me	=	ifsetor($_POST['remember_me'], false);
		$hide_me		=	ifsetor($_POST['hide_me'], false);

		// Did the user put in the needed stuff?
		$link	=	'login.php';
		if (!$username)
		{
			notify($this->lang->error, $this->lang->no_username, $link);
		}
		if (!$password)
		{
			notify($this->lang->error, $this->lang->no_password, $link);
		}

		// Doing a quick check to see if the username exists in the database
		$db_logincheck	=	$this->data->query('get_finduser_login', array($username));
		$db_logincheck	=	$db_logincheck[0];

		if (!is_numeric($db_logincheck->id))
		{
			notify($this->lang->error, $this->lang->no_such_user, $link);
		}

		// Account converted?
		if ($db_logincheck->converted == 1)
		{
			$_SESSION['converted_user'] = true; // Used in lostpass
			notify($this->lang->error, $this->lang->account_converted, 'lostpass.php', $this->lang->reset_password);
		}

		// Account is locked?
		if ($db_logincheck->login_failiures >= $this->config->max_fails)
		{
			$check = (forumtime() - ($this->config->lock_time * 60));
			if ($check < $db_logincheck->last_login_fail_time)
			{
				notify($this->lang->error, $this->lang->account_freezed, $link);
			}
		}

		// Well, we found a user, then let`s see if the user got the correct password :-)

		// Encrypt the password
		$password	=	encrypt($password);
		$user		=	$this->data->query('get_info_about_user_logging_in', array($password, $db_logincheck->name));
		$user		=	$user[0];

		// The password does not seem to be correct
		if (!isset($user->id))
		{
			sleep(2);
			$this->update_account_lock_status($username);

			/** @todo: If Admin/Mod, then Notify other admins/mods (or only mail the board mail)*/
			notify($this->lang->error, $this->lang->wrong_password, $link);
		}

		// Print error in case we are in a "wrong" group.
		// When is a user placed in group 6?
		switch ($user->group)
		{
			case 6:
				notify($this->lang->error, $this->lang->account_not_activated, $link);
			break;

			case 5:
				notify($this->lang->error, $this->lang->login_validat, $link);
			break;

			case 4:
				notify($this->lang->error, $this->lang->account_banned, $link);
			break;
		}

		// Let the user in

		// Updating sessions table
		$this->data->query('update_loginuser', array($user->id, $user->group, $user->name, $hide_me, $remember_me, $this->user->session->id()));

		// Clear any failed logins
		$this->data->query('clear_failed_logins', array($user->id));

		// Add users last login to the database
		$this->data->query('set_last_login', array($user->id));

		//If we are using global-site-login we are to send the user around while we are setting cookies
		if ($this->config->use_multi_login)
		{
			// Get sites and loop through them
			$db_sites	=	$this->data->cachedQuery('get_sites', array());
			$db_sites	=	$db_sites[0];

			// Multi-logging in
			$this->multi_login($db_sites->id, $this->user->session->id(), $username, $password, $hide_me);
		}
		// If we don't use global logins, just do the simple (boring?) routine.
		else
		{
			// I am in a good mood today so here, take a cookie :) (free cookies, for everybody!)
			if ($remember_me)
			{
				// The user wants to be remebered, lets save the username and password for a half year
				setcookie("{$this->config->cookie}_username", $username, (forumtime() + 15552000));
				setcookie("{$this->config->cookie}_password", $password, (forumtime() + 15552000));
			}
			// Also set a hide me cookie?
			if ($hide_me)
			{
				setcookie("{$this->config->cookie}_hideme", 1, forumtime() + 15552000);
			}
		}

		// And then unset the old data, making the new one shine through
		unset($_SESSION['user']);


		// @todo: Remove login restrictions (set both fields to 0)


		// Well, that`s all, let`s send the user to the main page again
		if($_SESSION['reffering_page'] != '')
		{
			redirect($_SESSION['reffering_page']);
		}
		else
		{
			redirect('index.php');
		}
	}


	/**
	 * Login form functions
	 *
	 */
	function login_page()
	{
		// Lets forward users that are already logged in to the frontpage.
		if ($this->user->id != '1')
		{
			if($_SESSION['reffering_page'] != '')
			{
				redirect($_SESSION['reffering_page']);
			}
			else
			{
				redirect('index.php');
			}
		}


		$navigation	=	array
		(
			'login.php'	=>	$this->lang->login,
		);

		$this->screen->add_html('login');
		$this->screen->add_wrapper($this->lang->login, $navigation);
		$this->screen->print_page();
	}


	/**
	 * Multi-site login function. Redirects the user in a frenzy across
	 * the different login domains and sets cookies in all directions.
	 *
	 * If this breaks, you have no chance if fixing it, and your career
	 * has just ended. Now you should just hide under your desk for the
	 * rest of your life or commit kamikaze and take the server with you.
	 * If not, the universe will implode, and we wouldn't want that to
	 * happen, would we?
	 *
	 * @param int $siteid
	 * @param string $_SID
	 * @param string $_name
	 * @param string $_pwd
	 * @param int $_hide
	 */
	function multi_login($siteid, $_SID, $_name, $_pwd, $_hide)
	{
		// Vars
		$SID	=	$_GET['SID'];
		$name	=	$_GET['name'];
		$pwd	=	$_GET['pwd'];
		$hide	=	$_GET['hide'];

		// Encrypt password
		$pwd 	=	encrypt($pwd);

		// Got a starter?
		if (!$siteid)
		{
			error_msg($this->lang->no_site_id, 1);
		}

		// Passport check!
		if (!$SID)
		{
			$SID	=	$this->user->session->id();
		}

		// Stop if no SID
		if (!$SID)
		{
			error_msg($this->lang->no_sid, 1);
		}

		// Get the forum-sites
		$togo	=	$this->data->cachedQuery('get_sites', array());

		// Now we will loop through all the sites (foreach is needed, but, like, you know, whateva')
		$round	=	0;
		$start	=	$_GET['site'];

		if (!is_numeric($start))
		{
			$data0	=	$togo[0];
			$url	=	$data0->forum_url;
			header("Location: {$url}login.php?multi_login&site=" . ($round + 1) . "&SID={$SID}&ref={$config->forum_url}&name={$name}&pwd={$pwd}&hide={$hide}");
			//continue;
		}
		else
		{
			/**
			 * Give cookie to user on this site.
			 * This needs to be before the redirect, so if redirects are not supported
			 * it will at least log in on the originating site.
			 * The user will be remembered, lets save the username and password
			 */

			ob_start(); //A hack to make sure cookies are really set
			setcookie("{$this->config->cookie}_username", $name, time()+31536000);
			setcookie("{$this->config->cookie}_password", $pwd, time()+31536000);

			// Hide user?
			if ($hide)
			{
				setcookie("{$this->config->cookie}_hideme", 1, time()+31536000);
			}

			// Loop over the forum websites
			foreach ($togo as $data1)
			{
				// Work if this is the site we are asked to cookie
				if ($round == $start)
				{
					$round++;

					//Move along to next site
					$url	=	$data1->forum_url;
					header("Location: {$url}login.php?multi_login&site={$round}&SID={$SID}&ref={$_GET['ref']}&name={$name}&pwd={$pwd}&hide={$hide}");
					ob_end_flush();
				}
				//If not, check if this is the last site, in so case we are returned back to start
				else
				{
					$round++;
					if ($round == count($togo))
					{
						setcookie("{$config->cookie}_SESSION", $SID, time()+31536000);

						// If we got here, we gotta be finished :)
						redirect($_GET['ref']);
					}
				}
			}
		}
	}



	/**
	 * Updates the 'failed logins' counter on given username
	 * @var String $username
	 */
	function update_account_lock_status($username)
	{
		if (strlen($username) > 0)
		{
			$this->data->query('increase_failed_logins_counter', array($username));
		}
	}


	/**
	 * Multi-site logout function, makes shure that a user gets
	 * logged out on all the sites in the forum network.
	 *
	 * @param int $siteId
	 */
	function multi_logout($siteId)
	{
		// Changing the sessions record to a guest only state
		$this->data->query('update_sessions_logout_user', array($this->user->session->id()));

		// Make sure that we don't have username and password stored
		setcookie("{$this->config->cookie}_username", "", time() - 10);
		setcookie("{$this->config->cookie}_password", "", time() - 10);
		setcookie("{$this->config->cookie}_hideme", "", time() - 10);

		// Got a starter?
		if (!$siteId)
		{
			error_msg($this->lang->no_site_id);
		}

		$SID	=	$this->user->session->id();

		// Stop if no SID
		if (!$SID)
		{
			error_msg($this->lang->no_session_id, 1);
		}

		// Get the forum-sites
		$togo	=	$this->data->cachedQuery('get_sites', array());

		// Now we will loop through all the sites (foreach is needed, but, like, you know, whateva')
		$round	=	0;
		$start	=	$_GET['site'];

		if (!is_numeric($start))
		{
			$data	=	$togo[0];
			$url	=	$this->data->forum_url;

			header("Location: {$url}login.php?logout&site=".($round + 1)."&SID={$_SID}&ref={$config->forum_url}&name={$_name}&pwd={$_pwd}&hide={$_hide}");
			continue;
		}
		else
		{
			// Loop over the forum websites
			foreach ($togo as $data)
			{
				// Work if this is the site we are asked to cookie
				if($round == $start)
				{
					$round++;

					// Move along to next site
					$url	=	$this->data->forum_url;
					header("Location: {$url}login.php?logout&site={$round}&SID={$SID}&ref={$_GET['ref']}&name={$name}&pwd={$pwd}&hide={$hide}");
					ob_end_flush();
				}
				// If not, check if this is the last site, in so case we are returned back to start
				else
				{
					$round++;

					if ($round == count($togo))
					{
						// If we got here, we gotta be finished :)
						redirect($_GET['ref']);
					}
				}
			}
		}
	}
}

$Login	=	new Login($data, $user, $screen, $config);

?>