<?PHP

/**
 * lostpass.php - Allows a user to choose a new password if the old one is forgotten.
 * @package Vikingboard
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.1
 * @todo This does not work when $config->multi_reg is true.
 *
 * $Id: lostpass.php 1769 2007-11-08 20:01:48Z paitrakt $
 */

//yes, we are here
define('VBOARD', __FILE__);


// Require kernel
require_once('inc/kernel.php');

class lost
{
    // Define variables
    var $data;
    var $user;
    var $screen;
    var $config;
    var $lang;


    /**
     * Frontpage constructor
     */
    function lost(&$data, &$user, &$screen, &$config)
    {
        // Set up some vars
        $this->data     =   &$data;
        $this->user     =   &$user;
        $this->screen   =   &$screen;
        $this->config   =   &$config;
        $this->lang     =   new lang_lostpass;

        $this->screen->set_lang('lang_lostpass');


        // What to do?
        $mode = ifsetor($_GET['mode'], false);

        switch($mode)
        {
        	// Validating email address and sending it a confirmation key
        	case '01':
        	$this->send_key();
        	break;

        	// Validating confirmation key and letting the user choose a new password
    	    case '02':
    	    $this->new_password();
	        break;

	        // Validating password and adding it to the database
	        case '03':
	        $this->update_password();
        	break;

        	// Displaying input form allowing user to start the process of getting a new password
	        default:
        	$this->index();
	        break;
        }
    }

    /**
     * Lost password
     *
     * Displays the first step in the password retrieval process.
     *
     * @return void
     */
    function index()
    {
    	$html	=	array(
					'title'	=> $this->lang->lostpass,
					'email'	=> $this->lang->email,
					'go'	=> $this->lang->go,
				);

	    $this->screen->add_html('lostpass_frontpage', $html);

    	// Navigation and output.
	    $navigation	=	array('lostpass.php' => $this->lang->lostpass);

    	$this->screen->add_wrapper($this->lang->lostpass, $navigation);
	    $this->screen->print_page();
}


    /**
     * Send confirmation key
     *
     * Checks email and sends confirmation key.
     *
     * @return void
     */
    function send_key()
    {
	
        // Did anyone stumble in here by accident? Let's stop em then.
	    if($_POST)
    	{
		    // Get the text.
	    	$lang_email		=	new lang_emails;
		    $lang_global	=	new lang_global;

    		// Checking input...
	    	$email	=	ifsetor($_POST['email'], false);

		    // User typed in email?
    		if (!$email)
	    	{
		    	notify($this->lang->error, $this->lang->email_null, 'lostpass.php');
    		}

	    	// Is it valid?
    		if (!is_valid_email_address($email))
	    	{
		    	notify($this->lang->error, $this->lang->unvalid_email, 'lostpass.php');
    		}

	    	// Do we really have a user registered with the email address provided?
		    $user_id	=	$this->data->query('get_userid_from_email_address', array($email));
            $user_id    =   $user_id[0];
    		$user_id	=	$user_id->id;

	    	if (!is_numeric($user_id))
		    {
    			notify($this->lang->error, $this->lang->failed_email, 'lostpass.php');
	    	}

    		// Creating and inserting the confirmation key as well as the reactivation group.
	    	$conf_key	=	md5(forumtime() . $user_id . $email);
		    $this->data->query('insert_confirmation_key', array($user_id, $conf_key));

    		// Compose the mail which will be informing the user about the password changing attempt.
	    	$mail_body	=	str_replace("%reset_password_link%", $this->config->forum_url . 'lostpass.php?mode=02&key=' . $conf_key, $lang_email->lostpass_act);

		    emailer($user_id, $lang_email->tit_lostpass, $mail_body);

    		// Output text.
	    	notify($this->lang->success, $this->lang->success_email, 'lostpass.php');
    	}
    	else
	    {
		    // Stumbled indeed! Move along, nothing to see here.
    		redirect('lostpass.php');
	    }
    }

    /**
     * New password
     *
     * Checks confirmation key, asks user to input email and password * 2
     *
     * @return void
     */
    function new_password()
    {

        // Do we have a confirmation key, and is it 32 chars (md5's are always 32 chars long) long?
	    $conf_key	=	ifsetor($_GET['key'], '');
    	if (isset($conf_key) && strlen($conf_key) == 32)
	    {
		    $html	=	array(
			    'title'				=> $this->lang->lostpass,
    			'email'				=> $this->lang->email,
	    		'new_password'		=> $this->lang->new_password,
		    	'retype_password'	=> $this->lang->retype_password,
			    'confirmation_key'	=> $conf_key,
    			'go'				=> $this->lang->go,
	    	);
    
    		$this->screen->add_html('lostpass_new_pass', $html);
    
	    	// Navigation and output.
		    $navigation	=	array('lostpass.php' => $this->lang->lostpass);
    		$this->screen->add_wrapper($lang_lp->lostpass, $navigation);
	    	$this->screen->print_page();

    	}
	    else
    	{
	    	// No confirmation key, or it is not 32 chars long.
		    redirect('lostpass.php');
    	}
    }


    /**
     * Update password
     *
     * Update the users password in the database
     *
     * @return void
     */
    function update_password()
    {

	    // Did anyone stumble in here by accident? Let's stop em then.
    	if ($_POST)
	    {
		    // Get the text.
	    	$lang_global	=	new lang_global;
    
	    	// Checking input...
		    $conf_key			=	$_POST['conf_key'];
    		$email				=	$_POST['email'];
	    	$new_password		=	$_POST['newpass'];
		    $retype_password	=	$_POST['retypepass'];

    		// Is there still a confirmation key, and does it have the correct lenght?
	    	if (!$conf_key || strlen($conf_key) != 32 )
    		{
	    		notify($this->error, $this->lang->conf_key_null, 'lostpass.php?mode=02');
    		}

	    	// User typed in an email?
		    if (!$email)
    		{
	    		notify($this->error, $this->lang->email_null, 'lostpass.php?mode=02');
    		}

	    	// Is it valid?
		    if (!is_valid_email_address($email))
    		{
	    		notify($this->error, $this->lang->unvalid_email, 'lostpass.php?mode=02');
    		}

    		// Do we really have a user registered with the email address provided?
	    	$user_id	=	$this->data->query('get_userid_from_email_address', array($email));
            $user_id    =   $user_id[0];

		    if (!$user_id->id)
    		{
	    		notify($this->error, $this->lang->failed_email, 'lostpass.php?mode=02');
		    }

    		// User typed in password in both fields?
	    	if (!$new_password || !$retype_password)
		    {
    			notify($this->error, $this->lang->no_passwd, 'lostpass.php?mode=02');
	    	}

		    // User typed in the same password in both fields?
		    if ($new_password != $retype_password)
		    {
    			notify($this->error, $this->lang->passwd_un, 'lostpass.php?mode=02');
	    	}

		    // Does the confirmation key match user id?
    		$user_id	=	$user_id->id;
	    	$db_key		=	$this->data->query('get_confirmation_key', array($user_id));
            $db_key     =   $db_key[0];
		    $db_key		=	$db_key->conf_key;
		    if ($conf_key != $db_key)
		    {   
			    notify($this->error, $this->lang->failed_email_key, 'lostpass.php?mode=02');
    		}

	    	// Update to new password, remove confirmation key, and then tell the user all about it :)
		    $password	=	encrypt($new_password);
    		$this->data->query('update_password', array($user_id, $password));
	    	$this->data->query('remove_confirmation_key', array($user_id));
            
            // Get rid of the converted flag
            if(array_key_exists('converted_user', $_SESSION))
            {
                unset($_SESSION['converted_user']);
                $this->data->query('remove_converted_flag', array($user_id));
            }

		    notify($this->error, $this->lang->success_update, 'index.php', $this->lang->back_to_index);
	    }
    	else
	    {
		    // Stumbled indeed! Move along, nothing to see here.
    		redirect('lostpass.php');
	    }
    }
}
$l = new lost($data, $user, $screen, $config);
?>
