<?php
/**
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Hotstian <hotstian@users.sourceforge.net>
 * @version 0.2
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: members.php 1820 2007-11-20 01:08:11Z jorgenboard $
 */

//yes, we are here
define('VBOARD', __FILE__);

// Get the kernel file
require_once('inc/kernel.php');

/**
 * Members
 *
 * Handles the member list
 *
 */
class Members
{
	function Members(&$data, &$screen, &$config, &$user)
	{
		$this->data		= $data;
		$this->screen	= $screen;
		$this->config	= $config;
		$this->user		= $user;

		$this->lang = new lang_members;
		$this->screen->set_lang('lang_members');

		//  What to do?
		switch(1)
		{
			case isset($_GET['online']):
			$this->print_list(TRUE);
			break;

			default:
			$this->print_list();
			break;
		}
	}

	/**
	 * Print list
	 *
	 * Prints out the member list
	 *
	 * @param bool $online Show only online users
	 */
	function print_list($online = FALSE)
	{
		// Orderby Whitelist
		$orderby_list = array
		(
			'id',
			'name',
			'posts',
			'registered',
			'title',
			'group'
		);

		if (!empty($_GET['orderby']) && !empty($_GET['order']))
		{
			if (in_array($_GET['orderby'], $orderby_list))
			{
				$orderby = $_GET['orderby'];
			}
			else
			{
				$orderby	= 'id';
			}

			$order = ifsetor($_GET['order'], 'ASC');

			if ($order != 'ASC')
			{
				$order 		= 'DESC';
				$new_order	= 'ASC';
			}
			else
			{
				$new_order	= 'DESC';
			}
		}
		else
		{
			// Default
			$orderby 	= 'id';
			$order		= 'ASC';
			$new_order	= 'DESC';
		}

		//what page is requested?
		$show_from = safe_user_int('s', 0);

		// Calculating LIMITs
		if ($show_from > 0)
		{
			$show_to = ($this->user->memberlist_per_page + $show_from);
		}
		else
		{
			$show_to = $this->user->memberlist_per_page;
		}

		//fetch members
		if ($online)
		{
			$members = $this->data->cachedQuery('get_online_members_list', array($this->config->cut_time, forumtime(), $orderby, $order, $show_from, $show_to));
		}
		else
		{
			$members = $this->data->cachedQuery('get_member_list', array($orderby, $order, $show_from, $show_to));
		}

		if ($members)
		{
			$memberlist = '';

			//loop and add members to list
			$i = 0;
			foreach ($members as $member)
			{
				//formatting registered date
				$register_date = date($this->config->timestamp2, $member->registered);

				//online or offline?
				if (!$online)
				{
					if ($this->user->is_online($member->id) === true && $this->user->hide === false)
					{
						$current_status = $this->lang->online;
					}
					else
					{
						$current_status = $this->lang->offline;
					}
				}
				else
				{
					$current_status = $this->lang->online;
				}

				$userData	=	array
				(
					'name'		=> $member->name,
					'posts'		=> $member->posts,
					'reg'		=> $register_date,
					'title'		=> $member->title,
					'group'		=> $member->g_name,
					'status'	=> $current_status,
					'id'		=> $member->id
				);

				$memberlist .= $this->screen->return_html('memberlist_data', $userData);
			}
		}
		else
		{
			$data	=	array
			(
				'colspan'	=>	7,
				'empty_row'	=>	$this->lang->no_members,
			);
			$memberlist	=	$this->screen->return_html('empty_row', $data);
		}

		//Do we need several pages to load this one?
		if ($online)
		{
			// Get number of online users
			$onl_users	=	$this->data->query('get_online_users', array(forumtime() - ($this->config->cut_time * 60)));
			$onl_users 	=	$onl_users[0];
			$pages = (int)ceil($onl_users->users / $this->user->memberlist_per_page);
		}
		else
		{
			// Get number of users
			$stats = $this->data->cachedQuery('get_forum_stats', array());
			$pages = ceil($stats[0]->total_users / $this->user->memberlist_per_page);
		}

		//current page
		$showfrom = ceil($show_from / $this->user->memberlist_per_page) + 1;

		//build the pagenav
		if ($pages > 1)
		{
			//include the pagefile (:p) and construct the nav
			include_once('inc/lib/pages.php');

			// Creating the URL to use
			if (($orderby != 'id') || ($order != 'ASC'))
			{
				$pageurl = 'members.php?orderby=' . $orderby . '&order=' . $order . '&s=';
			}
			else
			{
				$pageurl = 'members.php?s=';
			}

			$pagenav = $this->lang->pages . pages($showfrom, $pages, $pageurl, $this->user->memberlist_per_page);
		}
		else
		{
			$pagenav = '';
		}

		//Wrap page and print it
		$navigation	=	array
		(
			'members.php'	=>	$this->lang->memberlist_title,
		);

		$tohtml = array
		(
			'pagenav'		=> $pagenav,
			'memberlist'	=> $memberlist,
			'online'		=> $online,
			'rev_order'		=> $new_order
		);

		//add wrappers and send it to the printers
		$this->screen->add_html('memberlist', $tohtml);
		$this->screen->add_wrapper($this->lang->memberlist_title, $navigation);

		$this->screen->print_page();
	}
}

$Members = new Members($data, $screen, $config, $user);

?>