<?php

/**
 * Messages - Handles all messaging activites. Will be vastly enhanced in 0.2->0.4
 *
 * @package Vikingboard
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.2
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: messages.php 1868 2007-11-27 20:18:43Z jorgenboard $
 */


define('VBOARD', __FILE__);
require_once('inc/kernel.php');

class Messages
{
	function Messages($data, $screen, $user, $config)
	{
		$this->data 	= $data;
		$this->screen	= $screen;
		$this->user		= $user;
		$this->config	= $config;
		$this->lang		= new lang_messages;

		$this->screen->set_lang('lang_messages');

		// Kick out the groups that are not allowed here
		switch ($this->user->group)
		{
			// Banned
			case '4':
				error_msg($this->lang->banned_user, 1);
			break;

			// Pending activation/validation
			case '5':
				error_msg($this->lang->validating_user, 1);
			break;

			// Ghost/guest
			case '6':
				redirect('login.php?from=messages.php');
			break;
		}

		switch (true)
		{
			// New message
			case (isset($_GET['new'])):
				$this->new_message();
			break;

			// Send message
			case (isset($_GET['send'])):
				$this->send_message();
			break;

			// Read message
			case (isset($_GET['read'])):
				$this->read_message();
			break;

			// Trace message
			case (isset($_GET['trace'])):
				$this->trace_message();
			break;

			/*// Put it in the Drafts folder
			case (isset($_GET['draft'])):
				$this->store_as_draft();
			break;*/

			// Inbox
			default :
				$this->inbox();
			break;
		}
	}


	/**
	 * Messages menu
	 *
	 * Displays the menu
	 */
	function view_messages_menu()
	{
		$new_messages	=	$this->data->query('get_new_messages_inbox', array($this->user->id));
		$new_messages	=	$new_messages[0]->number;

		$folders		=	$this->data->query('get_folders', array($this->user->id));

		$data	=	array
		(
			'inbox_new_messages'	=>	$new_messages,
			'folders'				=>	$folders,
		);

		return $this->screen->return_html('messages_menu', $data);
	}

	/**
	 * New message page
	 *
	 * Let's the user construct a message for sending, and passing it on to a
	 * sending function, send_message()
	 *
	 * @todo 0.3: add popup-search for recipient names
	 *
	 * @return void
	 */
	function new_message()
	{
		//parsing
		include 'inc/lib/parser.php';
		$Parser	=	new Parser($this->data, $this->screen, 1);

		//Get user info and smiley settings
		$db_uinfo	=	$this->data->query('get_user_choices', array($this->user->id));
		$db_uinfo	=	$db_uinfo[0];

		//getting smileys
		include 'inc/lib/smiley.php';

		//Are we sending to a specific user?
		$to		=	safe_user_int('to', false);
		$from	=	'';

		if ($to)
		{
			$db_usr	=	$this->data->query('get_username_from_id', array($to));
			$db_usr	=	$db_usr[0];
			$from	=	$db_usr->name;
		}

		//Are we quoting a previous message?
		$quote = safe_user_int('quote', false);

		$title = '';
		$content = '';

		if ($quote)
		{
			//first of all, are we allowed to read this?
			$quotedata	=	$this->data->query('can_read_message', array($quote, $this->user->id));
			$quotedata	=	$quotedata[0];

			if ($quotedata->belongs_to == $this->user->id)
			{
				if ($quotedata) //we don't want empty quotes
				{
					//from who?
					$from	=	$this->data->query('get_username_from_id', array($quotedata->author_id));
					$from	=	$from[0];
					$from	=	$from->name;

					//what title?
					$title	=	$this->lang->spm_re . $quotedata->title;

					//when?
					$sendtdate	=	date($this->config->timestamp1, $quotedata->date);

					//construct quote
					$quoted_message	=	$Parser->deparse($quotedata->content, false);
					$content		=	"\n[quote={$from} @ {$sendtdate}]\n{$quoted_message}\n[/quote]\n";
				}
			}
		}

		// Are we recovering from an error with sending the message?
		$sending_error = safe_user_int('failed_to_send', false);
		$error = '';
		if ($sending_error)
		{
			$error_code = safe_pos_int($_SESSION['error_code'], false);
			$error_str = 'cp_spm_error_' . $error_code;

			$error = str_replace('%num%', $_SESSION['error_input'][3], $this->lang->{$error_str});
			$error = str_replace('%allowed%', $this->config->max_smileys, $error);

			$formdata = $_SESSION['error_input'][0];
			$formdata = htmlspecialchars($formdata);
			$content = strip_tags($formdata);

			$from  = $_SESSION['error_input'][1];
			$title = $_SESSION['error_input'][2];

			unset($_SESSION['error_input'], $_SESSION['error_code']);
		}

		$cpmenu	=	$this->view_messages_menu();

		//constructing the post buttons
		include 'inc/lib/post_buttons.php';
		$post_buttons = post_buttons($this->screen);
		$this->screen->set_lang('lang_messages');

		//constructing the template array
		$html	=	array
		(
			'send_to'		=>	$from,
			'cc_to'			=>	'',
			'title'			=>	$title,
			'post-buttons'	=>	$post_buttons,
			'menu'			=>	$cpmenu,
			'smileys'		=>	show_smileylist($this->data, $this->screen),
			'message'		=>	$content,
			'error'			=>	$error,
		);

		// Fetch master template used in page
		$this->screen->add_html('messages_send', $html);

		// Navigation and title
		$navigation	=	array
		(
			'messages.php'		=>	$this->lang->messages,
			'messages.php?new'	=>	$this->lang->compose,
		);

		// Wrap up the page nicely
		$this->screen->add_wrapper($this->lang->compose, $navigation);

		// Print finished parsed page :) (voila, c'est beau!)
		$this->screen->print_page();
	}


	/**
	 * Send message page
	 *
	 * Sends the message the user has composed, and deal with it as specified.
	 *
	 * @return void
	 */
	function send_message()
	{
		//post parsing going on
		include 'inc/lib/parser.php';
		$Parser	=	new Parser($this->data, $this->screen, $this->config, 1);

		//get all data
		if (empty($_POST['to']))
		{
			error_msg($this->lang->spm_err_to, 1);
		}
		else
		{
			$to	=	htmlentities($_POST['to'], ENT_QUOTES);
		}

		if (empty($_POST['pm_title']))
		{
			error_msg($this->lang->spm_err_title, 1);
		}
		else
		{
			//get and clean the title
			$title = $_POST['pm_title'];
			$title = htmlentities($title, ENT_QUOTES);
		}

		if (empty($_POST['post']))
		{
			error_msg($this->lang->spm_err_msg, 1);
		}
		else
		{
			$message	=	$Parser->parse($_POST['post']);

			// Is there too many smileys?
			if (is_int($message[0]))
			{
				// Return error msg and put data back into the textfield
				$_SESSION['error_input'] 	= array($message[1], $to, $title, $message[0]);
				$_SESSION['error_code'] 	= 1;

				/**
				 * @todo Use $_SESSION['form_failure'] instead of URLs, to avoid
				 * 		refresh troubles. This feature should also be implemented
				 * 		in a generic function:
				 *
				 * 		form_error(array $return_data, int $error_code, [ string $url ]);
				 *
				 * 		and for recipient
				 *
				 * 		bool is_form_error();
				 *
				 * 		and
				 *
				 * 		array get_form_error($error_code);
				 */
				redirect('messages.php?new&failed_to_send=1');
			}
		}

		//are we storing a personal copy?
		$store_copy	=	safe_user_int('store_copy', 0);

		//tracing?
		$trace	=	safe_user_int('trace', 0);

		//are we sending off copies?
		$cc	=	$_POST['cc'];

		//who is it from?
		$from	=	$this->user->id;

		//who is it to?
		if (!($t	=	$this->data->cachedQuery('get_userid_from_name', array($to))))
		{
			error_msg($this->lang->spm_er03, 1);
		}

		$to	=	$t[0]->id;

		//are we copying again? Watch out for copyrights!
		//
		//Dude, this is GPL. CopyLEFT, man!
		if (!empty($cc))
		{
			//our recipient list
			$recievers	=	explode("\n", $cc);

			//too many CC's?
			if (count($recievers) > $this->config->max_pm_cc)
			{
				error_msg($this->lang->spm_err_cc . $this->config->max_pm_cc, 1);
			}

			//looping over the CC recipients
			$cc_to	=	'';
			foreach ($recievers as $reciever)
			{
				$reciever	=	rtrim($reciever);

				//non-existing recipient?
				if (!($i = $this->data->cachedQuery('get_userid_from_name', array($reciever))))
				{
					error_msg($this->lang->spm_er03, 1);
				}

				$rec_id[]	=	$i[0]->id;
				$cc_to		.= $reciever . "\n";
			}
		}

		//Now let's start sending :D
		$date	=	time();

		// Defining the folders
		define('FOLDER_INBOX', 1);
		define('FOLDER_SENT', 2);
		define('FOLDER_DRAFTS', 3);

		//putting in the data first
		if ($this->data->query('create_new_message_data', array($message, $date, $from, $cc_to)))
		{
			//get the row ID from the last query
			$msg_id = $this->data->last_insert_id();

			//putting in Sent Items folder
			if ($store_copy == '1')
			{
				$this->data->query('create_new_message', array($msg_id, $title, $date, $from, $to, FOLDER_SENT, $trace, $from));
			}

			//sending to reciever
			$this->data->query('create_new_message', array($msg_id, $title, $date, $from, $to, FOLDER_INBOX, $trace, $to));

			//updating recipients unread messages count, to notify
			$this->data->query('update_new_message', array($to));

			if (!empty($cc))
			{
				//sending CC's
				foreach ($rec_id as $rec)
				{
					$this->data->query('create_new_message', array($msg_id, $title, $date, $from, $rec, FOLDER_INBOX, $trace, $rec));
					$this->data->query('update_new_message', array($to));
				}
			}
		}

		// Enough with the nonsense, let's notify the user and return to the inbox. :)
		notify($this->lang->spm_sent, $this->lang->spm_done, 'messages.php', $this->lang->spm_to_inbox);
	}


	/**
	 * Delete a message
	 *
	 * @param int $id
	 * @return bool
	 */
	function delete_message($id)
	{
		// If there is nothing to delete, let's leave
		if ($id == false)
		{
			return false;
		}

		//are we allowed to delete?
		$allowed	=	$this->data->query('allowed_to_read_message', array($id, $this->user->id));
		$allowed	=	$allowed[0];

		if ($allowed->belongs_to == $this->user->id)
		{
			// allright, let's delete this message
			$this->data->query('delete_message', array($id));
			$this->data->query('update_message_count', array($this->user->id));

			return true;
		}
		else
		{
			error_msg($this->lang->not_alwd_del_pm, 1);

			return false;
		}
	}

	/**
	 * Move a message to the archives
	 *
	 * @param int $id
	 * @return bool
	 */
	function move_message_to_archive($id)
	{
		if (!$id)
		{
			return false;
		}

		// Is this permitted (do we own the post?)?
		$allowed	=	$this->data->query('allowed_to_read_message', array($id, $this->user->id));

		// Yes, we are the owner
		if ($allowed[0]->belongs_to == $this->user->id)
		{
			// Do the stuff
			$this->data->query('move_message_to_archive', array($id));
			return true;
		}
		else
		{
			error_msg($this->lang->not_alwd_arch_pm, 1);
			return false;
		}
	}

	/**
	 * Mark a pm read or unread
	 *
	 * @param int $id
	 * @param string $action
	 * @return void
	 */
	function mark_message($id, $action)
	{
		//are we allowed to delete?
		$allowed	=	$this->data->query('allowed_to_read_message', array($id, $this->user->id));
		$allowed	=	$allowed[0];

		if ($allowed->belongs_to == $this->user->id)
		{
			//allright, let's mark stuff
			if ($action == 'markread')
			{
				$this->data->query('mark_message_as_read', array($id));
			}
			elseif ($action == 'markunread')
			{
				$this->data->query('mark_message_unread', array($id));
			}
		}
		else
		{
			error_msg($this->lang->pm_not_yours, 1);
			return false;
		}
	}


	/**
	 * Inbox page
	 *
	 * This is where the user will administrate his messages, read his/hers messages,
	 * archive them and generally do with them as the user likes.
	 *
	 * @todo Pagination and archive-stuff. Nice links to folders.
	 * @return void
	 */
	function inbox()
	{
		$act 	= ifsetor($_GET['act'], false);
		$action = ifsetor($_POST['action'], false);

		// We are adding a new folder
		if ($act == 'new_folder')
		{
			$this->new_message_folder();

			return true;
		}

		// Are we going to delete a message?
		if ($act == 'delete')
		{
			$id = safe_user_int('id', false);
			$this->delete_message($id);
		}

		// Are we multi-deleting?
		if ($action == 'delete' && !empty($_POST['marked']))
		{
			// Looping over the PMs
			foreach ($_POST['marked'] as $marked)
			{
				$marked = safe_pos_int($marked, false);
				$this->delete_message($marked);
			}
		}

		// Are we archiving?
		if ($action == 'archive' && !empty($_POST['marked']))
		{
			// Looping
			foreach ($_POST['marked'] as $marked)
			{
				$marked = safe_pos_int($marked, false);
				$this->move_message_to_archive($marked);
			}
		}

		// Are we moving a message?
		$moved = false;
		$movedto = '';
		if (strstr($action, 'move-to') && !empty($_POST['marked']))
		{
			$to = substr($action, 8);

			foreach ($_POST['marked'] as $marked)
			{
				$marked = safe_pos_int($marked, false);

				// Doing the moving itself
				if (!$this->move_message($marked, $to))
				{
					error_msg($this->lang->inb_move_err, 1);
					$err = true;
				}
			}

			// Redirect to ?f=$to on success or not?
			if (!isset($err))
			{
				$moved = true;

				// To have a nice name to present the user
				$movedto = $this->data->cachedQuery('get_folder_info', array($to));
				$movedto = $movedto[0]->name;
			}

		}

		// Are we marking stuff?
		if ($action != 'delete' && !empty($_POST['marked']))
		{
			// Loop de loop
			foreach ($_POST['marked'] as $marked)
			{
				$marked = safe_pos_int($marked, false);
				$this->mark_message($marked, $action);
			}
		}

		// What folder are we displaying?
		$f = safe_user_int('f', 1);

		$folder = $this->data->query('get_folder_info', array($f));
		$folder = $folder[0];

		if (($folder->owner != $this->user->id) && ($folder->owner != 0))
		{
			// Show index
			$f		=	1;
			$folder	=	$this->data->query('get_folder_info', array($f));
			$folder	=	$folder[0];
		}

		// How many messages in folder?
		$db_data		=	$this->data->query('get_num_messages_folder', array($this->user->id, $f));
		$messages_tot	=	$db_data[0];


		$messages_num	=	str_replace('%messages%', $messages_tot->number, $this->lang->inb_messages);

		// How many are we allowed to have?
		$messages_max	=	$this->config->pm_max;

		// Let's get our messages!
		$messages	=	$this->data->query('get_messages_folder', array($this->user->id, $f));

		$i		=	0;
		$row	=	'';

		if ($messages)
		{
			foreach ($messages as $msg)
			{
				// Who is it from?
				$from_id	=	$msg->from_id;
				$from		=	$this->data->cachedQuery('get_username_from_id', array($from_id));
				$from		=	$from[0];

				if ($f == 2)
				{
					// If this is sent items, who is it to?
					$to_id		=	$msg->to_id;
					$to			=	$this->data->cachedQuery('get_username_from_id', array($to_id));
					$to			=	$to[0];
				}

				$inboxrow	=	array
				(
					'id'		=>	$msg->id,
					'title'		=>	$msg->title,
					'fid'		=>	$f,
					'name'		=>	($f == 2 ? $to->name : $from->name),
					'uid'		=>	($f == 2 ? $msg->to_id : $msg->from_id),
					'date'		=>	date($this->config->timestamp1, $msg->date),
					'msg_id'	=>	$msg->msg_id,
					'read'		=>	$msg->read,
				);

				// Insert data into template, and repeat a lot
				$row .= $this->screen->return_html('messages_inbox_row', $inboxrow);

				// Clean up
				unset($inboxrow);
			}
		}
		else
		{
			$row = $this->screen->return_html('empty_row', array('colspan' => 7, 'empty_row' => $this->lang->inb_no_messages));
		}

		// Constructing the "Move to folder 'foo'" elements
		$folders		=	$this->data->query('get_folders', array($this->user->id));

		// Constructing the template array
		$html	=	array
		(
			'inbox_row'			=> $row,
			'tot_msg'			=> $messages_num,
			'messages'			=> $messages_tot->number,
			'menu'				=> $this->view_messages_menu(),
			'folder_name'		=> $folder->name,
			'folders'			=> $folders,
			'fid'				=> $f,
			'moved'				=> $moved,
			'movedto'			=> $movedto
		);

		// Fetch master template used in page
		$this->screen->add_html('messages_inbox', $html);

		// Navigation and title
		$navigation['messages.php']	=	$this->lang->messages;

		if ($f == 1 || $f > 4)
		{
			$navigation["messages.php?f=1"]	=	$this->lang->inbox;
		}
		if ($f != 1)
		{
			$navigation["messages.php?f={$f}"]	=	$folder->name;
		}

		// Wrap up the page nicely
		$this->screen->add_wrapper($this->lang->inbox, $navigation);

		// Print finished parsed page :) (voila, c'est beau!)
		$this->screen->print_page();

		// Free some mem
		unset($html);

		return true;
	}


	/**
	 * Read message
	 *
	 * This page lets the user read a message and do with it as he or she
	 * pleases. Delete, archive, move, reply, forward, etc.
	 *
	 * @todo A lot
	 * @return void
	 */
	function read_message()
	{
		//what message do we want to show?
		$msg	=	safe_user_int('id', 0);

		//are we allowed to show it?
		$read	=	$this->data->query('allowed_to_read_message', array($msg, $this->user->id));
		$read	=	$read[0];

		if ($read->belongs_to != $this->user->id)
		{
			error_msg($this->lang->pm_not_yours, 1);
		}

		//getting the message data
		$msg_data	=	$this->data->query('get_message_content', array($msg));
		$msg_data	=	$msg_data[0];

		//who is it from?
		$from	=	$this->data->cachedQuery('get_username_from_id', array($msg_data->from_id));
		$from	=	$from[0];

		//when shown for the first time, we'll update the status for this.
		if ($msg_data->really_read == '0')
		{
			$this->data->query('mark_message_read', array($msg, time()));
			$this->data->query('reduce_new_message', array($this->user->id));
		}

		//let's find out some stuff about the sender
		$db_user	=	$this->data->query('get_user_stuff', array($msg_data->author_id));
		$db_user	=	$db_user[0];

		//constructing the template array
		$html	=	array
		(
			'readpm-title'			=>	$msg_data->title,
			'readpm_date'			=>	date($this->config->timestamp1, forumtime($msg_data->date)),
			'id'					=>	$msg_data->id,
			'msg_id'				=>	$msg_data->msg_id,
			'pm_data' 				=>	$msg_data->content,
			'menu'					=>	$this->view_messages_menu(),
			'user_id'				=>	$msg_data->author_id,
			'user_name'				=>	$db_user->name,
			'user_avatar'			=>	$db_user->avatar,
			'user_title'			=>	$db_user->title,
			'user_registered'		=>	date($this->config->timestamp2, forumtime($db_user->registered)),
			'user_posts'			=>	$db_user->posts,
			'user_location'			=>	$db_user->location,
			'pm_signature'			=>	$db_user->signature,
		);

		//	Fetch master template used in page
		$this->screen->add_html('messages_read', $html);

		//	Navigation and title
		$navigation	=	array
		(
			'messages.php'								=>	$this->lang->messages,
			'messages.php?f=1'							=>	$this->lang->inbox,
			"messages.php?read&amp;id={$msg_data->id}"	=>	$msg_data->title,
		);

		//	Wrap up the page nicely
		$this->screen->add_wrapper($this->lang->rpm_title, $navigation);

		//	Print finished parsed page :) (voila, c'est beau!)
		$this->screen->print_page();

		return true;
	}


	/**
	 * Move message
	 *
	 * Move a message to a different folder
	 *
	 * @param int $pm_id
	 * @param int $to_folder
	 * @return true on success
	 */
	function move_message($pm_id, $to_folder)
	{
		if (empty($pm_id) || empty($to_folder))
		{
			return false;
		}

		// Does the message belong to the user?
		$owner = $this->data->cachedQuery('allowed_to_read_message', array($pm_id, $this->user->id));

		$owner = $owner[0]->belongs_to;

		if ($owner != $this->user->id)
		{
			return false;
		}

		// Does $to_folder belong to the user?
		$folder = $this->data->cachedQuery('get_folder_info', array($to_folder));

		if ($folder[0]->owner != $this->user->id && $folder[0]->owner != 0)
		{
			return false;
		}

		// All seems to be in order, let's move it!
		$moved = $this->data->cachedQuery('move_message', array($pm_id, $to_folder));

		if ($moved)
		{
			return true;
		}
		else
		{
			// Something went horribly wrong
			return false;
		}
	}

	/**
	 * New message folder
	 *
	 */
	function new_message_folder()
	{
		if (!isset($_POST['submit']))
		{
			// Constructing the template array
			$html	=	array
			(
				'menu'				=> $this->view_messages_menu(),
			);

			// Fetch master template used in page
			$this->screen->add_html('messages_new_folder', $html);

			// Navigation and title
			$navigation	=	array
			(
				'messages.php'				=>	$this->lang->messages,
				'messages.php?f=1'			=>	$this->lang->inbox,
				'messages.php?new_folder'	=>	$this->lang->new_folder,
			);

			// Wrap up the page nicely
			$this->screen->add_wrapper($this->lang->new_folder, $navigation);

			// Print finished parsed page :) (voila, c'est beau!)
			$this->screen->print_page();

			// Free some mem
			unset($html);
		}
		else
		{
			// Get name (and sort out fishies)
			if (isset($_POST['name']) && $_POST['name'] != "")
			{
				$name = htmlentities($_POST['name']);
			}
			else
			{
				redirect('messages.php');
			}

			// Insert folder
			$this->data->cachedQuery('insert_new_folder', array($name, $this->user->id));

			// Get folderid, for correct redirection
			$f = $this->data->last_insert_id();

			redirect('messages.php?f=' . $f);
		}

		return true;
	}


	/**
	 * Archive message
	 *
	 * This page saves the PM as a draft, instead of posting it.
	 *
	 * @return void
	 */
	function store_as_draft()
	{
		echo '<pre>';
		var_dump($_POST);
		echo '</pre>';
		return true;
	}

	/**
	 * Trace message
	 *
	 * This is where the user can see if the users PMs have been read or not, or
	 * whether it got deleted or not.
	 *
	 * @return void
	 */
	function trace_message()
	{
		//are we deleting/recalling?
		if (isset($_GET['act']) && ($_GET['act'] == 'delete'))
		{
			//is there a specified ID?
			$id = safe_user_int('id', false);
			if ($id)
			{
				//do we own the PM?
				$ownership	=	$this->data->query('allowed_to_recall_message', array($id, $this->user->id));
				$test	=	$ownership[0];

				if ($test->trace == '1')
				{
					$this->data->query('recall_message', array($id));
				}
			}
		}

		//running the query
		$trace	=	$this->data->query('get_traced_messages', array($this->user->id));

		$read	=	'';
		$unread	=	'';

		//looping over the results
		if ($trace)
		{
			foreach ($trace as $t)
			{
				//a userid is not much good, so we'll find the username
				$to	=	$this->data->query('get_username_from_id', array($t->to_id));
				$to	=	$to[0];

				//and convert the date
				$date	=	date($this->config->timestamp1, $t->date);

				//read or unread?
				if ($t->really_read == '0')
				{
					$html	=	array
					(
						'name'	=>	$to->name,
						'title'	=>	$t->title,
						'date'	=>	$date,
						'id'	=>	$t->id,
					);

					$unread	.=	$this->screen->return_html('messages_trace_unreadrow', $html);
					unset($html);
				}
				else
				{
					$html	=	array
					(
						'name'	=>	$to->name,
						'title'	=>	$t->title,
						'date'	=>	$date,
					);

					$read	.=	$this->screen->return_html('messages_trace_readrow', $html);
					unset($html);
				}
			}
		}

		if (empty($read))
		{
			$read .= $this->screen->return_html('messages_no_trace');
		}
		if (empty($unread))
		{
			$unread .= $this->screen->return_html('messages_no_trace');
		}

		// constructing the template array
		$html	=	array
		(
			'menu'		=>	$this->view_messages_menu(),
			'read-row' 	=>	$read,
			'unread-row' =>	$unread,
		);

		//	Fetch master template used in page
		$this->screen->add_html('messages_trace', $html);

		//	Navigation and title
		$navigation	=	array
		(
			'messages.php'			=>	$this->lang->messages,
			'messages.php?trace'	=>	$this->lang->trace_messages,
		);

		$this->screen->add_wrapper($this->lang->trace_message_title, $navigation);
		$this->screen->print_page();

		return true;
	}
}

$Messages	=	new Messages($data, $screen, $user, $config);

?>
