<?php
/**
 * Moderator methods
 *
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2.0
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License
 * @link http://vikingboard.com/
 * @version $Id: mod.php 1816 2007-11-20 00:03:34Z jorgenboard $
 *
 */

define('VBOARD', __FILE__);
require_once('inc/kernel.php');

/**
 * Moderator Control Panel
 *
 */
class Mod
{
	/**
	 * Mod Constructor
	 *
	 * @param Data $data
	 * @param User $user
	 * @param Screen $screen
	 * @param Config $config
	 * @return Mod
	 */
	function Mod(&$data, &$user, &$screen, &$config)
	{
		// Set up some vars
		$this->data		=	&$data;
		$this->user		=	&$user;
		$this->screen	=	&$screen;
		$this->config	=	&$config;
		$this->lang		=	new lang_mod;

		// Define constants
		define('TOPIC_CLOSED', 0);
		define('TOPIC_OPEN', 1);
		define('TOPIC_STICKY', 1);
		define('TOPIC_UNSTICK', 0);

		// Do something
		switch ($_GET['act'])
		{
			case 'delete':
				$this->delete_post(safe_user_int('post', false));
			break;

			case 'close':
				$this->change_topic_status_frontend(TOPIC_CLOSED);
			break;

			case 'open':
				$this->change_topic_status_frontend(TOPIC_OPEN);
			break;

			case 'sticky':
				$this->change_sticky_frontend(TOPIC_STICKY);
			break;

			case 'unstick':
				$this->change_sticky_frontend(TOPIC_UNSTICK);
			break;

			default:
				$this->mod_index();
			break;
		}
	}

	/**
	 * Moderator Control Panel
	 *
	 * Front index, for easy moderation and maintainance of the board
	 *
	 * This is @todo for 0.3/0.4
	 */
	function mod_index()
	{
	}

	/**
	 * Delete post
	 *
	 * Deletes a single post (or topic if the post is #1 in topic). Checks if the user got sufficent rights to delete the post/topic first.
	 *
	 * @todo We really should use POST for all the "active" actions, that is,
	 * everything that make a change on the server should use POST. This will
	 * avoid someone tricking an admin to do something stupid by just using a
	 * false URL. POST for initial stuff, and then use sessions for confirmation,
	 * as an attacker otherwise may use POST just as he would use GET to fool an admin.
	 * @param int post
	 * @return true if success, false if failure
	 */
	function delete_post($id)
	{
		if (!$id)
		{
			return false;
		}

		// Get information about the post
		$sql		=	$this->data->query('get_post_information', array($id));
		$sql		=	$sql[0];
		$ownedby	=	$sql->author_id;
		$forum		=	$sql->forum;

		// Finding which topic and if it's the first post in the topic
		$topic		=	$sql->tid;
		$first_post =	$this->data->query('first_post_in_topic', array($topic));
		$first_post	=	$first_post[0]->id;

		$confirmed 	=	ifsetor($_GET['confirmed'], false);

		// Delete whole topic if post is #1 in topic (let's also ask first)
		if ($first_post == $id)
		{
			if ($confirmed == 'yes')
			{
				// Make sure that the user is allowed to delete topics, and delete it if so
				if (($this->user->g_can_delete_own_topics && ($this->user->id == $ownedby)) || ($this->user->g_is_admin || $this->user->g_is_mod))
				{
					$this->data->query('delete_topic', array($topic));
					$this->data->query('reduce_forum_topic_count', array($forum));
					$num = $this->data->query('count_posts_in_topic', array($topic));	//Count number of posts to delete
					$num = $num[0]->total;

					$this->data->query('delete_posts_in_topic', array($topic)); //Then actually delete the posts
					$this->data->query('reduce_forum_post_count_dyn', array($forum, $num)); //and reduce the forum postcount

					// Get latest topic
					$lasttopic = $this->data->query('get_last_topic', array($forum));
					$lasttopic = $lasttopic[0];

					// And then update lastpost stats
					$this->data->query('update_last_topic', array($forum, $lasttopic->title, $lasttopic->id, $lasttopic->lpost_id, $lasttopic->lpost_uid, $lasttopic->lpost_time, $lasttopic->lpost_name));

					$topic_deleted	=	$this->lang->topic_deleted;
				}
				// User do not have the sufficent rights to delete a topic
				else
				{
					$error	=	$this->lang->cant_delete_topic;
				}
			}
			else
			{
				confirm($this->lang->confirm_delete_topic_header, $this->lang->confirm_delete_topic, array($this->lang->yes => "mod.php?act=delete&amp;post={$id}&amp;confirmed=yes", $this->lang->no => "topic.php?t={$topic}"));
			}

		}
		// Post was not #1 in topic, just delete that single post
		else
		{
			if ($confirmed == 'yes')
			{
				// Make sure that the user is allowed to delete posts, and delete it if so
				if (($this->user->g_can_delete_own_posts && ($this->user->id == $ownedby)) || ($this->user->g_is_admin || $this->user->g_is_mod))
				{
					$this->data->query('delete_post', array($id));
					$this->data->query('reduce_topic_reply_count', array($topic));
					$this->data->query('reduce_forum_post_count', array($forum));

					//and then revert lastpost stats
					$lastpost	=	$this->data->query('get_last_post', array($topic));
					$lastpost	=	$lastpost[0];

					$this->data->query('update_topic_last_post', array($lastpost->author_name, $lastpost->id, $lastpost->author_id, $lastpost->posted, $topic));
					$this->data->query('update_forum_last_post', array($lastpost->id, $lastpost->author_id, $lastpost->posted, $lastpost->author_name, $forum));
					$post_deleted	=	$this->lang->post_deleted;
				}
				else
				{
					$error	=	$this->lang->cant_delete;
				}
			}
			else
			{
				confirm($this->lang->confirm_delete_post_header, $this->lang->confirm_delete_post, array($this->lang->yes => "mod.php?act=delete&amp;post={$id}&amp;confirmed=yes", $this->lang->no => "topic.php?t=$topic"));
			}
		}

		// Post was deleted
		if (empty($error) && $post_deleted)
		{
			notify($this->lang->post_deleted_header, $post_deleted, "topic.php?t=$topic", $this->lang->go_back_topic);
			return true;
		}
		// Topic was deleted
		elseif (empty($error) && $topic_deleted)
		{
			notify($this->lang->topic_deleted_header, $topic_deleted, "forum.php?f=$forum", $this->lang->go_back_forum);
			return true;
		}
		// Nothing was deleted
		else
		{
			error_msg($error, 1);
			return false;
		}
	}

	/**
	 * Change topic status
	 *
	 * @access private
	 * @param int $tid
	 * @param int $status
	 * @return bool
	 */
	function change_topic_status($tid, $status)
	{
		// Input check
		if (!$tid)
		{
			return false;
		}
		if (!isset($status))
		{
			return false;
		}

		// Check for suffient rights
		if (!($this->user->g_is_mod || $this->user->g_is_admin))
		{
			return false;
		}

		// Do the closing
		$closed = $this->data->query('change_topic_status', array($tid, $status));

		// And then return data
		if ($closed)
		{
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Frontend for change_topic_status(), for direct user contact (GET variables)
	 *
	 * @param int $status
	 */
	function change_topic_status_frontend($status)
	{
		// Get vars
		$tid 	= safe_user_int('tid', false);

		// Open/close it
		if ($tid)
		{
			$success = $this->change_topic_status($tid, $status);
		}
		else
		{
			error_msg($this->lang->no_topic_selected, 1);
		}

		// Let's go back
		if ($success)
		{
			// Go back to where you were (the topic)
			redirect('topic.php?t=' . $tid);
		}
		else
		{
			error_msg($this->lang->could_not_change_status, 1);
		}
	}

	/**
	 * Change a topic's sticky status (on or off)
	 *
	 * @access private
	 * @param int $tid
	 * @param int $sticky
	 */
	function change_sticky($tid, $sticky)
	{
		// Making sure everything's safe
		$tid = safe_pos_int($tid, false);

		if ($tid === false || ($sticky > 1))
		{
			return false;
		}
		else
		{
			$success = $this->data->query('change_topic_sticky', array($tid, $sticky));

			// Let's go home
			if ($success)
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}

	/**
	 * Frontend for change_sticky()
	 *
	 * @param int $status
	 */
	function change_sticky_frontend($status)
	{
		// Get vars
		$tid 	= safe_user_int('tid', false);

		// Open/close it
		if ($tid)
		{
			$success = $this->change_sticky($tid, $status);
		}
		else
		{
			error_msg($this->lang->no_topic_selected, 1);
		}

		// Let's go back
		if ($success)
		{
			// Go back to where you were (the topic)
			redirect('topic.php?t=' . $tid);
		}
		else
		{
			error_msg($this->lang->could_not_change_sticky, 1);
		}
	}

}

$Mod	=	new Mod($data, $user, $screen, $config);
?>