<?php
/**
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2
 *
 * $Id: post.php 1856 2007-11-25 18:49:49Z ueland $
 *
 * @todo 	- Stop users from quoting the above post when replying?
 * 			- Use per-topic ID's when quoting etc., to avoid users
 * 			  putting in id=*id_from_hidden_forum* and stuff
 */

//yes, we are here
define('VBOARD', __FILE__);


//Kernel ftw!
require_once('inc/kernel.php');
require_once('inc/lib/parser.php');

/**
 * Post class, handles all topic and post posting
 *
 */
class Post
{
	// Define variables
	var $data;
	var $user;
	var $screen;
	var $config;
	var $lang;
	var $parser;


	/**
	 * Frontpage constructor
	 */
	function Post(&$data, &$user, &$screen, &$config, &$session)
	{
		// Set up some vars
		$this->data		=	&$data;
		$this->user		=	&$user;
		$this->screen	=	&$screen;
		$this->config	=	&$config;
		$this->session	=	&$session;
		$this->lang		=	new lang_post;
		$this->screen->set_lang('lang_post');

		switch(true)
		{
			case (isset($_POST['post'])) :
				$this->post_a_post();
			break;

			case (isset($_GET['new_topic']) && safe_user_int('forum', false)) :
				$this->todo	=	'new_topic';
				$this->show_post_page();
			break;

			case (isset($_GET['reply']) && safe_user_int('topic', false)) :
				$this->todo	=	'new_reply';
				$this->show_post_page();
			break;

			case (isset($_GET['edit']) && safe_user_int('post', false)) :
				$this->todo	=	'edit_post';
				$this->show_post_page();
			break;

			case (isset($_GET['multiquote']) && safe_user_int('post', false)) :
				$this->multiquote();
			break;

			case (isset($_GET['quick_preview'])) :
				$this->parse_quick_preview();
			break;

			default :
				redirect('index.php');
		}
	}


	/**
	 * Checks permissions
	 * @todo Everybody can edit others posts
	 */
	function check_permissions($post_author_id = false)
	{
		// Check if the user is allowed to create new posts/topics
		if (!$this->user->g_create_new_posts && ($this->todo == 'new_reply' || $this->todo == 'new_topic'))
		{
			// Nope
			// Redirect if it is a guest
			if ($this->user->id == 1)
			{
				redirect('login.php');
			}
			else
			{
				// Or display a error message if not
				if ($this->todo == 'new_reply')
				{
					error_msg($this->lang->not_permission_to_post);
				}
				else
				{
					error_msg($this->lang->not_permission_to_start_topic);
				}
			}
		}

		// If we are editing, some more checks are needed
		if ($this->todo == 'edit_post' || $this->todo == 'edit_topic')
		{
			// Guests are not allowed to edit any posts
			if ($this->user->id == 1)
			{
				redirect('login.php');
			}
			// Check if the post belong to the user
			elseif ($post_author_id == $this->user->id)
			{
				// Check if the user is allowed to edit his own posts
				if (!$this->user->g_can_edit_own_posts)
				{
					error_msg($this->lang->not_permission_to_edit);
				}
			}
			else
			{
				// The post does not belong to the user, so let's check if he is an admin or a mod
				if (!$this->user->g_is_admin || !$this->user->g_is_mod)
				{
					error_msg($this->lang->not_permission_to_edit);
				}
			}
		}
	}


	/**
	 * The post page, where the user inputs all kinds of stuff etc
	 */
	function show_post_page()
	{
		$this->Parser	=	new Parser($this->data, $this->screen, $this->config);

		// Set input data
		$forum_id	=	safe_user_int('forum', false);
		$topic_id	=	safe_user_int('topic', false);
		$post_id	=	safe_user_int('post', false);
		$quote_id	=	safe_user_int('quote', false);

		// Get Topic ID if we only have a post ID to go after
		if(!$topic_id && $post_id > 0)
		{
			$topic_id	=	$this->data->cachedQuery('get_post_information', array($post_id));
			$topic_id	=	$topic_id[0]->id;
		}

		// Get Forum ID if we dont have it
		if(!$forum_id && $topic_id > 0)
		{
			$forum_id	=	$this->data->cachedQuery('get_topic_information', array($topic_id));
			$forum_id	=	$forum_id[0]->fid;
		}

		if ($this->todo == 'edit_post')
		{
			$first_post	=	$this->data->cachedQuery('get_first_post_in_topic', array($topic_id));
			$first_post	=	$first_post[0]->id;

			// Fetch the post we are editing, if we are editing
			$post			=	$this->data->cachedQuery('get_post_information', array($post_id));
			$post			=	$post[0];

			// Deparse the post
			$post_content	=	$this->Parser->deparse($post->post, false);

			// Check if this is the first post in the topic
			if ($first_post == $post_id)
			{
				$this->todo	=	'edit_topic';
				$post_title	=	$post->title;
			}

			$this->check_permissions($post->author_id);
		}
		else
		{
			$this->check_permissions();
			$post_content	=	'';
		}

		// Get some information about the forum and category we are in
		$forum		=	$this->data->cachedQuery('get_forum_info', array($forum_id));
		$forum		=	$forum[0];
		$category	=	$this->data->cachedQuery('get_category_info', array($forum->cat));
		$category	=	$category[0];

		// Start on the navigation
		$navigation["index.php?c={$category->id}"]	=	$category->name;
		$navigation["forum.php?f={$forum_id}"]		=	$forum->name;

		if ($this->todo == 'edit_topic' || $this->todo == 'edit_post' || $this->todo == 'new_reply')
		{
			if (!isset($post_title))
			{
				$post_title	=	$this->data->cachedQuery('get_topic_title', array($topic_id));
				$post_title	=	$post_title[0]->title;
			}
			$navigation["topic.php?t={$topic_id}"]	=	$post_title;
		}

		// Choose a title and the last bit of the navigation
		switch($this->todo)
		{
			case 'edit_topic' :
				$title	=	$post_title;
				$navigation["post.php?edit&amp;post={$post_id}"]	=	$this->lang->edit_topic;
			break;

			case 'edit_post':
				$title	=	$this->lang->edit_post;
				$navigation["post.php?edit&amp;post={$post_id}"]	=	$title;
			break;

			case 'new_topic':
				$title	=	$this->lang->new_topic;
				$navigation["post.php?new_topic&amp;forum={$forum_id}"]	=	$title;
			break;

			case 'new_reply':
				$title	=	$this->lang->new_reply;
				$navigation["post.php?reply&amp;topic={$topic_id}"]	=	$title;
			break;
		}

		// Let's get started on the quoting

		// A single quoted post
		if($quote_id)
		{
			$quoted_post =	$this->data->cachedQuery('get_post_information', array($quote_id));
			$quoted_post =	$quoted_post[0];

			$post_content	.=	'[quote=' . $quoted_post->author_name . "]\n" . $this->Parser->deparse($quoted_post->post, false) . "\n[/quote]\n";
		}

		// Multiquoted posts
		$multiquote	=	ifsetor($_COOKIE[$this->config->cookie . '_multiquote'], false);
		if($multiquote && ($this->todo == 'new_reply' || $this->todo == 'new_topic'))
		{
			// Make sure the posts are placed in the correct order, and that we are within the allowed quote
			$quoted_posts	=	array_reverse(explode('_', $multiquote, $this->config->max_quotes));
			$quoted	=	'';

			foreach($quoted_posts as $multiquote_id)
			{
				if(is_numeric($multiquote_id))
				{
					$quoted	.=	"$multiquote_id, ";
				}
			}

			// Remove the last ', '
			$quoted	=	substr_replace($quoted, '', -2, 2);

			// Look up in DB and add to post_content
			$quoted_posts	=	$this->data->query('get_quoted_posts', array($quoted, $this->config->max_quotes));
			foreach($quoted_posts as $quoted_post)
			{
				$post_content	.=	'[quote=' . $quoted_post->name . "]\n" . $this->Parser->deparse($quoted_post->post, false) . "\n[/quote]\n";
			}

			// Remove cookie info
			setcookie($this->config->cookie . '_multiquote', '', -3600);
		}

		// If this is an already existing topic, we can check for topic subscriptions
		if ($this->todo == 'edit_post' || $this->todo == 'edit_topic' || $this->todo == 'new_reply')
		{
			$subscription	=	$this->data->cachedQuery('get_topic_subscription_from_user', array($topic_id, $this->user->id));
		}

		if (($this->todo == 'new_reply' || $this->todo == 'new_topic') && $this->user->id != '1')
		{
			$guest_privelegies	=	$this->data->cachedQuery('get_guest_privelegies');
			$post_as_guest		=	($guest_privelegies[0]->g_create_new_posts) ? true : false;
		}
		else
		{
			$post_as_guest	=	false;
		}

		// Load needed libraries
		require_once('inc/lib/post_buttons.php');
		require_once('inc/lib/smiley.php');

		// Let`s build the page
		$data	=	array
		(
			'todo'			=>	$this->todo,
			'title'			=>	$title,
			'user_is_guest'	=>	($this->user->id == '1') ? true : false,
			'post_title'	=>	ifsetor($post_title, false),
			'post_content'	=>	ifsetor($post_content, false),
			'post_buttons'	=>	post_buttons($this->screen),
			'smileys'		=>	show_smileylist($this->data, $this->screen),
			'poll'			=>	(isset($_GET['poll']) && ($first_post == $post_id)) ? true : false,
			'subscribed'	=>	ifsetor($subscription[0], false),
			'post_as_guest'	=>	$post_as_guest,
			'category'		=>	$category->id,
			'forum_id'		=>	$forum_id,
			'sid'			=>	$this->session->id(),
			'user_id'		=>	$this->user->id,
			'post_id'		=>	ifsetor($post_id, false),
			'topic_id'		=>	ifsetor($topic_id, false),
		);

		$this->screen->set_lang('lang_post'); //The post_buttons are overriding language.
		$this->screen->add_html('post_body', $data);
		$this->screen->add_wrapper($title, $navigation);
		$this->screen->print_page();
	}


	/**
	 * Posts something
	 */
	function post_a_post()
	{
		$todo 			=	ifsetor($_POST['do'], false);
		$sid		 	=	ifsetor($_POST['sid'], false);
		$post 			=	ifsetor($_POST['post'], false);
		$show_smileys 	=	ifsetor($_POST['enable_smileys'], false);
		$subscribe		=	ifsetor($_POST['subscribe'], false);
		$post_as_guest	=	ifsetor($_POST['post_as_guest'], false);

		$topic_id 		=	safe_user_int('topic_id', false);

		$forum_id 		=	safe_user_int('forum_id', false);
		$cat_id 		=	safe_user_int('category_id', false);
		$post_id 		=	safe_user_int('post_id', false);

		$title			=	ifsetor($_POST['title'], false);
        $title          =   strip_magic_quotes_gpc($title);
		$title 			=	htmlentities($title, ENT_QUOTES);

		$poll_title 	=	ifsetor($_POST['pollquestions'], false);
		$poll_title		=	htmlentities($poll_title, ENT_QUOTES);
		$poll_questions	=	ifsetor($_POST['poll_questions'], false);
		$poll_questions =	htmlentities($poll_questions, ENT_QUOTES);

		//Set "todo"
		switch($todo)
		{
			case 'new_topic' :
				$this->todo =	'new_topic';
			break;

			case 'new_reply' :
				$this->todo	=	'new_reply';
			break;

			case 'edit_post' :
				$this->todo	=	'edit_post';
			break;

			case 'edit_topic' :
				$this->todo	=	'edit_topic';
			break;

			default:
				$this->todo = 'new_topic';
		}

		//Check to see if we got what we need
		if (!$sid)
		{
			error_msg($this->lang->missing_data, 1);
		}

		if (!$title && ($this->todo == 'new_topic' || $this->todo == 'edit_topic'))
		{
			error_msg($this->lang->missing_title, 1);
		}

		//We DO need a post (by the way, Weird Al rules!)
		if (!$post || strlen($post) < 1)
		{
			error_msg($this->lang->missing_post, 1);
		}

		//If we are polling, make shure we are allowed to
		if (strlen($poll_questions) > 1 && strlen($poll_questions) > 1)
	   	{
			if ($this->user->g_create_new_polls == 1)
			{
				$do_poll	=	1;
			}
			else
			{
				error_msg($this->lang->cant_poll, 1);
			}
		}
		else
		{
			$do_poll	=	0;
		}


		//Can we post?
		if ($this->user->g_create_new_posts == 0 && ($this->todo == 'new_topic' || $this->todo == 'new_reply'))
	   	{
			error_msg($this->lang->cant_post, 1);
		}

        // Get topic info
        $topic_info = $this->data->cachedQuery('get_topic_status', array($topic_id));
        $topic_info = $topic_info[0];

        // Closed topic?
		if (($topic_info->status != 1) && (!$this->user->g_is_mod) && (!$this->user->g_is_admin))
		{
			error_msg($this->lang->cant_post_closed, 1);
		}

        // Forum id not set?
        if(is_numeric($topic_info->forum) && is_numeric($forum_id) == false) {
            $forum_id = $topic_info->forum;
        }

		//Check edit ability if doing a post edit
		if ($this->todo == 'edit_post' || $this->todo == 'edit_topic')
		{
			//Got post ID?
			if (!$post_id)
			{
				error_msg($this->lang->missing_data, 1);
			}
			$owner	=	$this->data->cachedQuery('get_post_owner', array($post_id));
			$owner	=	$owner[0]->id;

			//If you are not mod..
			if ($this->user->g_is_mod == 0)
			{

				//Can user edit own posts?
				if ($sql == $this->user->id && $this->user->g_can_edit_own_posts == 0)
			   	{
					error_msg($this->lang->cant_edit, 1);

				//This is not "your" post?
				}
				elseif ($sql != $this->user->id)
				{
					error_msg($this->lang->cant_edit, 1);
				}
			}
		}

		//Magic key check (Isn't this already tackled above? - jorgis)
		if (strlen($sid) < 1)
		{
			error_msg($this->lang->missing_data, 1);
		}

		//Parse poll if we have any
		if ($do_poll == 1)
		{
			$poll_questions = nl2br($poll_questions);
			$poll_questions = explode('<br />', $poll_questions);
			if (count($poll_questions) < 1)
			{
				$do_poll	=	0;
			}
		}

		//Do post parsing
		$this->Parser	=	new Parser($this->data, $this->screen, $this->config, $show_smileys);
		$post			=	$this->Parser->parse($post);

		//DO DATABASE INSERT STUFF HERE
		$lang_email	=	new lang_emails;

		//Update a edited post
		if ($this->todo == 'edit_post')
		{
			$this->data->query('update_post_data', array($post_id, $this->user->id, $post));
		}

		if ($this->todo == 'edit_topic')
		{
			$this->data->query('update_topic_data', array($post_id, $this->user->id, $title, $post));
		}

		if ($this->todo == 'new_reply' || $this->todo == 'new_topic')
		{
			$guest_privelegies	=	$this->data->cachedQuery('get_guest_privelegies');
			if ($post_as_guest && $guest_privelegies[0]->g_create_new_posts)
			{
				$poster_id		=	'1';
				$poster_name	=	$this->lang->guest;
			}
			else
			{
				$poster_id		=	$this->user->id;
				$poster_name	=	$this->user->name;
				$post_as_guest	=	false;
			}
		}

		//Create a new topic
		if ($this->todo == 'new_topic')
		{
			//Save topic
			$this->data->query('insert_newtopic', array($forum_id, $cat_id, $title, $poster_id, $poster_name));

			$sql		=	$this->data->query('get_newest_topicid_from_user', array($poster_id));
			$sql		=	$sql[0];
			$topic_id	=	$sql->id;

			//Save post
			$this->data->query('insert_new_post', array($topic_id, $poster_name, $poster_id, $post, get_remote_address()));

			$sql	=	$this->data->query('get_new_post_id', array($poster_id, $topic_id));
			$sql	=	$sql[0];

			//Update statistics
			if (!$post_as_guest)
			{
				$this->data->query('update_user_postcount', array($this->user->id));
			}
			$this->data->query('update_stats_on_newtopic', array($forum_id, $title, $topic_id, $sql->id, $poster_name, $poster_id));
			$this->data->query('update_global_stats', array('topics'));
			$this->data->query('update_global_stats', array('posts'));

			//Get forum subscribers
			$subscriptions		=	$this->data->query('get_subscriptions_in_forum', array($forum_id));
			if (is_array($subscriptions))
			{
				$email_title	=	str_replace('%topic_title%', $title, $lang_email->new_topic_title);
				$email_message	=	str_replace('%forum_id%', $forum_id, $lang_email->subscribe_new_topic);
				/* @todo Add forum name */
				$email_message	=	str_replace('%forum_name%', $forum_name, $email_message);
				$email_message	=	str_replace('%topic_id%', $topic_id, $email_message);
				$email_message	=	str_replace('%topic_title%', $title, $email_message);
				foreach ($subscriptions as $s)
				{
					emailer($s->uid, $email_title, $email_message);
				}
			}
		}

		//Reply to a topic
		if ($this->todo == 'new_reply')
		{
			//Save post
			$this->data->query('insert_new_post', array($topic_id, $poster_name, $poster_id, $post, get_remote_address()));

			//Get post ID
			$sql	=	$this->data->query('get_new_post_id', array($poster_id, $topic_id));
			$sql	=	$sql[0];

			//Get topic`s name
			$topic_name	=	$this->data->query('get_topic_title', array($topic_id));
			$topic_name	=	$topic_name[0];

			//Get topic subscribers
			$subscriptions	=	$this->data->query('get_topic_subscribers', array($topic_id));
			if (is_array($subscriptions))
			{
				$email_title	=	str_replace('%topic_title%', $title, $lang_email->new_topic_reply_title);
				$email_message	=	str_replace('%topic_id%', $topic_id, $lang_email->subscribe_new_reply);
				$email_message	=	str_replace('%topic_title%', $title, $email_message);

				foreach ($subscriptions as $s)
				{
					emailer($s->uid, $email_title, $email_message);
				}
			}

			//updating stats
			if (!$post_as_guest)
			{
				$this->data->query('update_user_postcount', array($this->user->id));
			}
			$this->data->query('update_forum_stats_on_reply', array($forum_id, $topic_id, $sql->id, $poster_name, $poster_id, $topic_name->title));
			$this->data->query('update_topic_stats_on_reply', array($topic_id, $sql->id, $poster_name, $poster_id));
			$this->data->query('update_global_stats', array('posts'));
		}

		//Update polls?
		if ($do_poll == 1 && ($this->todo == 'edit_post' || $this->todo == 'edit_topic'))
		{
			$this->data->query('update_poll_title', $array($topic_id, $poll_title));

			$i	=	0;
			foreach ($poll_questions as $foo)
			{
				$this->data->query('update_poll_question', array($topic_id, $foo, $i));
				$i++;
			}
		}

		// Insert new poll
		if ($do_poll == 1 && $this->todo == 'new_topic')
		{
			$this->data->query('insert_new_poll_title', array($topic_id, $poll_title));

			foreach ($poll_questions as $foo)
			{
				$this->data->query('insert_new_poll_question', array($topic_id, $foo));
			}
		}

		// Update topic subscription
		if (!$post_as_guest)
		{
			if ($this->todo == 'new_topic' && $subscribe)
			{
				$this->data->query('add_topic_subscription', array($topic_id, $this->user->id));
			}
			elseif ($todo == 'edit_post' || $todo == 'edit_topic' || $todo == 'new_reply')
			{
				$subscription	=	$this->data->query('get_topic_subscription_from_user', array($topic_id, $this->user->id));
				if ($subscribe && !isset($subscription[0]))
				{
					$this->data->query('add_topic_subscription', array($topic_id, $this->user->id));
				}
				elseif (!$subscribe && isset($subscription[0]))
				{
					$this->data->query('remove_topic_subscription', array($topic_id, $this->user->id));
				}
			}
		}

		// Done!
		// And...oh yes, im white and nerdy!
		redirect("topic.php?t={$topic_id}&lastpost");
	}


	/**
	 * Multiquote
	 *
	 * Adds or removes post to multiquote cookie
	 * @param int post_id
	 * @return void
	 */
	function multiquote()
	{
		$post	=	$_GET['post'];
		$cookie	=	ifsetor($_COOKIE["{$this->config->cookie}_multiquote"], '');

		// Is the post already quoted?
		if (strstr($cookie, "_{$post}"))
		{
			// Remove post
			$new_cookie	=	str_replace("_{$post}", '', $cookie);
		}
		else
		{
			// Add post
			$new_cookie	=	$cookie . "_{$post}";
		}

		setcookie("{$this->config->cookie}_multiquote", $new_cookie);

		// Off we go :)
		if($_SESSION['reffering_page'] != '')
		{
			redirect($_SESSION['reffering_page']);
		}
		else
		{
			redirect("topic.php?post={$post}");
		}
	}

	function parse_quick_preview()
	{
		$content	=	ifsetor($_POST['post_content'], false);

		if ($content)
		{
			$this->Parser	=	new Parser($this->data, $this->screen, $this->config);
			
			if (isset($_GET['deparse']))
			{
				$content	=	$this->Parser->deparse($content, false);
			}
			else
			{
				$content	=	$this->Parser->parse($content, false);
			}

			echo $this->screen->return_html('quick_reply_preview', array('content' => $content));
		}

	}
}

$Post = new Post($data, $user, $screen, $config, $session);

?>
