<?php

/**
 * @name Profile
 *
 * @package Vikingboard
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2.0
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: profile.php 1863 2007-11-27 19:22:05Z paitrakt $
 */

define('VBOARD', __FILE__);

require_once('inc/kernel.php');

class Profile
{
	var $data;
	var $user;
	var $screen;
	var $config;
	var $lang;
	var $parser;

	/**
	 * Constructor
	 *
	 * @return Constructor
	 */
	function Profile(&$data, &$user,& $screen, &$config)
	{
		$this->data		=	&$data;
		$this->user		=	&$user;
		$this->screen	=	&$screen;
		$this->config	=	&$config;
		$this->lang		=	new lang_profile;
		$this->screen->set_lang('lang_profile');

		$user_id	=	safe_user_int('u', false);

		// Seconds in ...
		define('HOUR', 3600);
		define('DAY', 86400);

		// Showtime!
		$this->show_profile($user_id);
	}


	/**
	* Show profile
	*
	* @param int user_id
	* @return void
	*/
	function show_profile($user_id)
	{
		$user	=	$this->data->query('get_profile_data', array($user_id));
		$user	=	$user[0];

		// Check if there is a user, and that it's not a guest
		if (!$user || $user_id == 1)
		{
			error_msg($this->lang->no_user, 1);
		}

		// Getting total number of forum posts to calculate users percentage of total
		$stats	=	$this->data->query('num_forum_posts');
		$stats	=	$stats[0];

		$posts			= $this->lang->num_posts;
		$total_percent 	= round(($user->posts / $stats->total_posts) * 100, 2);

		// This check should never be passed (I can't see how someone can get
		// more than the total number of posts posted on his/hers user..)
		if ($total_percent > 100)
		{
			$total_percent	=	100;
		}

		$days_registered	=	ceil((forumtime() - $user->registered) / DAY);
		$per_day 			=	round($user->posts / $days_registered, 2);

		$posts	=	str_replace('%posts%', $user->posts, $posts);
		$posts	=	str_replace('%percent_forum_total%', $total_percent, $posts);
		$posts	=	str_replace('%posts_per_day%', $per_day, $posts);

		// Check if user has been online the last config->cut_time minutes
		$is_online = $this->user->is_online($user_id);

		// Has logged on sometime
		if (is_numeric($is_online))
		{
			$last_seen		=	str_replace
			(
				'%last_seen%',
				date($this->config->timestamp1, forumtime($is_online)),
				$this->lang->last_seen
			);
		}

		if ($user->birthday)
		{
			$age	=	str_replace('%age%', $user->age, $this->lang->age);
		}

		$websites	=	false;
		if ($user->websites)
		{
			$website	=	explode(';', $user->websites);
			foreach($website as $w)
			{
				$split = explode('|', $w);

				$w_truncated	=	truncate_word($split[1]);
				$website_data	=	array
				(
					'link'	=>	$split[0],
					'text'	=>	$w_truncated,
				);
				$websites		.=	$this->screen->return_html('profile_website_link', $website_data);
			}
		}

		$last_posted_in	=	false;
		$last_posts		=	$this->data->query('get_last_posts', array($user_id, 10));
		foreach($last_posts as $post)
		{
			$post_data	=	array
			(
				'link'	=>	"topic.php?t={$post->id}",
				'text'	=>	$post->title,
			);
			$last_posted_in	.=	$this->screen->return_html('list_item_link', $post_data);
		}

		$im_networks		=	false;
		$config_networks	=	explode('|', $this->config->im_networks);
		$user->im_networks	=	explode('|', $user->im_networks);

		for($i = 0; $i < count($config_networks); $i++)
		{
			if (!empty($user->im_networks[$i]))
			{
				$icon = str_replace(' ', '-', $config_networks[$i]);
				$icon = str_replace('!', '', $icon);
				$icon = strtolower($icon);

				$data	=	array
				(
					'network'		=>	$config_networks[$i],
					'address'		=>	$user->im_networks[$i],
					'network_icon'	=>	$icon
				);

				$im_networks	.=	$this->screen->return_html('im_network', $data);
			}
		}

		/**
		 * @todo Fix emailer
		 */
		$profile_title	=	str_replace('%username%', $user->name, $this->lang->users_profile);



		// Find the total time spent on this board
		$time_online			=	$this->data->cachedQuery('get_time_online', array($user->id));
		$time_online			=	$time_online[0]->time_online;
		$hours_online			=	round($time_online / (HOUR), 2);
		$hours_per_day			=	round($hours_online / ((forumtime() - $user->registered) / (DAY)), 2);

		$time_online			=	str_replace('%total%', $hours_online, $this->lang->time_online);
		$time_online_per_day	=	str_replace('%day%', $hours_per_day, $this->lang->time_online_per_day);

		// Only show online time to the user and the admins
		if (($this->user->id != $user->id) && !$this->user->g_is_admin)
		{
			$time_online = false;
		}

		$data	=	array
		(
			'profile_is_self'		=>	(($this->user->id == $user->id) ? true : false),
			'user_is_logged_in'		=>	(($this->user->id != 1) ? true : false),
			'users_profile'			=>	$profile_title,
			'username'				=>	$user->name,
			'user_id'				=>	$user->id,
			'title'					=>	$user->title,
			'avatar'				=>	(($user->avatar) ? $user->avatar : false),
			'avatar_text'			=>	str_replace('%username%', $user->name, $this->lang->avatar),
			'posts'					=>	$posts,
			'registered_date'		=>	date($this->config->timestamp1, forumtime($user->registered)),
			'registered_time_ago'	=>	human_time($user->registered),
			'online'				=>	($is_online === true && $user->hide === false),
			'last_seen'				=>	((isset($last_seen)) ? $last_seen : $this->lang->never_logged_on),
			'time_online'			=>	$time_online,
			'time_online_per_day'	=>	$time_online_per_day,
			'personal_info'			=>	(($user->real_name || $user->location || $user->birthday != '0000-00-00' || $websites) ? true : false),
			'real_name'				=>	(($user->real_name) ? $user->real_name : false),
			'location'				=>	(($user->location) ? $user->location : false),
			'birthday'				=>	(($user->birthday != '0000-00-00') ? $user->birthday : false),
			'birthday_num_years'	=>	(($user->birthday) ? $age : false),
			// @todo 'email'					=>	(($user->use_emailer) ? true : false),
			'email'					=>	false,
			'websites'				=>	(($websites) ? $websites : false),
			'im_networks'			=>	(($im_networks) ? $im_networks : false),
			'last_posted_in'		=>	(($last_posted_in) ? $last_posted_in : false),
		);

		$navigation	=	array
		(
			"profile.php?u={$user->id}"	=>	$profile_title,
		);

		$this->screen->add_html('profile', $data);
		$this->screen->add_wrapper($profile_title, $navigation);
		$this->screen->print_page();
	}
}

$Profile	=	new Profile($data, $user, $screen, $config);

?>
