<?php

/**
 * Settings - Handles all User settings. Will be vastly enhanced in 0.2->0.4
 *
 * @package Vikingboard
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @version 0.2
 * @copyright Copyright (c) 2004-2007 The Vikingboard Project
 * @license GPL {@link http://www.gnu.org/licenses/gpl.html}
 * @link http://vikingboard.com/
 *
 * $Id: settings.php 1885 2007-12-03 00:16:43Z jorgenboard $
 */

/**
 * Define VBOARD, used for security checks in the kernel
 * Kernel is loaded afterwards.
 */
define('VBOARD', __FILE__);
require_once('inc/kernel.php');

class Settings
{
	function Settings($data, $screen, $user, $config)
	{
		$this->data 	= $data;
		$this->screen	= $screen;
		$this->user		= $user;
		$this->config	= $config;
		$this->lang		= new lang_settings;

		$this->screen->set_lang('lang_settings');

		// Kick out the groups that are not allowed here
		switch ($this->user->group)
		{
			// Banned
			case '4':
			error_msg($this->lang->error_banned, 1);
			break;

			// Pending activation/validation
			case '5':
			error_msg($this->lang->error_inactive, 1);
			break;

			// Ghost/guest
			case '6':
			redirect('login.php?from=settings.php');
			break;
		}

		switch ($_GET)
		{
			// Settings Index
			case (isset($_GET['index'])):
				$this->index();
			break;

			// Profile edit and update
			case (isset($_GET['profile'])):
				$this->profile();
			break;

			case (isset($_GET['update_profile'])):
				$this->update_profile();
			break;
/*
			// Avatar edit and update
			case (isset($_GET['avatar'])):
				$this->avatar();
			break;

			case (isset($_GET['update_avatar'])):
				$this->update_avatar();
			break;
*/
			// Signature edit and update
			case (isset($_GET['signature'])):
				$this->signature();
			break;

			case (isset($_GET['update_signature'])):
				$this->update_signature();
			break;

			// Forum subscriptions
			case (isset($_GET['forum_subscriptions'])):
				$this->forum_subscriptions();
			break;

			// Topic subscriptions
			case (isset($_GET['topic_subscriptions'])):
				$this->topic_subscriptions();
			break;

			// Forum settings
			case (isset($_GET['forum_settings'])):
				$this->forum_settings();
			break;

			default:
				$this->index();
			break;
		}
	}

	/**
	 * Settings menu
	 *
	 * Displays the menu in Settings.
	 */
	function view_settings_menu()
	{
		return $this->screen->return_html('settings_menu');
	}

	/**
	 * Settings mainpage
	 *
	 * This is the settings index and is shown as a default
	 */
	function index()
	{
		// Getting some nice user info and avatar info from db
		$profile		=	$this->data->query('get_profile_data', array($this->user->id));
		$profile		=	$profile[0];

		$avatar			=	$this->data->query('get_user_avatar', array($this->user->id));
		$avatar			=	$avatar[0];
		$current_avatar	=	$avatar->avatar;

		$signature		=	$this->data->query('get_user_signature', array($this->user->id));
		$signature		=	$signature[0];
		$current_signature		=	$signature->signature;

		$settings_menu	=	$this->view_settings_menu();

		$html	=	array
		(
			'menu'					=>	$settings_menu,
			'avatar_image'			=>	(empty($current_avatar) ? 0 : 1),
			'current_avatar'		=>	$current_avatar,
			'signature'				=>	$current_signature,
			'location_data'			=>	$profile->location,
			'homepage_data'			=>	$profile->homepage,
			'email_data'			=>	$profile->email,
			//'msn_data'				=>	$profile->im_msn,
			'msn_data'				=> '',
			'interests_data'		=>	$profile->interests
		);

		$this->screen->add_html('user_cp_frontpage', $html);

		// Add the wrapper
		$navigation	=	array
		(
			'settings.php'			=>	$this->lang->settings,
			'settings.php?index'	=>	$this->lang->frontpage
		);

		$this->screen->add_wrapper($this->lang->frontpage, $navigation);

		// Print page
		$this->screen->print_page();

		return true;
	}


	/**
	* Profile setup page
	*
	* Lets the user change his/her profile information
	*/
	function profile()
	{
		$user	=	$this->data->query('get_profile_data', array($this->user->id));
		$user	=	$user[0];

		// IM Networks
		$im_networks	=	explode('|', $user->im_networks);

		$im = ''; $i = 0;
		foreach ($im_networks as $im_account)
		{
			if (!empty($im_account))
			{
				$im .= $this->screen->return_html('settings_profile_im', array('i' => $i, 'im' => $im_account));
			}
			$i++;
		}
		$im	.=	 $this->screen->return_html('settings_profile_im', array('i' => false, 'im' => false));

		// Websites
		$websites = explode(';', $user->websites);

		$web = ''; $i = 0;
		foreach ($websites as $website)
		{
			if (!empty($website))
			{
				$website = explode('|', $website);
				$web .= $this->screen->return_html('settings_profile_website', array('i' => $i, 'url' => $website[0], 'descr' => $website[1]));
			}
		}
		$web .= $this->screen->return_html('settings_profile_website', array('i' => $i, 'url' => $website[0], 'descr' => $website[1]));

		// Birthday
		$birthday = explode('-', $user->birthday);
		$birthday = date_dropdown(&$this->screen, $birthday[2], $birthday[1], $birthday[0]);

		$vars	=	array
		(
			'menu' 				=>	$this->view_settings_menu(),
			'avatar'			=>	$user->avatar,
			'remote_avatar'		=>	($user->avatar ? $user->avatar : ''),
			'upload_avatar'		=>	false,
			'no_avatar'			=>	false,
			'im'				=>	$im,
			'real_name'			=>	$user->real_name,
			'location'			=>	$user->location,
			'websites'			=>	$web,
			'birthday_dropdown'	=>	$birthday,
			'allow_pm'			=>	$user->allow_pm,
			'allow_email'		=>	$user->allow_email,
			'im_networks'		=>	$im_networks,
		);

		$this->screen->add_html('settings_profile', $vars);

		// Add the wrapper
		$navigation	=	array(
			'settings.php'			=>	$this->lang->settings,
			'settings.php?profile'	=>	$this->lang->menu_profile
		);

		$this->screen->add_wrapper($this->lang->menu_profile, $navigation);

		// Print page
		$this->screen->print_page();

		return true;
	}


	/**
	 * Profile update page
	 *
	 * Updates the user profile. Not a real page, no data is outputted.
	 * @todo Fix slashes problem if you use a quotation mark in your name.
	 */
	function update_profile()
	{
		// Define different IM-IDs
		define('AIM', 0);
		define('GTALK', 1);
		define('IRC', 2);
		define('JABBER', 3);
		define('SKYPE', 4);
		define('MSN', 5);
		define('YAHOO', 6);
		define('OTHER', 7);

		if (!empty($_POST['submit']))
		{
			$real_name 	= $_POST['real_name'];
			$location 	= $_POST['location'];

			// Check validity of birthdate
			$birthday = safe_pos_int($_POST['day'], 1);
			if ($birthday > 31)
			{
				$birthday 	= 01;
			}

			$birthmonth = safe_pos_int($_POST['month'], 1);
			if ($birthmonth > 12)
			{
				$birthmonth = 01;
			}

			$birthyear = safe_pos_int($_POST['year'], 1970);
			if ($birthyear < 1900)
			{
				$birthyear = 1970;
			}

			// Compiling string in valid MySQL DATE() format
			$birth_string = "{$birthyear}-{$birthmonth}-{$birthday}";

			// Compiling IM-networks array
			$network_array = array(AIM => '', GTALK => '', IRC => '', JABBER => '', SKYPE => '', MSN => '', YAHOO => '', OTHER => '');

			foreach ($_POST['network'] as $id => $network)
			{
				$network_array[$network] = str_replace('|', '', $_POST['name'][$id]);
			}

			$im_networks = implode('|', $network_array);

			// Compiling website array
			$websites = '';

			foreach ($_POST['website'] as $id => $url)
			{
				$url = str_replace('|', '', $url);
				$url = str_replace(';', '', $url);

				$descr = str_replace('|', '', $_POST['websitedescr'][$id]);
				$descr = str_replace(';', '', $descr);

				$websites .= $url . '|' . $descr . ';';
			}

			$allow_pm = safe_user_int('pm', 0);

			// Avatar
			$avatar	=	ifsetor($_POST['remote_avatar'], false);

			if ($avatar)
			{
				// Prefix properly if needed
				if (!strstr($avatar, 'http://'))
				{
					$avatar = 'http://' . $avatar;
				}

				// Check validity
				if (!is_valid_url($avatar))
				{
					$avatar = '';
				}

				// Getting avatar size.
				$size	=	getimagesize($avatar);

				// How large can it be?
				$max					=	$this->config->avatar_size;
				list($max_x, $max_y)	=	split ('x', $max);
				$Max_size				=	$max_x * $max_y;
				$x						=	$size[0];
				$y						=	$size[1];

				$avatarsize	=	$x * $y;
				$filetype	=	$size['mime'];

				// Checking the image, if it really is a image
				if (stristr($filetype, 'image/') == FALSE )
				{
					error_msg($this->lang->avatar_err_not_img, 1);
				}

				// and if it's not too big.
				if ($avatarsize > $Max_size || $x > $max_x || $y > $max_y)
				{
					error_msg(str_replace('%pixels%', $this->config->avatar_size, $this->lang->avatar_err_too_big), 1);
				}
			}

			// It appears everything has gone well, or has at least been taken care of
			$this->data->query('update_profile_data', array($this->user->id, $avatar, $real_name, $location, $birth_string, $websites, $im_networks, $allow_pm));

		}

		//return, and display a notice
		redirect('settings.php?profile&updated=1');
	}


	/**
	* Signature displaying page
	*
	* Displays the users signature
	* @todo Include a option in Admin CP for choosing wether smileys are allowed or not
	*/
	function signature()
	{
		//including the parser
		include 'inc/lib/parser.php';
		$Parser	=	new Parser($this->data, $this->screen, 1);

		//getting signature info
		$db_sign	=	$this->data->query('get_user_signature', array($this->user->id));
		$db_sign	=	$db_sign[0];

		//doing the menu
		$settings_menu	=	$this->view_settings_menu();

		$max_signature_length = str_replace('%length%', $this->config->max_signature, $this->lang->signature_max_length);

		//making the template array
		$vars	=	array
		(
			'menu'				=>	$settings_menu,
			'no_signature'		=>	empty($db_sign->signature),
			'signature'			=>	$db_sign->signature,
			'signature_edit'	=>	$Parser->deparse($db_sign->signature, TRUE),
			'signature_max_length'		=>	$max_signature_length
		);

		$this->screen->add_html('settings_signature', $vars);

		// Adding the wrapper
		$navigation	=	array
		(
			'settings.php' => $this->lang->settings,
			'settings.php?signature' => $this->lang->menu_signature
		);

		$this->screen->add_wrapper($this->lang->signature, $navigation);

		// And printing the page
		$this->screen->print_page();

		return true;
	}


	/**
	* Signature updating page
	*
	* Updates the users signature
	*/
	function update_signature()
	{
		//in-clud-i-ng
		include 'inc/lib/parser.php';
		$Parser	=	new Parser($this->data, $this->screen, 1);

		//parsing it good
		$signature	=	$Parser->parse($_POST['new_sig']);

		if (strlen($signature) > $this->config->max_signature)
		{
			notify($this->lang->menu_signature, $this->lang->signature_too_long, 'settings.php?signature', $this->lang->go_back);
		}

		// Into the DB it goes
		$this->data->query('update_user_signature', array($this->user->id, $signature));

		// And returning right away
		redirect('settings.php?signature');
	}


	/**
	 * Forum subscriptions
	 *
	 * Lets the user see a list of the forums he or she subsribes to
	 */
	function forum_subscriptions()
	{
		// Let's get the menu
		$settings_menu	=	$this->view_settings_menu();

		$id			=	safe_user_int('id', 0);
		$lang_global=	new lang_global;

		// Do the user want to remove a subscription?
		if (isset($_GET['act']) && ($_GET['act'] == 'remove'))
		{
			$forums	=	$_POST['forums'];

			if (empty($forums) && $id != 0)
			{
				$forums[]	=	$id;
			}

			$lang_global	=	new lang_global;

			// Selected anything?
			if (!empty($forums))
			{
				// Construct forum list
				$forums = implode(', ', $forums);

				$this->data->query('remove_forum_subscriptions', array($this->user->id, $forums));

				notify($htis->lang->forum_subs_title, $this->lang->forum_subs_removed, 'settings.php?forum_subscriptions');
			}
			// No forums
			else
			{
				notify($this->lang->forum_subs_title, $this->lang->forum_subs_no_selected, 'settings.php?forum_subscriptions');
			}
		}
		// Nope, just show the subscriptions
		else
		{
			// Get all the forums the user is subscribing to
			$subscr	=	$this->data->query('get_forum_subscriptions', array($this->user->id));

			$i		=	0;
			$rows	=	'';

			// Looping through the subscriptions
			if ($subscr)
			{
				foreach ($subscr as $sub)
				{
					// Get forum subscription info
					$forum	=	$this->data->query('get_forum_subscr_info', array($sub->fid));
					$forum	=	$forum[0];

					// Is there a last post in the forum?
					$last_post	=	'';
					if ($forum->lpost_tid)
					{
						$last_post	=	$lang_global->last_post_medium;

						$last_post	=	str_replace('%topic_title%', $forum->lpost_title, $last_post);
						$last_post	=	str_replace('%topic_id', $forum->lpost_tid, $last_post);
						$last_post	=	str_replace('%author_id%', $forum->lpost_uid, $last_post);
						$last_post	=	str_replace('%author%', $forum->lpost_name, $last_post);
						$last_post	=	str_replace('%time%', date($this->config->timestamp1, forumtime($forum->lpost_time)), $last_post);
					}

					// Adding output to array
					$forum_subscr_row	=	array
					(
						'forum-id'			=>	$sub->fid,
						'forum-title'		=>	$forum->name,
						'forum-topics'		=>	$forum->stat_threads,
						'forum-posts'		=>	$forum->stat_posts,
						'forum-last-post'	=>	$last_post
					);

					// Output of one forum row
					$rows	.=	$this->screen->return_html('settings_forum_subscriptions_row', $forum_subscr_row);

					$i++;
				}
			}
			// No subscriptions?
			else
			{
				$html = array(	'colspan' 	=> 7,
								'empty_row' => $this->lang->forum_subs_no_subscriptions
							 );
				$rows		.=	$this->screen->return_html('empty_row', $html);
			}

			// Main output array
			$html	=	array
			(
				'menu'				=>	$settings_menu,
				'forums'			=>	$rows,
				'subscriptions'		=>	$i
			);

			// Aaand, let me present you ... the output!
			$this->screen->add_html('settings_forum_subscriptions', $html);

			// Navigation
			$navigation	=	array
			(
				'settings.php'						=>	$this->lang->settings,
				'settings.php?forum_subscriptions'	=>	$this->lang->forum_subs_title
			);

			// Wrapper and print :)
			$this->screen->add_wrapper($this->lang->forum_subs_title, $navigation);
			$this->screen->print_page();
		}

		return true;
	}


	/**
	 * Topic subscriptions
	 *
	 * Lets the user see a list of the topic he or she subsribes to
	 */
	function topic_subscriptions()
	{
		// Let's get the menu
		$settings_menu	=	$this->view_settings_menu();

		// Do the user want to remove a subscription?
		if (isset($_GET['act']) && ($_GET['act'] == 'remove'))
		{
			$topics	=	$_POST['topics'];

			if (!$topics)
			{
				$topics[]	=	safe_user_int('id', 0);
			}

			// Selected anything?
			if ($topics)
			{
				$this->data->query('remove_topic_subscriptions', array($this->user->id, $topics));
				notify($this->lang->topic_subs_removed, 'settings.php?topic_subscriptions');

				return true;
			}
			// No topics
			else
			{
				/**
				 * @todo This is an error message, and as such it should convey an easily spottable
				 * different meaning than notify(). I suggest to use error_msg() with backlinks for this
				 * kind of stuff.
				 */
				notify($this->lang->topic_subs_no_selected, 'settings.php?topic_subscriptions');
			}
		}
		// Nope, just show the subscriptions
		else
		{
			// Get all the topics the user is subscribing to
			$subscr	=	$this->data->query('get_topic_subscriptions', array($this->user->id));

			$i		=	0;
			$rows	=	'';

			// Getting the global language string, we want the last post :)
			$lang_global	=	new lang_global;

			// Looping through the subscriptions
			if ($subscr)
			{
				foreach ($subscr as $sub)
				{
					$i++;

					// Get topic info
					$topic	=	$this->data->cachedQuery('get_topic_subscr_info', array($sub->tid));
					$topic	=	$topic[0];

					// Is there a last post in the topic?
					$last_post	=	'';

					if ($topic->lpost_pid)
					{
						$last_post	=	$lang_global->last_post_short;

						$last_post	=	str_replace('%topic_id', $topic->lpost_tid, $last_post);
						$last_post	=	str_replace('%author_id%', $topic->lpost_uid, $last_post);
						$last_post	=	str_replace('%author%', $topic->lpost_name, $last_post);
						$last_post	=	str_replace('%time%', date($this->config->timestamp1, forumtime($topic->lpost_time)), $last_post);
					}

					// Adding output to array
					$topic_subscr_row	=	array
					(
						'topic-id'			=>	$sub->tid,
						'topic-title'		=>	$topic->title,
						'topic-replies'		=>	$topic->replies,
						'topic-views'		=>	$topic->views,
						'topic-last-post'	=>	$last_post,
					);

					// Output of one topic row
					$rows	.=	$this->screen->return_html('settings_topic_subscriptions_row', $topic_subscr_row);
				}
			}
			// No subscriptions?
			else
			{
				$html = array(	'colspan'	=> 	7,
								'empty_row'	=>	$this->lang->topic_subs_no_subscriptions
							 );
				$rows		.=	$this->screen->return_html('empty_row', $html);
			}

			// Main output array
			$html	=	array
			(
				'menu'				=>	$settings_menu,
				'topics'			=>	$rows,
				'num_subscriptions'	=>	$i
			);

			$this->screen->add_html('settings_topic_subscriptions', $html);

			// Navigation
			$navigation	=	array
			(
				'settings.php'						=>	$this->lang->settings,
				'settings.php?topic_subscriptions'	=>	$this->lang->topic_subs_title
			);

			$this->screen->add_wrapper($this->lang->topic_subs_title, $navigation);
			$this->screen->print_page();
		}

		return true;
	}

	/**
	 * Update avatar
	 */
	function update_avatar()
	{
		//are we even doing a change?
		if (!empty($_POST['new_avatar']))
		{
			$new_avatar	=	htmlspecialchars($_POST['new_avatar'], ENT_NOQUOTES);

			//Getting picture size.
			$size	=	getimagesize($new_avatar);

			//how large can we be?
			$max		=	$this->config->avatar_size;
			list ($max_x, $max_y)	=	split ('x', $max);
			$Max_size	=	$max_x * $max_y;
			$x			=	$size[0];
			$y			=	$size[1];

			$avatarsize	=	$x * $y;
			$filetype	=	$size['mime'];

			// Checking the image, if it really is a image
			if (stristr($filetype, 'image/') !== FALSE )
			{
				// and if it's not too big.
				if ($avatarsize <= $Max_size && $x <= $max_x && $y <= $max_y)
				{
					$this->data->query('update_user_avatar', array($this->user->id, $new_avatar));

					//And returning right away
					redirect('settings.php?avatar');
				}
				else
				{
					error_msg(str_replace('%pixels%', $this->config->avatar_size, $this->lang->cp_avatar_err_toobig), 1);
				}
			}
			else
			{
				error_msg($this->lang->cp_avatar_err_notimg, 1);
			}
		}
		else
		{
			redirect('settings.php?avatar');
		}
	}

	/**
	 * Forum Settings
	 *
	 * Handles all user customizable settings, like theme choices,
	 * language choices and posts per page etc.
	 *
	 */
	function forum_settings()
	{
		// Have we submitted?
		if (isset($_POST['submit']))
		{
			$theme 	= safe_user_int('theme', false);
			$lang	= safe_user_int('lang', false);

			if ($theme && $lang)
			{
				if (($theme == $this->user->skin_set) && ($lang == $this->user->lang))
				{
					$updated = $this->lang->forum_settings_no_changes;
				}
				else
				{
					// Save the changes in the database
					$this->data->query('update_user_settings', array($lang, $theme, $this->user->id));

					// Kill the $user cache
					$this->user->reload();

					notify($this->lang->menu_forum_settings, $this->lang->forum_settings_saved);
				}
			}
			else
			{
				error_msg($this->lang->forum_settings_invalid_input, 1);
			}
		}

		// Find all themes
		$themes = $this->data->cachedQuery('get_forum_themes');
		$theme_dropdown = '';

		foreach ($themes as $theme)
		{
			$selected = ($theme->id == $this->user->skin_set) ? 'selected="selected"' : '';

			$html	=	array
			(
				'value'		=> $theme->id,
				'selected'	=> $selected,
				'name'		=> $theme->name,
			);

			$theme_dropdown .= $this->screen->return_html('dropdown_option', $html);
			unset($html);
		}

		// Find all languages
		$langs = $this->data->cachedQuery('get_forum_languages');
		$lang_dropdown = '';

		foreach ($langs as $lang)
		{
			$selected = ($lang->id == $this->user->lang) ? 'selected="selected"' : '';

			$html	=	array
			(
				'value' 	=> $lang->id,
				'selected' 	=> $selected,
				'name'		=> $lang->language,
			);

			$lang_dropdown .= $this->screen->return_html('dropdown_option', $html);
		}

		// Show form
		$html	=	array
		(
			'menu' 			=> $this->view_settings_menu(),
			'lang_options' 	=> $lang_dropdown,
			'theme_options'	=> $theme_dropdown,
			'updated' 		=> (isset($updated)) ? $updated : '',
		);

		$this->screen->add_html('settings_forum_settings', $html);

		$navigation = array
		(
			'settings.php'				=> $this->lang->settings,
			'settings.php?forum_settings'	=> $this->lang->menu_forum_settings,
		);

		$this->screen->add_wrapper($this->lang->menu_forum_settings, $navigation);

		$this->screen->print_page();
	}
}

$Settings = new Settings($data, $screen, $user, $config);

?>
