/**
 * vikingboard.js
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @author Various others, specified in source
 * @package Vikingboard
 * @version 0.2.0
 *
 * $Id: general.js 1872 2007-11-27 20:47:10Z paitrakt $
 */


/**
 * Dollar function, aka getElementById
 *
 * Short for getElementById, pluss some more
 * @author Prototype JavaScript Framwork,  Sam Stephenson <http://www.prototypejs.org/>
 * @param string, object
 * @return array elements
 */
function $()
{
	var elements = new Array();
	for (var i = 0; i < arguments.length; i++)
	{
		var element = arguments[i];
		if (typeof element == 'string')
			element = document.getElementById(element);
		if (arguments.length == 1)
			return element;
		elements.push(element);
	}
	return elements;
}
Object.prototype.$	=	$;

/**
 * Double dollar function, aka getElementsByClassName
 *
 * @author Jonathan Snook <http://www.snook.ca/jonathan>
 * @author Robert Nyman <http://www.robertnyman.com>
 * @param string strTagName
 * @param string strClassName
 * @return array
 */
function $$(strTagName, strClassName)
{
	strClassName	=	strClassName.replace(/\-/g, "\\-");

	var arrElements			=	(strTagName == "*" && this.all) ? this.all : this.getElementsByTagName(strTagName);
	var arrReturnElements	=	new Array();
	var oRegExp				=	new RegExp("(^|\\s)" + strClassName + "(\\s|$)");
	var oElement;

	for(var i = 0, l = arrElements.length; i < l; i++)
	{
		oElement	=	arrElements[i];
		if(oRegExp.test(oElement.className))
			arrReturnElements.push(oElement);
	}

	return (arrReturnElements)
}
Object.prototype.$$	=	$$;


/**
 * In array
 *  Hunts for a value in the specified array
 * @author Aaron Gustafson <aaron at easy-designs dot net>
 * @param string needle
 * @return bool
 */
function inArray(needle)
{
	for (var i=0; i < this.length; i++)
	{
		if (this[i] === needle)
		{
			return i;
		}
	}
	return false;
}
Array.prototype.inArray = inArray;

 /**
 * Each
 *
 * Applies a function to each element in an array
 * @param function func
 */
function each(func)
{
	for(var i = 0, l = this.length; i < l; i++)
	{
		func(this[i]);
	}
}
Array.prototype.each   =       each;
Object.prototype.each  =       each;

/**
 * Is array
 *
 * Verifies if something is an array
 * @author Aaron Gustafson <aaron at easy-designs dot net>
 * @return bool
 */
function isArray()
{
	return (typeof(this.length) == "undefined") ? false : true;
}
Array.prototype.isArray = isArray;


/**
 * Add class
 *
 * Appends the specified class to the object
 * @author Aaron Gustafson <aaron at easy-designs dot net>
 * @return void;
 */
function addClass(theClass)
{
	if (this.className != '')
	{
		if (!this.hasClass(theClass))
		{
			this.className += ' ' + theClass;
		}
	}
	else
	{
		this.className = theClass;
	}
}
Object.prototype.addClass = addClass;


/**
 * Remove class
 *
 * Removes the specified class to the object
 * @author Aaron Gustafson <aaron at easy-designs dot net>
 * @return void
 */
function removeClass(theClass) {
	var oldClass = this.className;
	var regExp = new RegExp('\\s?'+theClass+'\\b');

	if (oldClass.indexOf(theClass) != -1)
	{
		this.className = oldClass.replace(regExp,'');
	}
}
Object.prototype.removeClass	=	removeClass;

/**
 * Has class
 *
 * Checks if an object has a specific class
 * @param string class
 * @return bool
 */
function hasClass(strClass)
{
	if(strClass)
	{
		return (this.className.indexOf(strClass) != -1) ? true : false;
	}
	else
	{
		return (this.getAttributeNode("class") != null) ? true : false;
	}
}
Object.prototype.hasClass	=	hasClass;
 
/**
 * Toggle
 *
 * Toggles the visibility of an element
 */
function toggle()
{
	if(this.hasClass('hidden'))
	{
		this.removeClass('hidden');
	}
	else
	{
		this.addClass('hidden');
	}
}
Object.prototype.toggle	=	toggle;

/**
 * Read cookie
 * 
 * Reads a cookie and returns the content
 * @param string cookiename
 * @return string
 */
function readCookie(strCookie)
{
	// Cookies, yummy :D
	var cookies	=	document.cookie.split(';'), c, content = '';

	for(var i = 0, l = cookies.length; i < l; i++)
	{
		c	=	cookies[i];

		// Remove spaces
		while (c.charAt(0) == ' ')
		{
			c	=	c.substring(1, c.length);
		}

		// Is it the right cookie?
		if(c.indexOf(strCookie) == 0)
		{
			// Yes it was, put the content in a variable
			content	=	c.substring(strCookie.length + 1, c.length);
			break;
		}
	}
	return content;
}

/**
 * Write cookie
 *
 * @param string strCookie
 * @param string strContent
 */
function writeCookie(strCookie, strContent, intSeconds)
{
	var expire	=	'';
	if(intSeconds)
	{
		var date	=	new Date();
		date.setTime(date.getTime()+intSeconds);
		expire	=	'; expires=' + date.toGMTString();
	}

	document.cookie	=	strCookie + '=' + strContent + expire;
}


/**
 * Delete cookie
 *
 * @param string strCookie
 */
function deleteCookie(strCookie)
{
	// Set the cookie to expire one hour ago
	document.cookie	=	strCookie + '=; expires=-3600';
}


/**
 * Get server content
 *
 * Connects to strUrl with the parameters strParams and request method 
 * strMethod (POST|GET) and retrieves content given by server. 
 * Content is put in strTarget, which temporarily is filled with strWait 
 * while the content is retrieved.
 *
 * @param string strUrl
 * @param string strParams
 * @param string strMethod (POST|GET)
 * @param string strTarget
 * @param string strWait
 * @param string strError
 */
function getServerContent(strUrl, strParams, strMethod, strTarget, strWait, strError)
{
	var http;

	try
	{
		// Modern browsers, such as Opera, Firefox, Safari etc.
		http	=	new XMLHttpRequest();
	}
	catch (error)
	{
		try
		{
			// IE
			http	=	new ActiveXObject('Microsoft.XMLHTTP'); 
		}
		catch (error)
		{
			try
			{
				http	=	new ActiveXObject('Msxml2.XMLHTTP'); 
			}
			catch(error)
			{
				alert(error);
			}
		}
	}

	strMethod.toUpperCase();

	var objTarget	=	$(strTarget);

	var alertResponse	=	function()
	{
		/*
			http.readyState:
			0 = uninitialized
			1 = loading
			2 = loaded
			3 = interactive
			4 = complete
		*/
		// Check if the http request is complete
		if (http.readyState == 4)
		{
			if (http.status == 200 || http.status == 304)
			{
				// Add the response text to the document
				objTarget.innerHTML	=	http.responseText;
			}
			else
			{
				// Request failed, add error-class and error-message
				objTarget.addClass('error');
				objTarget.innerHTML	=	strError;
			}
		}
		else
		{
			// Request loading, display wait-message
			objTarget.innerHTML	=	strWait;
		}
	};

	// GET-request
	if (strMethod == 'GET')
	{
		http.onreadystatechange	=	alertResponse;
		http.open('GET', strUrl + strParams);
		http.send(null);
	}
	// POST-request
	else if(strMethod == 'POST')
	{
		http.onreadystatechange	=	alertResponse;
		http.open('POST', strUrl);
		http.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		http.send(strParams);
	}
	else
	{
		return false;
	}

	return true;
}


/**
 * Add DOM Load Event
 *
 * @author Jesse Skinner
 * @author Dean Edwards
 * @author Matthias Miller
 * @author John Resig
 *
 * Special thanks to Dan Webb's domready.js Prototype extension
 * and Simon Willison's addLoadEvent
 *
 * For more info, see:
 * @url http://www.thefutureoftheweb.com/blog/adddomloadevent
 *
 * @example Call addDOMLoadEvent one or more times with functions, ie:
 *
 *    function something() {
 *       // do something
 *    }
 *    addDOMLoadEvent(something);
 *
 *    addDOMLoadEvent(function() {
 *        // do other stuff
 *    });
 *
 */
addDOMLoadEvent = (function(){
    // create event function stack
    var load_events = [],
        load_timer,
        script,
        done,
        exec,
        old_onload,
        init = function () {
            done = true;

            // kill the timer
            clearInterval(load_timer);

            // execute each function in the stack in the order they were added
            while (exec = load_events.shift())
                exec();

            if (script) script.onreadystatechange = '';
        };

    return function (func) {
        // if the init function was already ran, just run this function now and stop
        if (done) return func();

        if (!load_events[0]) {
            // for Mozilla/Opera9
            if (document.addEventListener)
                document.addEventListener("DOMContentLoaded", init, false);

            // for Internet Explorer
            /*@cc_on @*/
            /*@if (@_win32)
                document.write("<script id=__ie_onload defer src=//0><\/scr"+"ipt>");
                script = document.getElementById("__ie_onload");
                script.onreadystatechange = function() {
                    if (this.readyState == "complete")
                        init(); // call the onload handler
                };
            /*@end @*/

            // for Safari
            if (/WebKit/i.test(navigator.userAgent)) { // sniff
                load_timer = setInterval(function() {
                    if (/loaded|complete/.test(document.readyState))
                        init(); // call the onload handler
                }, 10);
            }

            // for other browsers set the window.onload, but also execute the old window.onload
            old_onload = window.onload;
            window.onload = function() {
                init();
                if (old_onload) old_onload();
            };
        }

        load_events.push(func);
    }
})();


/**
 * Table stripes
 *
 * Adds nice stripes to tables
 * @param string strTable
 */
function tableStripes(strTable, evenClass, oddClass, even)
{
	var objTable, arrTableTr, arrRowTd;
	if (!evenClass)
		evenClass	=	'even';
	if (!oddClass)
		oddClass	=	'odd';

	if (strTable == '')
	{
		objTable	=	$(strTable);
		arrTableTr	=	objTable.getElementsByTagName('tr');
		arrTableTd	=	objTable.getElementsByTagName('td');
	}
	else
	{
		arrTableTr	=	document.getElementsByTagName('tr');
		arrTableTd	=	document.getElementsByTagName('td');
	}

	arrTableTr.each(function()
	{
		arrRowTd	=	arguments[0].getElementsByTagName('td');
		arrRowTd.each(function()
		{
			if (even)
			{
				arguments[0].addClass(oddClass);
			}
			else
			{
				arguments[0].addClass(evenClass);
			}
		});
		even	=	!even;
	});
}


/**
 * Multiquote
 *
 * Adds multiquote onclick to a-elements with the 'multiquote'-class
 */
function multiquote()
{
	var posts		=	$('thread');
	if(posts)
	{
		var objQuotes	=	posts.$$('a', 'multiquote');
		var q, strParentId;
		for(i = 0, l = objQuotes.length; i < l; i++)
		{
			objQuotes[i].onclick	=	function (ev)
			{
				intPost	=	ev.target.parentNode.parentNode.parentNode.id.replace('post-', '');

				var strCookie		=	'Vikingboard_multiquote';
				var strQuoted		=	readCookie(strCookie);
				var objPost			=	$('post-' + intPost);
				var objQuoteLink	=	objPost.$$('a', 'multiquote')[0];

				if (strQuoted.match('_' + intPost))
				{
					strQuoted	=	strQuoted.replace('_' + intPost, '');
					objQuoteLink.removeClass('format-quote-multi-remove');
					objQuoteLink.removeClass('quoted');
					objQuoteLink.addClass('format-quote-multi');
					objQuoteLink.getElementsByTagName('span')[0].innerHTML	=	'+';
				}
				else
				{
					strQuoted	+=	'_' + intPost;
					objQuoteLink.removeClass('format-quote-multi');
					objQuoteLink.addClass('quoted');
					objQuoteLink.addClass('format-quote-multi-remove');
					objQuoteLink.getElementsByTagName('span')[0].innerHTML	=	'-';
				}

				writeCookie(strCookie, strQuoted);

				return false;
			}
		}
	}
}


/**
 * Quick search
 * Adds default text to quick search field, and removes it onclick.
 */
function quick_search()
{
	var objQuick	=	$('global-search');

	// Check if the quick search field exists (does so only when the user is logged in)
	if(objQuick)
	{
		var objInput			=	objQuick.$('query');
		objInput.defaultValue	=	'Search ...';

		objInput.onclick	=	function ()
		{
			if (objInput.value == objInput.defaultValue)
			{
				objInput.value	=	'';
			}
		};

		objInput.onblur	=	function ()
		{
			if (objInput.value == '')
			{
				objInput.value	=	objInput.defaultValue;
			}
		};
	}
}

/**
 * Update post title
 * Changes the post page title to whatever the user types in the title field
 */
function update_post_title()
{
	var post	=	$('post');
	if (post) var objPostTitle	=	post.$('title');

	if (objPostTitle)
	{
		// Get the original page title for later use
		var objHeader	=	$('post-title');
		var strOldTitle	=	objHeader.innerHTML;

		// Everytime the user types something into the title field update page title
		objPostTitle.onkeyup	=	function()
		{
			var newTitle	=	this.value;

			// Check if the title is empty or not
			if (newTitle != '')
			{
				// Replace &, < and > with HTML entities
				newTitle	=	newTitle.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;');
			}
			else
			{
				// No title specified, change back to the original title
				newTitle	=	strOldTitle;
			}

			objHeader.innerHTML	=	newTitle;
		};
	}
}

/**
 * Quick post preview
 *
 * Adds a 'Preview' button below the quick post form.
 */
function quick_post_preview()
{
	var quick_reply_buttons	=	$('quick-reply-buttons');

	// Check if there is an element with the id 'quick-reply-buttons'
	if (quick_reply_buttons)
	{
		// Config vars, edit here
		var config	=	{
			post_button_text:'Post',
			preview_button_text:'Preview',
			edit_button_text:'Edit',
			quick_post_textarea:'quick-reply-post',
			parsing_post_text:'Parsing post, please wait ...',
			parsing_post_error:'Could not parse post. Please try again.'
		};
		// Stop edit

		// Creating the 'Preview'-button
		var preview_button		=	document.createElement('input');
		preview_button.type		=	'button';
		preview_button.value	=	config.preview_button_text;

		preview_button.onclick	=	function()
		{
			// Get the quick post textarea and content
			var quick_post_textarea	=	$(config.quick_post_textarea);
			var post_content		=	quick_post_textarea.value;

			var quick_post_form		=	quick_post_textarea.parentNode.parentNode;
			var quick_post			=	quick_post_form.parentNode;
			// Creating a div to place the 
			var parsed_post			=	document.createElement('div');
			parsed_post.id			=	'quick-reply-post-preview';

			// Hide the form
			quick_post_form.addClass('hidden');
			// Add newly created div for the parsed post
			quick_post.appendChild(parsed_post, quick_post_form);

			// Get the parsed post, place it in the 'parsed_post'-div
			getServerContent('post.php?quick_preview', 'post_content=' + post_content, 'POST', 'quick-reply-post-preview', config.parsing_post_text, config.parsing_post_error);

			// Creating a button for posting the previewed post content
			var post_preview	=	document.createElement('input');
			post_preview.type	=	'submit';
			post_preview.value	=	config.post_button_text;
			post_preview.onclick	=	function ()
			{
				// Submit the hidden form
				quick_post_form.submit();
			};

			// Creating a button for switching back to edit mode
			var edit_preview	=	document.createElement('input');
			edit_preview.type	=	'submit';
			edit_preview.value	=	config.edit_button_text;
			edit_preview.onclick	=	function()
			{
				// Remove the hidden class on the form
				quick_post_form.removeClass('hidden');
				// Remove the parsed post, post-button and edit-button
				quick_post.removeChild(parsed_post);
				quick_post.removeChild(post_preview);
				quick_post.removeChild(edit_preview);
			};

			// Adding the post- and edit-buttons
			quick_post.appendChild(post_preview);
			quick_post.appendChild(edit_preview);
		}

		// Add the preview button
		quick_reply_buttons.appendChild(preview_button);
	}

	return false;
}

function profile_settings_websites()
{
	var websites	=	$('websites');

	if (websites)
	{
	//	var template	=	websites.getElementsByTagName('fieldset')[0];
		var objNewWebsite	=	document.createElement('fieldset');
//		objNewWebsite.appendChild(o
		var foo	=	template;
		var objAddNewLink		=	document.createElement('a');
		objAddNewLink.innerHTML	=	'Add new';
		objAddNewLink.onclick	=	function()
		{
			websites.appendChild(template);
		};
		websites.appendChild(objAddNewLink);
	//	document.write(template.innerHTML);
	}

	return false;
}

addDOMLoadEvent(function()
{
	quick_search();
	multiquote();
	update_post_title();
	quick_post_preview();
//	profile_settings_websites();
});
