<?php

/**
 * Topic page, Handles topic stuff
 *
 * @package Vikingboard
 * @author Tor Henning Ueland <tor.henning@vikingboard.com>
 * @author Jørgen Tellnes <jorgis@jorgis.com>
 * @author Marius Pedersen Taule <marius@vikingboard.com>
 * @version 0.2
 *
 * $Id: topic.php 1859 2007-11-27 18:59:28Z jorgenboard $
 *
 */

//yes, we are here
define('VBOARD', __FILE__);

require_once('inc/kernel.php');
require_once('inc/lib/pages.php');

class Topic
{
	var $data;
	var $user;
	var $screen;
	var $config;
	var $lang;
	var $parser;

	var $topicID = 0;
	var $postID = 0;
	var $start_at_post = 0;


	/**
	 * Topic constructor, check`s input to figure out
	 * what should be done.
	 *
	 * Decides what should happen in topic.php
	 * @param int post
	 * @return void
	 */
	function Topic(&$data, &$user, &$screen, &$config, &$session)
	{
		//Set variables
		$this->user 	=	&$user;
		$this->data   	=	&$data;
		$this->config	=	&$config;
		$this->screen	=	&$screen;
		$this->session	=	&$session;
		$this->lang 	=	new lang_topic;
		$this->screen->set_lang('lang_topic');

		//Set topic ID
		$this->topicID = safe_user_int('t', 0);

		//Set post ID
		$this->postID = safe_user_int('p', 0);

		//Set page ID
		$this->start_at_post = safe_user_int('start', 0);

		// Choose what to do
		switch (true)
		{
			// Find the newest post
			case (isset($_GET['lastpost']) && safe_user_int('t', false)):
				$this->go_to_last_post();
			break;

			// Subscribe to topic
			case (isset($_GET['subscribe']) && safe_user_int('t', false)):
				$this->add_topic_subscription();
			break;

			case safe_user_int('post', false) :
				$this->go_to_post();
			break;

			// Show a topic
			default:
				$this->show_topic();
			break;
		}
	}


	/**
	 * Go to last post
	 *
	 * Finds the latest post in a topic and sends the user to it
	 * @param int topic
	 * @return void
	 */
	function go_to_last_post()
	{
		//Start by finding posts in this topic
		$last_post	=	$this->data->cachedQuery('get_last_post_data', array($this->topicID));
		$last_post	=	$last_post[0];
		$replies	=	$last_post->replies;
		$pages		=	ceil($replies / $this->config->show_maxposts);

		if ($replies > $this->config->show_maxposts)
		{
			$start_at	=	(($pages) * $this->config->show_maxposts);
			$start_at	=	"&start={$start_at}";
		}
		else
		{
			$start_at	=	false;
		}

		// Off we go :)
		redirect("topic.php?t={$this->topicID}{$start_at}#post-{$last_post->id}");
	}


	/**
	 * Add topic subscription
	 *
	 *
	 * @param int topic
	 * @return void
	 */
	function add_topic_subscription()
	{
		$row	=	$this->data->query('get_topic_subscription', array($this->user->id, $this->topicID));

		if (!$row[0]->id)
		{
			$this->data->query('new_topic_subscription', array($this->user->id, $this->topicID));
			notify($this->lang->subscription, $this->lang->new_topic_subscription);
		}
		else
		{
			notify($this->lang->subscription, $this->lang->existing_topic_subscription);
		}
	}


	/**
	 * Go to post
	 *
	 */
	function go_to_post()
	{
		$post	=	$_GET['post'];
		$topic	=	$this->data->query('get_topic_from_post', array($post));

		if (isset($post[0]))
		{
			$start	=	'';
			redirect("topic.php?t={$topic->id}&start={$start}#post-{$post}");
		}
		else
		{
			redirect('index.php');
		}
	}

	/**
	 * Show topic
	 *
	 * Gets posts from database, checks for user permissions, adds buttons (edit/delete/report/reply/multiquote) and so on
	 * @param int startPost
	 * @return void
	 */
	function show_topic()
	{
		// A little check to make sure that we got a topic to show
		if ($this->topicID < 1)
		{
			error_msg($this->lang->notopic, true);
		}

		// Starting at which post?
		$startPost	=	safe_pos_int($this->start_at_post, 0);

		// Updating the topic read count
		$this->data->query('update_topic_view_counter', array($this->topicID));

		// Getting the topic information
		$topic	=	$this->data->cachedQuery('get_topic_information', array($this->topicID));
		$topic	=	$topic[0];

		// Return a error message if the topic was not found in the database
		if (is_object($topic) == false)
		{
			error_msg($this->lang->notopic, 1);
		}

		// Get poll information if we need it
		if ($topic->has_poll == 1)
		{
			$poll	=	$this->data->query('get_poll', array($this->topicID));

			$variables['poll']		=	$poll;
			$variables['poll_name']	=	$poll[0]->name;
		}

		// Getting post ID`s to select in the next query
		$postIDs	=	$this->data->query('get_postIDs_in_topic', array($this->topicID, $startPost, $this->config->show_maxposts));
		$theIDs		=	'';
		foreach ($postIDs as $id)
		{
			$theIDs	.=	$id->id . ',';
		}

		// Updating forum read cookie (yummy!)
		setcookie($this->config->cookie . "_forumread[{$topic->fid}]", forumtime(), forumtime() + (86400 * 365), '/');

		// Getting the posts in this topic
		$posts	=	$this->data->query('get_posts_in_topic', array($theIDs, $this->config->show_maxposts));

		// Let`s navigate
		$current	=	ceil($startPost / $this->config->show_maxposts) + 1;
		$pages		=	ceil(($topic->replies + 1) / $this->config->show_maxposts);

		if ($pages > 1)
		{
			$topic_navigation	=	pages($current, $pages, "topic.php?t={$this->topicID}&amp;start=", $this->config->show_maxposts);
			$topic_navigation	=	str_replace('%pages%', $topic_navigation, $this->lang->several_pages);
		}
		else
		{
			$topic_navigation	=	false;
		}

		$multiquote	=	ifsetor($_COOKIE["{$this->config->cookie}_multiquote"], '');

		$topic_posts	=	'';
		$post_num		=	$startPost + 1;
		foreach ($posts as $p)
		{
			// Add a notice if the post has been edited
			if ($p->times_edited > 0)
			{
				$edited_by	=	$this->data->cachedQuery('get_username_from_id', array($p->last_edit_by));
				$edited_by	=	$edited_by[0];

				$edited	=	$this->lang->edited_post;

				$edited	=	str_replace('%num%', $p->times_edited, $edited);
				$edited	=	str_replace('%last_edit_by%', $edited_by->name, $edited);
				$edited	=	str_replace('%last_edit%', date($this->config->timestamp1, forumtime($p->last_edit)), $edited);
			}
			else
			{
				$edited_by	=	false;
				$edited		=	false;
			}

			// Give the user a short description
			if ($p->author_id == 1)
			{
				// It's a guest, be aware!
				$posted_by			=	$this->lang->posted_by_guest;
				$user_description	=	$this->lang->user_description_guest;
			}
			else
			{
				// Get the user details
				$user	=	$this->data->cachedQuery('get_author_details', array($p->author_id));
				$user	=	$user[0];

				$posted_by	=	str_replace('%author_id%', $p->author_id, $this->lang->posted_by);
				$posted_by	=	str_replace('%author_name%', $user->name, $posted_by);

				// If the user lives somewhere, share it with the world
				if ($user->location)
				{
					$user_description	=	$this->lang->user_description;
					$user_description	=	str_replace('%location%', $user->location, $user_description);
				}
				else
				{
					$user_description	=	$this->lang->user_description_no_location;
				}

				// And in with the other vars
				if (!empty($user->title))
				{
					$user_description = str_replace('%title%', $user->title, $user_description);
				}
				else
				{
					/** This is a hack. We should use a seperate group name for display here. @todo for 0.2.1 or 0.3.x */
					$user_description = str_replace('%title%', strtolower($user->group_name), $user_description);
				}

				$user_description	=	str_replace('%joined%', date($this->config->timestamp2, forumtime($user->registered)), $user_description);
				$user_description	=	str_replace('%total_posts%', $user->posts, $user_description);
			}

			// Is the post multiquoted?
			$multiquoted	=	(strstr($multiquote, "_{$p->id}") ? true : false);

			$pm			=	($this->user->id == 1 || $p->author_id == 1) ? false : true;
			$can_post	=	($this->user->g_create_new_posts) ? true : false;
			$can_edit	=	(($p->author_id == $this->user->id && $this->user->g_can_edit_own_posts) || ($this->user->g_is_mod || $this->user->g_is_admin)) ? true : false;
			$can_delete	=	(($p->author_id == $this->user->id && $this->user->g_can_delete_own_posts) || ($this->user->g_is_mod || $this->user->g_is_admin)) ? true : false;
			$can_report	=	($this->user->id == 1) ? false : true;
			$actions	=	($can_post || $can_edit || $can_delete || $can_report) ? true : false;

			$post	=	array
			(
				'id'					=>	$p->id,
				'tid'					=>	$this->topicID,
				'post_num'				=>	$post_num,
				'author_id'				=>	$p->author_id,
//				'author_name'			=>	$author_name,
				'posted_by'				=>	$posted_by,
				'poster_was_starter'	=>	(($p->author_id == $topic->starter_id) ? true : false),
				'pm'					=>	$pm,
				'time'					=>	date($this->config->timestamp1, forumtime($p->posted)),
				'avatar'				=>	$p->avatar,
				'user_not_guest'		=>	(($p->author_id != 1) ? true : false),
				'user_description'		=>	$user_description,
				'content'				=>	$p->post,
				'edited'				=>	((isset($edited)) ? $edited : false),
				'signature'				=>	(($p->signature) ? $p->signature : false),
				'multiquoted'			=>	$multiquoted,
				'actions'				=>	$actions,
				'can_post'				=>	$can_post,
				'can_edit'				=>	$can_edit,
				'can_delete'			=>	$can_delete,
				'can_report'			=>	$can_report,
			);

			$topic_posts	.=	$this->screen->return_html('single_post', $post);

			$post_num++;
		}

		// Give the user a short description
		if ($this->user->id == 1)
		{
			// It's a guest, be aware!
			$posted_by			=	$this->lang->quick_post_by_guest;
			$user_description	=	$this->lang->user_description_posting_as_guest;
		}
		else
		{
			$posted_by	=	str_replace('%author_id%', $this->user->id, $this->lang->quick_post_by);
			$posted_by	=	str_replace('%author_name%', $this->user->name, $posted_by);

			// If the user lives somewhere, share it with the world
			if (isset($this->user->location))
			{
				$user_description	=	$this->lang->user_description;
				$user_description	=	str_replace('%location%', $this->user->location, $user_description);
			}
			else
			{
				$user_description	=	$this->lang->user_description_no_location;
			}

			/**
			 * @todo Seems foolish to have duplicate code for displaying a single post
			 * because we need a single post without post buttons and a textarea
			 * instead of the post, when we could simplify it by outsourcing it
			 * into a single_post($show_postbuttons, $post_contents) thingy or whatever.
			 *
			 * Code reuse is NOT abuse!
			 */

			// And in with the other vars
			if (!empty($this->user->title))
			{
				$user_description = str_replace('%title%', $this->user->title, $user_description);
			}
			else
			{
				/** This is a hack. We should use a seperate group name for display here. @todo for 0.2.1 or 0.3.x */
				$user_description = str_replace('%title%', strtolower($this->user->g_name), $user_description);
			}

			$user_description	=	str_replace('%joined%', date($this->config->timestamp2, forumtime($this->user->registered)), $user_description);
			$user_description	=	str_replace('%total_posts%', $this->user->posts, $user_description);
		}

		$post_a_reply	=	str_replace('%topic_id%', $topic->id, $this->lang->post_a_reply);

		$data	=	array
		(
			'id'				=>	$topic->id,
			'title'				=>	$topic->title,
			'logged_in'			=>	(($this->user->id == 1) ? false : true),
			'posts'				=>	$topic_posts,
			'forum_id'			=>	$topic->fid,
			'category_id'		=>	$topic->catid,
			'poll'				=>	(($topic->has_poll) ? true : false),
			'poll_results'		=>	(($topic->has_poll && isset($_GET['poll_results'])) ? true : false),
			'topic_navigation'	=>	$topic_navigation,
			'can_post'			=>	($this->user->g_create_new_posts) ? true : false,
			'posted_by'			=>  $posted_by,
			'post_num'			=>	$topic->replies + 2,
			'user_avatar'		=>	$this->user->avatar,
			'user_details'		=>	$user_description,
			'current_time'		=>	date($this->config->timestamp1, forumtime(time())),
			'sid'				=>	$this->session->id(),
			'post_a_reply'		=>	$post_a_reply,
			'has_power'			=>	$this->user->g_is_mod || $this->user->g_is_admin,
			'is_closed'			=> 	($topic->status ? 0 : 1),
			'is_unsticky'		=> 	($topic->sticky ? 0 : 1)
		);

		$navigation	=	array
		(
			"index.php?c={$topic->catid}"	=>	$topic->catname,
			"forum.php?f={$topic->fid}"		=>	$topic->fname,
			"topic.php?t={$this->topicID}"	=>	$topic->title,
		);

		$this->screen->add_html('topic', $data);
		$this->screen->add_wrapper($topic->title, $navigation);
		$this->screen->print_page();
	}
}

// Start :)
$Topic = new Topic($data, $user, $screen, $config, $session);

?>